#ifndef annot_h
#define annot_h

enum ANNOT_TYPE
	{
#define A(a)	ANNOT_TYPE_##a,
#include "annots.h"
	};

const char *AnnotTypeToStr(ANNOT_TYPE Type);

#pragma pack(push,1)
struct AnnotData
	{
	uint16 GeneIndex;
	uint8 CEC;
	ANNOT_TYPE Type;
	gpos_t Pos;
	gpos_t Length;
	uint8 GeneArrayIndex;

	bool Strand;
	uint8 Frame; // 0, 1 or 2
	uint8 ExonIndex;
	uint8 ExonCount;
	uint8 MicrosatMotifLength;

	uint8 GetTandemMotifLength() const;
	bool SplitOnInsert(gpos_t Length) const;
	bool DeconstrainOnInsert(gpos_t Length) const;
	gpos_t MinLength() const;

	AnnotData()
		{
		GeneIndex = UINT16_MAX;
		Type = ANNOT_TYPE_Invalid;
		Pos = GPOS_T_MAX;
		Length = GPOS_T_MAX;
		Strand = false;
		Frame = SCHAR_MAX;
		ExonIndex = UCHAR_MAX;
		ExonCount = UCHAR_MAX;
		GeneArrayIndex = UINT8_MAX;
		MicrosatMotifLength = UCHAR_MAX;
		}

	void LogMe() const
		{
		Log("%s CEC=%u Pos=%"PF" Len=%"PF,
		  TypeStr(),
		  CEC,
		  Pos,
		  Length);
		if (GeneIndex == UINT16_MAX)
			Log(" Gene=*");
		else
			Log(" Gene=%u", GeneIndex);
		if (IsCoding())
			Log(" Ex=%u Ec=%u St=%c Frame=%d",
			  ExonIndex, ExonCount, pom(Strand), Frame);
		}

	bool IsCoding() const
		{
		return Type == ANNOT_TYPE_CDS;
		}

	bool IsStartExon() const
		{
		return IsCoding() && ExonIndex == 0;
		}

	bool IsEndExon() const
		{
		return IsCoding() && ExonIndex + 1 == ExonCount;
		}

	gpos_t GetEndPos() const
		{
		return Pos + Length - 1;
		}

	const char *TypeStr() const
		{
		return AnnotTypeToStr(Type);
		}
	
	bool operator<(const AnnotData &rhs) const
		{
	// Assumes no two AnnotData's with same Pos
		return Pos < rhs.Pos;
		}

	bool operator==(const AnnotData &rhs) const
		{
	// Assumes no two AnnotData's with same Pos
		return Pos == rhs.Pos;
		}
	};
#pragma pack(pop)

uint8 IncFrame(uint8 Frame, gpos_t Length);

#endif // annot_t
