#include "utils.h"
#include "cmafile.h"
#include "revfile.h"
#include "seqfile.h"
#include "getopts.h"

static void GetChrNameIndex(const RevFile &Rev, unsigned GenomeIndex,
  ChrNameIndex &CNI)
	{
	const GenInfo &GI = Rev.GetGenomeInfo(GenomeIndex);
	unsigned ChrCount = GI.GetChrCount();
	for (unsigned ChrIndex = 0; ChrIndex < ChrCount; ++ChrIndex)
		{
		const ChrInfo &CI = GI.GetChrInfoByIndex(ChrIndex);
		unsigned ChrId = CI.GetId();
		const std::string &ChrName = CI.GetName();
		CNI.Add(ChrId, ChrName.c_str());
		}
	}

static void Add(const RevFile &Rev, unsigned AlnIndex, CMAFile &CMA)
	{
	StdStrVec ssRows;
	unsigned AlnInfoId = Rev.GetAlnRows(AlnIndex, ssRows);

	AlnInfo AI;
	unsigned AlnIndex2 = Rev.GetAlnInfo(AlnInfoId, AI);

	if (AlnIndex != AlnIndex2)
		Die("Add assert failed");

	StrVec Rows;
	CoordVec Coords;

	const unsigned GenomeCount = (unsigned) AI.Segs.size();
	for (unsigned GenomeIndex = 0; GenomeIndex < GenomeCount; ++GenomeIndex)
		{
		const GCSeg &S = AI.Segs[GenomeIndex];

		const char *Row = ssRows[GenomeIndex].c_str();
		Rows.push_back(Row);

		CoordData CD;
		CD.StartPos = S.lo;
		CD.EndPos = S.hi;
		CD.ChrIndex = S.ChrId;
		CD.Reversed = !S.Plus;

		Coords.push_back(CD);
		}

	CMA.AppendCMA(Coords, Rows);
	}

void CvtRev2CMA()
	{
	const char *RevFileName = RequiredValueOpt("fromrev");
	const char *CMAFileName = RequiredValueOpt("tocma");

	RevFile Rev;
	Rev.Open(RevFileName);

	const std::string &GenomeNameA = Rev.GetGenomeName(0);
	const std::string &GenomeNameB = Rev.GetGenomeName(1);

	StdStrVec GenomeNames;
	GenomeNames.push_back(GenomeNameA);
	GenomeNames.push_back(GenomeNameB);

	ChrNameIndex CNIA;
	ChrNameIndex CNIB;
	
	GetChrNameIndex(Rev, 0, CNIA);
	GetChrNameIndex(Rev, 1, CNIB);

	ChrNameIndexVec ChrNameIndexes;
	ChrNameIndexes.push_back(&CNIA);
	ChrNameIndexes.push_back(&CNIB);

	CMAFile CMA;
	CMA.Create(CMAFileName, GenomeNames, ChrNameIndexes);

	const unsigned AlnCount = Rev.GetAlnCount();
	for (unsigned AlnIndex = 0; AlnIndex < AlnCount; ++AlnIndex)
		Add(Rev, AlnIndex, CMA);

	Rev.Close();
	CMA.Close();
	}
