#include "evo.h"

static gpos_t g_AccumLength;

static void DumpIE(unsigned MULIndex, const MULIndexEntryData &IE)
	{
	Dump("%5u", MULIndex);
	if (IE.FileBytePos == OFF64_T_MAX)
		Dump("             *");
	else
		Dump("  %12"PF, IE.FileBytePos);

	if (IE.FileBitPos == UINT_MAX)
		Dump("    *");
	else
		Dump("  %3u", IE.FileBitPos);

	Dump("  %4c", tofi(IE.Leaf));

	if (IE.InMULIndex == UINT_MAX)
		Dump("           *");
	else
		Dump("  %10u", IE.InMULIndex);

	if (IE.MI.ChrIndex == UINT_MAX)
		Dump("         *");
	else
		Dump("  %8u", IE.MI.ChrIndex);

	if (IE.MI.MinPos == GPOS_T_MAX)
		Dump("             *");
	else
		Dump("  %10"PF, IE.MI.MinPos);

	if (IE.MI.AncestralLength == GPOS_T_MAX)
		Dump("             *");
	else
		Dump("  %10"PF, IE.MI.AncestralLength);

	if (IE.MI.EvolvedLength == GPOS_T_MAX)
		Dump("             *");
	else
		Dump("  %10"PF, IE.MI.EvolvedLength);

	if (IE.MI.MaxIndex == GPOS_T_MAX)
		Dump("             *");
	else
		Dump("  %10"PF, IE.MI.MaxIndex);

	Dump("  %10"PF, g_AccumLength);

	Dump("  %c", pomi(IE.MI.Plus));

	if (IE.Next == UINT_MAX)
		Dump("       *");
	else
		Dump("  %6u", IE.Next);
	if (IE.Magic != MULIEMagic)
		Log(" Magic is 0x%x should be 0x%x", IE.Magic, MULIEMagic);
	Dump("\n");
	}

static void DumpIEHdr(const char *AccumLenHdr = "AccumLen")
	{
	Dump("\n");
	Dump(
"Index       BytePos  Bit  Leaf  InMULIndex  ChrIndex      MinPos      AncLen  EvolvedLen    MaxIndex  %10.10s  +    Next\n", AccumLenHdr);
	Dump(
"=====  ============  ===  ====  ==========  ========  ==========  ==========  ==========  ==========  ==========  =  ======\n");
	}

void DumpEvoIndex(FILE *f, unsigned MULCount)
	{
	EvoFileHeader Hdr;
	vector<MULIndexEntryData> IEs;
	GenInfo Genome;
	ReadEvoIndex(f, Hdr, IEs, Genome, false);

	Dump("\n");
	Dump("Genome:\n");
	Genome.LogMe();

	Dump("\n");
	Dump("Index order:\n");
	DumpIEHdr();
	for (unsigned MULIndex = 0; MULIndex < MULCount; ++MULIndex)
		{
		const MULIndexEntryData &IE = IEs[MULIndex];
		DumpIE(MULIndex, IE);
		g_AccumLength += IE.MI.EvolvedLength;
		}
	g_AccumLength = 0;
	Dump("\n");
	Dump("Evolved order:\n");
	DumpIEHdr("ChrPos");
	unsigned MULIndex = Hdr.FirstMULIndex;
	unsigned PrevChrIndex = UINT_MAX;
	while (MULIndex != UINT_MAX)
		{
		asserta(MULIndex < SIZE(IEs));
		const MULIndexEntryData &IE = IEs[MULIndex];
		if (IE.MI.ChrIndex != PrevChrIndex)
			{
			g_AccumLength = 0;
			PrevChrIndex = IE.MI.ChrIndex;
			}
		DumpIE(MULIndex, IE);
		MULIndex = IE.Next;
		g_AccumLength += IE.MI.EvolvedLength;
		}
	Dump("\n");
	}

void DumpEvoHdr(FILE *f)
	{
	EvoFileHeader Hdr;
	ReadEvoHdr(f, Hdr);

	off64_t DataStartPos = (off64_t)
	  (sizeof(EvoFileHeader) + sizeof(MULIndexEntryData));
	Dump("File size       %12"PF"\n", (uint64) GetStdioFileSize(f));
	Dump("sizeof(IE)      %12u\n", Hdr.SizeofIE);
	Dump("TotalBaseCount  %12"PF"\n", Hdr.TotalBaseCount);
	Dump("LeafBaseCount   %12"PF"\n", Hdr.LeafBaseCount);
	Dump("FirstMUL        %12u\n", Hdr.FirstMULIndex);
	Dump("TotalMULCount   %12u\n", Hdr.TotalMULCount);
	Dump("LeafMULCount    %12u\n", Hdr.LeafMULCount);
	Dump("Data start      %12"PF"\n", DataStartPos);

	DumpEvoIndex(f, Hdr.TotalMULCount);
	}

void DumpEvoHdr()
	{
	const char *FileName = RequiredValueOpt("dumpevohdr");
	FILE *f = OpenStdioFile(FileName);
	DumpEvoHdr(f);

	EvoFileHeader Hdr;
	ReadEvoHdr(f, Hdr);
	const unsigned MULCount = Hdr.TotalMULCount;
	BoolVec InList(MULCount, false);
	unsigned MULIndex = Hdr.FirstMULIndex;
	unsigned Counter = 0;
	for (;;)
		{
		if (MULIndex == UINT_MAX)
			break;
		asserta(MULIndex < MULCount);
		asserta(Counter++ < MULCount);
		InList[MULIndex] = true;
		MULIndexEntryData IE;
		ReadIE(f, MULIndex, IE);
		MULIndex = IE.Next;
		}

	gpos_t BaseCount = 0;
	gpos_t ActiveBaseCount = 0;
	FORIX(MUL)
		{
		MULIndexEntryData IE;
		ReadIE(f, MULIndex, IE);
		BaseCount += IE.MI.EvolvedLength;
		if (InList[MULIndex])
			ActiveBaseCount += IE.MI.EvolvedLength;
		}

	unsigned LinkedOutCount = 0;
	FORIX(MUL)
		{
		if (!InList[MULIndex])
			{
			if (LinkedOutCount == 0)
				{
				Log("\n");
				Log("Linked out: ");
				}
			if (++LinkedOutCount%16 == 0)
				Log("\n");
			Log(" %u", MULIndex);
			}
		}
	if (LinkedOutCount > 0)
		Log("\n");
	else
		Log("None linked out.\n");

	Log("\n");
	Log("Inactive MULs  %12u\n", LinkedOutCount);
	Log("Total bases    %12"PF"\n", BaseCount);
	Log("Active bases   %12"PF"\n", ActiveBaseCount);
	Log("Inactive bases %12"PF"\n", BaseCount - ActiveBaseCount);
	}
