#include "evo.h"

FILE *g_fEvo;
FILE *g_fOut;

static gpos_t g_ReadBaseCount = 0;
static gpos_t g_ColIndex = 0;
static gpos_t g_TotalBaseCount = GPOS_T_MAX;
static bool g_MULPlus = true;
static list<char> g_MULSeq;

static void FlushSeq()
	{
	for (list<char>::const_iterator p = g_MULSeq.begin();
	  p != g_MULSeq.end(); ++p)
		{
		char Letter = *p;
		if (g_ColIndex%64 == 0 && g_ColIndex != 0)
			fprintf(g_fOut, "\n");
		fputc(Letter, g_fOut);
		++g_ColIndex;
		}
	g_MULSeq.clear();
	}

void GotBaseCallback(const BaseInfo &Base)
	{
	if (++g_ReadBaseCount%500000 == 0)
		{
		unsigned uTotalBaseCount = (unsigned) (g_TotalBaseCount/100);
		unsigned uBaseCount = (unsigned) (g_ReadBaseCount/100);
		ProgressStep(uBaseCount, uTotalBaseCount);
		}

	char Letter = Base.GetOrientedLetter();
	if (g_MULPlus)
		g_MULSeq.push_back(Letter);
	else
		g_MULSeq.push_front(CompLetter(Letter));
	}

void EvoToFASTA()
	{
	const char *EvoFileName = RequiredValueOpt("evo2fasta");
	const char *FASTAFileName = RequiredValueOpt("out");
	DoEvoToFASTA(EvoFileName, FASTAFileName);
	}

void DoEvoToFASTA(const char *EvoFileName, const char *FASTAFileName,
  bool ValidateConsecutive)
	{
	bool Dump = FlagOpt("dump");
	bool DumpHeaders = FlagOpt("dumphdrs");
	bool Internal = FlagOpt("all");
	bool MULs = FlagOpt("mulseqs");

	g_fEvo = OpenStdioFile(EvoFileName);
	g_fOut = CreateStdioFile(FASTAFileName);

	EvoFileHeader EvoHdr;
	vector<MULIndexEntryData> IEs;
	GenInfo Genome;
	ReadEvoIndex(g_fEvo, EvoHdr, IEs, Genome);
	if (Internal)
		g_TotalBaseCount = EvoHdr.TotalBaseCount;
	else
		g_TotalBaseCount = EvoHdr.LeafBaseCount;

	const unsigned MULCount = Internal ?
	  EvoHdr.TotalMULCount : EvoHdr.LeafMULCount;

	Progress("%u MULs", MULCount);
	ProgressStart("Converting");
	gpos_t TotalReadBaseCount = 0;
	unsigned PrevChrIndex = UINT_MAX;
	unsigned MULIndex = EvoHdr.FirstMULIndex;
	for (;;)
		{
		if (MULIndex == UINT_MAX)
			break;

		MicroFromFileOpts Opts;
		Opts.GotBaseCallback = GotBaseCallback;
		Opts.Dump = Dump;
		Opts.DumpHeaders = DumpHeaders;
		Opts.ScanOnly = true;
		Opts.ValidateConsecutive = ValidateConsecutive;

		MULIndexEntryData IE;
		ReadIE(g_fEvo, MULIndex, IE);

		Log("MUL Index %u\n", MULIndex);
		Log("IE=");
		IE.LogMe();
		Log("\n");

		g_MULPlus = IE.MI.GetPlus();

		MicroEvolver M;
		gpos_t ReadBaseCount = M.FromFile(g_fEvo, IE, &Opts);

		if (MULs)
			{
			if (g_ColIndex != 0)
				{
				fprintf(g_fOut, "\n");
				g_ColIndex = 0;
				}
			fprintf(g_fOut, ">MUL%u%c\n", MULIndex, pom(IE.MI.Plus));
			}
		else if (IE.MI.ChrIndex != PrevChrIndex)
			{
			if (g_ColIndex != 0)
				{
				fprintf(g_fOut, "\n");
				g_ColIndex = 0;
				}
			const ChrInfo &CI = Genome.GetChrInfo(IE.MI.ChrIndex);
			fprintf(g_fOut, ">%s\n", CI.GetName().c_str());
			PrevChrIndex = IE.MI.ChrIndex;
			}
		FlushSeq();

		asserta(IE.MI.EvolvedLength == ReadBaseCount);
		Log("MUL %u size %"PF"\n", MULIndex, ReadBaseCount);
		TotalReadBaseCount += ReadBaseCount;

		MULIndex = IE.Next;
		}

	FlushSeq();
	fprintf(g_fOut, "\n");

	ProgressDone();
	Progress("%"PF" bases read", g_ReadBaseCount);

	if (TotalReadBaseCount != g_ReadBaseCount)
		Die("Total %"PF", g_Total %"PF,
		  TotalReadBaseCount, g_ReadBaseCount);

	if (g_ReadBaseCount != g_TotalBaseCount)
		Die("Read %"PF" bases, total %"PF,
		  g_ReadBaseCount, g_TotalBaseCount);

	CloseStdioFile(g_fEvo);
	CloseStdioFile(g_fOut);
	}
