#include "evolver.h"

#define TRACE	0

bool IsStartCodon(const string &Codon);
bool IsStopCodon(const string &Codon);

void Evolver::SetIntrons(vector<gpos_t> &GeneStartPositions,
  vector<gpos_t> &GeneStopPositions)
	{
	const unsigned N = SIZE(GeneStartPositions);
	asserta(N == SIZE(GeneStopPositions));
	asserta(N < UINT16_MAX);
#if	TRACE
	Log("\n");
	Log("Gene Index       Start        Stop    CE Bases  NeutlBases       Total\n");
	Log("==========  ==========  ==========  ==========  ==========  ==========\n");
#endif
	for (uint16 GeneIndex = 0; GeneIndex < N; ++GeneIndex)
		{
		gpos_t StartPos = GeneStartPositions[GeneIndex];
		gpos_t StopPos = GeneStopPositions[GeneIndex];
		if (StartPos == GPOS_T_MAX)
			{
			asserta(StopPos == GPOS_T_MAX);
			continue;
			}
		SetIntronsOneGene(GeneIndex, StartPos, StopPos);
		}
	Log("\n");
	}

void Evolver::SetIntronsOneGene(uint16 GeneIndex, gpos_t StartPos,
  gpos_t StopPos)
	{
	CodonInfo StartCodon;
	CodonInfo StopCodon;

	GetCodon(StartPos, StartCodon);
	GetCodon(StopPos, StopCodon);

	if (!IsStartCodon(StartCodon.Letters))
		Warning("Gene %u unexpected start codon %c%c%c",
		  (unsigned) GeneIndex,
		  StartCodon.Letters[0],
		  StartCodon.Letters[1],
		  StartCodon.Letters[2]);

	if (!IsStopCodon(StopCodon.Letters))
		Warning("Gene %u unexpected stop codon %c%c%c",
		  (unsigned) GeneIndex,
		  StopCodon.Letters[0],
		  StopCodon.Letters[1],
		  StopCodon.Letters[2]);

	gpos_t MinPos = min(StartPos, StopPos);
	gpos_t MaxPos = max(StartPos, StopPos);

	unsigned CEBaseCount = 0;
	unsigned UCBaseCount = 0;
	bool CEWarning = false;
	for (unsigned Pos = MinPos; Pos <= MaxPos; ++Pos)
		{
		BaseInfo &Base = GetBase(Pos);
		if (Base.IsCoding() || Base.IsNCCE())
			{
			++CEBaseCount;
			if (Base.IsNCCE() && Base.GeneIndex == UINT16_MAX)
				{
				if (!CEWarning)
					{
					Warning("NCCE in gene %u missing gene_index",
					  (unsigned) GeneIndex);
					CEWarning = true;
					}

				Base.GeneIndex = GeneIndex;
				}
			if (Base.GeneIndex != GeneIndex)
				Die("Gene index mismatch %u,%u pos %"PF,
				  (unsigned) Base.GeneIndex,
				  (unsigned) GeneIndex,
				  Pos);
			}
		else
			{
			asserta(Base.IsUnconstrained());
			++UCBaseCount;
			Base.GeneIndex = GeneIndex;
			}
		}
#if	TRACE
	Log("%10u  %10"PF"  %10"PF"  %10u  %10u  %10u\n",
	  (unsigned) GeneIndex,
	  StartPos,
	  StopPos,
	  CEBaseCount,
	  UCBaseCount,
	  CEBaseCount + UCBaseCount);
#endif
	}
