#include "evo.h"

bool Evolver::AcceptRangeMutation(MICRO_MUT_TYPE MutationType, gpos_t Pos_,
  gpos_t Length) const
	{
	gpos_t Pos = Pos_;
	bool AnyBaseIsExonic = false;
	bool AllBasesAreExonic = (Length > 0);
	for (gpos_t i = 1; ; ++i)
		{
		const BaseInfo &Base = GetBase(Pos);
		if (Base.IsCoding())
			AnyBaseIsExonic = true;
		else
			AllBasesAreExonic = false;
		double Prob = P_AcceptProbs[Base.ProbIndex];
		bool Accept = RandTrue(Prob);
		if (!Accept)
			return false;

		if (i >= Length)
			break;
		Pos = GetNextPos(Pos);
		}

	if (AnyBaseIsExonic)
		{
	// Length-changing mutation?
		if ((MutationType == MICRO_MUT_TYPE_Delete ||
		  MutationType == MICRO_MUT_TYPE_Insert))
			{
		// Punt if not all exon, rare, will probably reject anyway,
		// & hard to figure out correctly.
			if (!AllBasesAreExonic)
				return false;
		
		// Frame shifting strictly forbidden
			if (Length%3 != 0)
				return false;
			}
		}
	return true;
	}

bool Evolver::AcceptInsert(gpos_t Pos, gpos_t Length) const
	{
	gpos_t NextPos = GetNextPos(Pos);
	if (NextPos == GPOS_T_MAX)
		return false;

	const BaseInfo &Base = GetBase(Pos);
	const BaseInfo &NextBase = GetBase(NextPos);

// Frame shifting strictly forbidden
	if ((Base.IsCoding() || NextBase.IsCoding()) && Length%3 != 0)
		return false;

// Otherwise, compute minimum of probs from the two bases
// immediately before and after the insert.
	double GetInsertAcceptProb(uint8 ProbIndex, gpos_t Length);
	double Prob1 = GetInsertAcceptProb(Base.ProbIndex, Length);
	double Prob2 = GetInsertAcceptProb(NextBase.ProbIndex, Length);
	double Prob = min(Prob1, Prob2);
	bool Accept = RandTrue(Prob);
	return Accept;
	}
