#ifndef randdist_h
#define randdist_h

#include <vector>
#include "myrand.h"

using namespace std;

struct ProbDensFunc
	{
	virtual double Func(double x) const = 0;
	};

// Generate random numbers from a probability density function
// provided via object member function.
class RandDist
	{
public:
	RandDist();
	void Init(const ProbDensFunc &PDF, double Min_x, double Max_x,
	  unsigned BinCount = 1000);
	double Get() const;
	void Test(unsigned SampleCount, unsigned SampleBinCount) const;

private:
	double m_Min_x;
	double m_Max_x;
	vector<unsigned> m_ThreshVec;
	};

double Normal(double x, double Mean, double StdDev);

class NormalPDF : public ProbDensFunc
	{
public:
	NormalPDF(double Mean, double StdDev) : m_Mean(Mean), m_StdDev(StdDev)
		{
		}
private:
// Disable default c'tor
	NormalPDF();

public:
	virtual double Func(double x) const
		{
		return Normal(x, m_Mean, m_StdDev);
		}

private:
	double m_Mean;
	double m_StdDev;
	};

struct RDParams
	{
	double Weight;
	double Mean;
	double Shape;
	};

enum CE_AND_SPEED
	{
	CDS_Slow,
	CDS_Medium,
	CDS_Fast,
	NCCE_Slow,
	NCCE_Medium,
	NCCE_Fast
	};

#endif // randdist_h
