#include "evo.h"

static unsigned Counts[UCHAR_MAX];
static FILE *g_fOut;

static void OnInvalidLetter(unsigned char c)
	{
	++(Counts[c]);
	}

static void OnSeq(const string &Label, const string &Seq_)
	{
	string Seq(Seq_);
	for (string::iterator p = Seq.begin(); p != Seq.end(); ++p)
		{
		char c = *p;
		if (!isacgt(c))
			{
			OnInvalidLetter(c);
			*p = RandLetter();
			}
		}
	fprintf(g_fOut, ">%s\n", Label.c_str());
	WriteFasta(g_fOut, Seq.c_str(), SIZE(Seq));
	}

static void ReportCounts()
	{
	unsigned Total = 0;
	for (unsigned i = 0; i <= UCHAR_MAX; ++i)
		{
		if (Counts[i] == 0)
			continue;
		if (Total == 0)
			{
			Dump("\n");
			Dump("Letter      Number\n");
			Dump("======  ==========\n");
			}
		if (isprint(i))
			Dump("%6c", (char) i);
		else
			Dump("    %02x", i);
		Dump("  %10u", Counts[i]);
		Dump("\n");
		Total += Counts[i];
		}
	Dump("\n");
	Dump("Total of %u invalid letters replaced\n", Total);
	}

// Randomize Ns (and other invalid characters) in FASTA input
void RandomizeNs()
	{
	const char *InFileName = RequiredValueOpt("randns");
	const char *OutFileName = RequiredValueOpt("out");

	g_fOut = CreateStdioFile(OutFileName);

	memset(Counts, 0, sizeof(Counts));
	ReadFasta(InFileName, OnSeq);
	ReportCounts();

	CloseStdioFile(g_fOut);
	}
