#include "utils.h"
#include <string>

using namespace std;

void DeleteWhiteSpace(const string &In, string &Out);
bool ReadEvoParamsLine(FILE *f, string &Line);
void ParseStrVec(const std::string &In, std::vector<std::string> &Out, char Sep);

extern double P_AcceptProbs[256];

void ReadAcceptProbs(FILE *f)
	{
	unsigned ProbIndex = 0;
	for (;;)
		{
		string Line;
		bool EndOfFile = ReadEvoParamsLine(f, Line);
		if (EndOfFile)
			Die("Premature EOF in distribution, missing '//'");

		if (Line.empty())
			continue;

		if (Line == "//")
			break;

		string sLine;
		DeleteWhiteSpace(Line, sLine);

		vector<string> Values;
		ParseStrVec(sLine, Values, ';');

		for (vector<string>::const_iterator p = Values.begin();
		  p != Values.end(); ++p)
			{
			if (ProbIndex >= 256)
				Die("Too many accept probs, must be exactly 256");

			const string &Value = *p;
			double Prob = atof(Value.c_str());
			if (Prob == 0 && ProbIndex != 0xff)
				Die("AcceptProbs: ProbIndex %02x=0.0, only ff may be 0.0",
				  ProbIndex);
			if (Prob == 1.0 && ProbIndex != 0)
				Die("AcceptProbs: ProbIndex %02x=1.0, only 00 may be 1.0",
				  ProbIndex);
			if (Prob < 0.0 || Prob > 1.0)
				Die("Invalid accept prob '%s'", Value.c_str());
			P_AcceptProbs[ProbIndex++] = Prob;
			}
		}

	if (ProbIndex != 256)
		Die("%u accept probs specified, must be exactly 256", ProbIndex);
	}

void LogAcceptProbs()
	{
	Log("\n");
	Log("AcceptProbs=\n");
	for (unsigned i = 0; i < 256; ++i)
		Log("  %0.6f;    #  %02x\n", P_AcceptProbs[i], i);
	Log("//\n");
	Log("\n");
	}
