#include "evo.h"

static char *Letters[4] = { "A", "C", "G", "T" };

void ReadNucMx(FILE *f)
	{
	string Line;

	bool EndOfFile = ReadEvoParamsLine(f, Line);
	if (EndOfFile)
		Die("NucleotideSubstProbs=, premature end-of-file");
	vector<string> Fields;
	ParseStrVecWS(Line, Fields);

	if (Fields.size() != 4 || Fields[0] != "A" || Fields[1] != "C" || Fields[2] != "G"
	  || Fields[3] != "T")
		Die("NucleotideSubstProbs=, expected heading line A C G T, got '%s'",
		  Line.c_str());

	for (unsigned i = 0; i < 4; ++i)
		{
		bool EndOfFile = ReadEvoParamsLine(f, Line);
		if (EndOfFile)
			Die("NucleotideSubstProbs=, premature end-of-file");

		vector<string> Fields;
		ParseStrVecWS(Line, Fields);

		if (Fields.size() != 5 || Fields[0] != Letters[i])
			Die("NucleotideSubstProbs=, expected row for %s, got '%s'",
			  Letters[i], Line.c_str());

		for (unsigned j = 0; j < 4; ++j)
			{
			const string &f = Fields[j+1];
			if (f == ".")
				{
				P_SubstProbs[i][j] = 0;
				continue;
				}
			double p = atof(f.c_str());
			if (p < 0 || p > 1)
				Die("NucleotideSubstProbs=, invalid probability '%s' in line '%s'",
				  Fields[j+1].c_str(), Line.c_str());
			if (i == j && p != 0)
				Die("NucleotideSubstProbs=, diagonal must be zero");
			P_SubstProbs[i][j] = p;
			}
		}
	}

void LogNucMx()
	{
	Log("\n");
	Log("NucleotideSubstProbs=\n");
	Log("          A        C        G        T\n");
	for (unsigned i = 0; i < 4; ++i)
		{
		Log("%s ", Letters[i]);
		for (unsigned j = 0; j < 4; ++j)
			{
			if (i == j)
				Log("        .");
			else
				Log("  %7.5f", P_SubstProbs[i][j]);
			}
		Log("\n");
		}
	Log("\n");
	}
