#include "randdist.h"
#include "utils.h"

/***
Format is:

<Name>=
	Weight, Mean, Shape;
	Weight, Mean, Shape;
	...
	Weight, Mean, Shape;
//

The <Name>= line has already been read before ReadSpeed
is called.
***/

using namespace std;

bool ReadEvoParamsLine(FILE *f, string &Line);
void DeleteWhiteSpace(const string &In, string &Out);
void ParseStrVec(const std::string &In, std::vector<std::string> &Out, char Sep);

void ReadSpeed(FILE *f, vector<RDParams> &RDPs)
	{
	RDPs.clear();
	for (;;)
		{
		string Line;
		bool EndOfFile = ReadEvoParamsLine(f, Line);
		if (EndOfFile)
			Die("Premature EOF in speed params, missing '//'");

		if (Line.empty())
			continue;

		if (Line == "//")
			return;

		string sLine;
		DeleteWhiteSpace(Line, sLine);

		vector<string> Triples;
		ParseStrVec(sLine, Triples, ';');

		for (vector<string>::const_iterator p = Triples.begin();
		  p != Triples.end(); ++p)
			{
			const string &Triple = *p;

			vector<string> v;
			ParseStrVec(Triple, v, ',');
			if (v.size() != 3)
				Die("Invalid triple in speed '%s'", Line.c_str());

			RDParams RDP;
			RDP.Weight = atof(v[0].c_str());
			RDP.Mean = atof(v[1].c_str());
			RDP.Shape = atof(v[2].c_str());

			RDPs.push_back(RDP);
			}
		}	
	}
