#include "stack.h"

void Stacker::LogBaseInfo(const BaseInfo &Base) const
	{
	Base.LogMe();
	}

void Stacker::LogBase(unsigned BaseIndex) const
	{
	const BaseInfo &Base = GetBase(BaseIndex);
	Log("%5u", BaseIndex);
	Log("  %3u", Base.GenIndex);
	Log("  %5u", Base.ChrId);
	Log("  %10u", (unsigned) Base.Pos);
	if (isprint(Base.Letter))
		Log("  %c", Base.Letter);
	else
		Log(" <UNPRINTABLE 0x%02x>", Base.Letter);

	const IntVec &Neighbors = Edges[BaseIndex];
	const BoolVec &Pluses = EdgePluses[BaseIndex];
	unsigned NeighborCount = SIZE(Neighbors);
	asserta(NeighborCount == SIZE(Pluses));

	Log("  ");
	for (unsigned i = 0; i < NeighborCount; ++i)
		{
		unsigned NeighborIndex = Neighbors[i];
		bool EdgePlus = Pluses[i];

		const BaseInfo &Neighbor = GetBase(NeighborIndex);
		Log("%c", EdgePlus ? Neighbor.Letter : CompLetter(Neighbor.Letter));
		}

	if (NeighborCount < 8)
		for (unsigned i = 0; i < 8 - NeighborCount; ++i)
			Log(" ");

	Log("   ");
	for (unsigned i = 0; i < NeighborCount; ++i)
		{
		unsigned NeighborIndex = Neighbors[i];
		bool EdgePlus = Pluses[i];
		Log(" %u%c", NeighborIndex, EdgePlus ? '+' : '-');
		}
	}

void Stacker::LogBWS(const BaseWithSign &BWS) const
	{
	Log("%u,%u:%u[%c=%c]%c",
	  BWS.Base.GenIndex,
	  BWS.Base.ChrId,
	  (unsigned) BWS.Base.Pos,
	  BWS.Base.Letter,
	  BWS.GetLetter(),
	  BWS.Plus ? '+' : '-');
	}

void Stacker::LogGraph() const
	{
	const unsigned BaseCount = SIZE(Bases);
	asserta(BaseCount == SIZE(Edges));
	asserta(BaseCount == SIZE(EdgePluses));

	Log("\n");
	Log("Graph:\n");
	Log("Index  Gen    Chr         Pos  c  Neighbors\n");
	Log("=====  ===  =====  ==========  =  =========\n");

	for (std::map<BaseInfo, unsigned>::const_iterator p = BaseInfoToIndex.begin();
	  p != BaseInfoToIndex.end(); ++p)
		{
		unsigned BaseIndex = p->second;
		LogBase(BaseIndex);
		Log("\n");
		}
	}

void Stacker::LogCCs(const IntVecVec &CCs, const BoolVec &Signs) const
	{
	Log("\n");
	Log("CCs:\n");
	const unsigned CCCount = SIZE(CCs);
	FORIX(CC)
		{
		const IntVec &CC = CCs[CCIndex];
		Log("\n");
		Log("CC%u(%u)\n", CCIndex, SIZE(CC));
		Log("Index  Gen    Chr         Pos  c  Neighbors, sign\n");
		Log("=====  ===  =====  ==========  =  ===============\n");
		for_CIntVec(CC, p)
			{
			unsigned BaseIndex = *p;
			LogBase(BaseIndex);
			Log(" %c", Signs[BaseIndex] ? '+' : '-');
			Log("\n");
			}
		}
	}

void Stacker::LogOutputColBases() const
	{
	const unsigned ColCount = SIZE(OutputColBases);
	Log("\n");
	Log("OutputCols:\n");
	FORIX(Col)
		{
		const std::vector<BaseWithSign> &Col = OutputColBases[ColIndex];
		const unsigned BaseCount = SIZE(Col);
		for (unsigned i = 0; i < BaseCount; ++i)
			{
			const BaseWithSign &BWS = Col[i];
			char Letter = BWS.GetLetter();
			Log("%c", Letter);
			}
		Log("  ");
		for (unsigned i = 0; i < BaseCount; ++i)
			{
			const BaseWithSign &BWS = Col[i];
			Log(" [%u,%u]%u%c",
			  BWS.Base.GenIndex,
			  BWS.Base.ChrId,
			  (unsigned) BWS.Base.Pos,
			  BWS.Plus ? '+' : '-');
			}
		Log("\n");
		}
	}

void Stacker::LogCol(const std::vector<BaseWithSign> &Col) const
	{
	const unsigned BaseCount = SIZE(Col);
	for (unsigned i = 0; i < BaseCount; ++i)
		{
		const BaseWithSign &BWS = Col[i];
		char Letter = BWS.GetLetter();
		Log(" [%u,%u]%u %c%c",
		  BWS.Base.GenIndex,
		  BWS.Base.ChrId,
		  (unsigned) BWS.Base.Pos,
		  Letter,
		  BWS.Plus ? '+' : '-');
		}
	}

void Stacker::LogOutputBlock() const
	{
	Log("OutputBlock:\n");
	StdStrVec OutputRows;
	BuildOutputRows(OutputRows);
	LogAIRows(OutputAI, OutputRows);
	}
