#include "stack.h"

unsigned g_MaxUnstackedSegCount = 100;
unsigned g_UnstackedBaseCount;
unsigned g_TotalUnstackedBaseCount;
unsigned g_UnstackedSegCount;

void StackStats()
	{
	const char *InFileName = RequiredValueOpt("stats");
	IntOpt("maxunstackedsegs", &g_MaxUnstackedSegCount);

	RevFile In;
	In.Open(InFileName);
	
	const unsigned GenCount = In.GetGenomeCount();
	for (unsigned GenIndex = 0; GenIndex < GenCount; ++GenIndex)
		{
		const GenInfo &GI = In.GetGenomeInfo(GenIndex);
		std::string GenName = In.GetGenomeName(GenIndex);
		TruncLabel(GenName);
		unsigned ChrCount = In.GetChrCount(GenIndex);

		Progress("Genome %.16s has %u chrs", GenName.c_str(), ChrCount);
		for (unsigned ChrIndex = 0; ChrIndex < ChrCount; ++ChrIndex)
			{
			const ChrInfo &CIA = GI.GetChrInfoByIndex(ChrIndex);
			unsigned ChrId = CIA.GetId();

			double Pct = ChrIndex*100.0/ChrCount;
			const std::string &ChrName = GI.GetChrName(ChrId);

			IntVec AIIndexes;
			In.GetAppendAlnInfoIndexesChr(GenIndex, ChrId, AIIndexes);

			AIXVec AIXs;
			In.GetAlnInfoXs(AIIndexes, AIXs);

			DumpStackStats(In, GenIndex, ChrId, AIXs);

			Progress("Chr %.16s %u of %u %.1f%% %u overlaps, total %u",
			  ChrName.c_str(),
			  ChrIndex,
			  ChrCount,
			  Pct,
			  g_UnstackedBaseCount,
			  g_TotalUnstackedBaseCount);
			}
		}

	DumpStackStatsTotal();
	In.Close();
	}
