#include "stdio.h"
#include "timing.h"
#include "types.h"
#include <time.h>

static unsigned *g_Calls;
static TICKS *g_Ticks;
static double g_TicksPerSec = 2.95e9;

const char *TimerOpToStr(TIMEROP t)
	{
	switch (t)
		{
#define T(t)	case TIMEROP_##t: return #t;
#include "timerops.h"
		}
	return "?";
	}

void CalibrateTimer()
	{
	printf("Calibrating timer: ");
	time_t secs1 = time(0);
	TICKS ticks1 = GetClockTicks();
	time_t secs2 = 0;
	time_t last_secs2 = 0;
	for (;;)
		{
		secs2 = time(0);
		if (secs2 != last_secs2)
			printf(" %ld", (long) (10 - (secs2 - secs1)));
		last_secs2 = secs2;
		if (secs2 - secs1 >= 10)
			break;
		}
	TICKS ticks2 = GetClockTicks();
	g_TicksPerSec = (double) (ticks2 - ticks1)/(double) (secs2 - secs1);
	printf("\n%.3g ticks/sec\n", g_TicksPerSec);
	}

void InitTiming()
	{
	g_Calls = new unsigned[TimerOpCount];
	g_Ticks = new TICKS[TimerOpCount];
	memset(g_Calls, 0, TimerOpCount*sizeof(unsigned));
	memset(g_Ticks, 0, TimerOpCount*sizeof(TICKS));
	}

void TimerDone(TIMEROP t, TICKS t1, TICKS t2)
	{
	++(g_Calls[t]);
	g_Ticks[t] += t2 - t1;
	}

void TimingReport()
	{
	printf("\n");
	printf("                      Op        Calls                 Ticks        Secs   Secs/call\n");
	printf("========================  ===========  ====================  ==========  ==========\n");
	for (unsigned i = 0; i < TimerOpCount; ++i)
		{
		unsigned Calls = g_Calls[i];
		uint64 Ticks = (uint64) g_Ticks[i];
		double Secs = Ticks/g_TicksPerSec;
		double SecsPerCall = Calls == 0 ? 0 : Secs/Calls;

		printf("%24.24s", TimerOpToStr((TIMEROP) i));
		printf("  %11u", Calls);
		printf("  %20"UINT64_PRINTF, Ticks);
		printf("  %10.3g", Secs);
		printf("  %10.3g\n", SecsPerCall);
		}
	}
