#ifndef ALIGN4_H
#define ALIGN4_H

#include "filter4.h"
#include "utils.h"
#include "getopts.h"
#include "progress.h"
#include "revfile.h"
#include "align_params.h"
#include "alnai.h"

void GetAlignParams();

void AlignAIsGlobal(const AIVec &AIs, StdStrVec &Paths);

void WriteAln(RevFile *ptrRev, const AlnInfo &AI, const StdStr &Path,
  AlnAttr Attr, unsigned Value);

void WriteAlns(RevFile *ptrRevFile, const AIVec &AIs, const StdStrVec &Paths,
  AlnAttr Attr, unsigned Value);

void WriteAIs(RevFile *ptrRev, const AIVec &AIs, AlnAttr Attr, unsigned Value);

void MakeHoles4(const AIVec &InHits, IntVec &HitToLoHole, IntVec &HitToHiHole,
  IntVec &HoleToLoHit, IntVec &HoleToHiHit, AIVec &HoleAIs);

void MakeChains4(unsigned RealignedHitCount, unsigned HoleCount,
  const IntVec &HoleToLoHit, const IntVec &HoleToHiHit, IntVecVec &Chains);

void AlignChains4(const IntVecVec &Chains, const AIVec &RealignedHits,
  const StdStrVec &RealignedPaths, const AIVec &RealignedHoleAIs,
  const StdStrVec &RealignedHolePaths, const IntVec &HitToLoHole,
  const IntVec &HitToHiHole, AIVec &ChainHits, StdStrVec &ChainPaths);

void GetBendpoints(const std::vector<AlnInfo> &AIs, unsigned GenomeIndex,
  std::vector<AIBP> &BPs, unsigned Margin);

void MakeAln(const char *A, unsigned LA, const char *B,
  unsigned LB, const std::string &ssPath, std::string &Row1,
  std::string &Row2);

const RevFile &GetSeqRevFileA();

const RevFile &GetSeqRevFileB();

unsigned CalcRadius(coord_t Length1, coord_t Length2, coord_t AlnLength);

void STRVECToStdStrVec(const STRVEC svPaths, StdStrVec &Paths);

void OutputPairwiseAln(RevFile &Out, const RevFile &SeqRevFileA,
  const RevFile &SegRevFileB, const AlnInfo &UserAI, const StdStr &Path);

void MakeAln(const char *Seq1, unsigned Length1, const char *Seq2,
  unsigned Length2, const std::string &Path, std::string &Row1,
  std::string &Row2);

void Slide(const char *Row1, const char *Row2, unsigned AlnLength,
  unsigned WindowLength, unsigned WindowOffset, unsigned MinBadLength,
  IntVec &ColStarts, IntVec &ColEnds);

void PathFromAln(const char *Row1, const char *Row2, unsigned AlnLength,
  std::string &Path);

void ColToSeq(const IntVec &Cols, const std::string &Path, IntVec &SeqPos1,
  IntVec &SeqPos2);

score_t GetAlnRuns(const char *Row1, const char *Row2, unsigned Length,
  IntVec &RunStarts, IntVec &RunEnds, std::vector<score_t> &RunScores);

void PostProcSlide(const char *Seq1, unsigned Length1, const char *Seq2,
  unsigned Length2, const std::string &Path, AIVec &Hits, StdStrVec &Paths,
  IntVec &ColStarts, IntVec &ColEnds);

void GetBoundRect(const AIVec &AIs, unsigned GenomeIndexA, unsigned GenomeIndexB,
  AlnInfo &Rect);

void ValidateHitPath(const AlnInfo &Hit, unsigned GenomeIndexA,
  unsigned GenomeIndexB, const StdStr &Path);

void GetConnectedComponents(const IntVecVec &AdjMx_, IntVecVec &CCs, bool ShowProgress);

RevFile *GetTraceRevFilePtr();

void AlignNWLocal(const char *Seq1, coord_t Length1, const char *Seq2,
  coord_t Length2, unsigned Radius, AIVec &Hits, StrVec &Paths);

unsigned CalcRadius(coord_t Length1, coord_t Length2, coord_t AlnLength);

void AddOffsets(const AlnInfo &Region, AlnInfo &Hit);

void AddOffsetsMinus(const AlnInfo &Region, coord_t LengthA,
  coord_t LengthB, AlnInfo &Hit);

void AddOffsetsVec(const AlnInfo &Region, AIVec &AIs);

void AddOffsetsMinusVec(const AlnInfo &Region, coord_t LengthA, coord_t LengthB,
  AIVec &AIs);

void LogAIs(const AIVec &AIs);

extern unsigned g_EndHoleSize;

#endif	// ALIGN4_H
