
#include "align4.h"

#define TRACE	0

unsigned g_EndHoleSize = 500;

static RevFile *g_ptrOutRevFile;
static RevFile *g_ptrSeqRevFileA;
static RevFile *g_ptrSeqRevFileB;
static RevFile *g_ptrTraceRevFile;

static coord_t g_ChrLengthA;
static coord_t g_ChrLengthB;

coord_t GetChrLengthA()
	{
	return g_ChrLengthA;
	}

coord_t GetChrLengthB()
	{
	return g_ChrLengthB;
	}

static void SetChrLengths()
	{
	g_ChrLengthA = GetFilterChrInfoA().GetLength();
	g_ChrLengthB = GetFilterChrInfoB().GetLength();
	}

const RevFile &GetSeqRevFileA()
	{
	if (g_ptrSeqRevFileA == 0)
		Die("g_ptrSeqRevFileA == 0");
	return *g_ptrSeqRevFileA;
	}

const RevFile &GetSeqRevFileB()
	{
	if (g_ptrSeqRevFileB == 0)
		Die("g_ptrSeqRevFileB == 0");
	return *g_ptrSeqRevFileB;
	}

RevFile *GetTraceRevFilePtr()
	{
	return g_ptrTraceRevFile;
	}

RevFile &GetOutRevFile()
	{
	if (g_ptrOutRevFile == 0)
		Die("g_ptrOutRevFile == 0");
	return *g_ptrOutRevFile;
	}

void WriteAIs(RevFile *ptrRev, const AIVec &AIs, AlnAttr Attr, unsigned Value)
	{
	if (ptrRev == 0)
		return;
	const unsigned AICount = (unsigned) AIs.size();
	for (unsigned AIIndex = 0; AIIndex < AICount; ++AIIndex)
		{
		const AlnInfo &AI = AIs[AIIndex];
		AlnInfo AICopy = AI;
		unsigned BundleId = GetFilterBundleId();
		AICopy.SetBundleIndex(BundleId);
		if (Attr != ALN_ATTR_None)
			AICopy.SetAttr(Attr, Value);
		FilterAwareOutputAlnInfo(*ptrRev, AICopy);
		}
	}

void WriteAln(RevFile *ptrRev, const AlnInfo &AI, const StdStr &Path,
  AlnAttr Attr, unsigned Value)
	{
	if (ptrRev == 0)
		return;

	const RevFile &SeqRevFileA = GetSeqRevFileA();
	const RevFile &SeqRevFileB = GetSeqRevFileB();

	AlnInfo AICopy = AI;
	unsigned BundleId = GetFilterBundleId();
	AICopy.SetBundleIndex(BundleId);
	AICopy.SetAttr(Attr, Value);
	OutputPairwiseAln(*ptrRev, SeqRevFileA, SeqRevFileB, AICopy, Path);
	}

void WriteAlns(RevFile *ptrRev, const AIVec &AIs, const StdStrVec &Paths,
  AlnAttr Attr, unsigned Value)
	{
	if (ptrRev == 0)
		return;

	const unsigned AICount = (unsigned) AIs.size();
	for (unsigned AIIndex = 0; AIIndex < AICount; ++AIIndex)
		{
		const AlnInfo &AI = AIs[AIIndex];
		const StdStr &Path = Paths[AIIndex];

		WriteAln(ptrRev, AI, Path, Attr, Value);
		}
	}

static void AfterAllChrPairs()
	{
	Progress("Closing sequence files");

	g_ptrSeqRevFileA->Close();
	delete g_ptrSeqRevFileA;
	g_ptrSeqRevFileA = 0;

	g_ptrSeqRevFileB->Close();
	delete g_ptrSeqRevFileB;
	g_ptrSeqRevFileB = 0;

	if (g_ptrTraceRevFile != 0)
		{
		Progress("Closing trace file");
		g_ptrTraceRevFile->Close();
		delete g_ptrTraceRevFile;
		g_ptrTraceRevFile = 0;
		}

	Progress("Closing output file");
	if (g_ptrOutRevFile == 0)
		Die("g_ptrOutRevFile == 0");

	g_ptrOutRevFile->Close();
	delete g_ptrOutRevFile;
	g_ptrOutRevFile = 0;
	}

void InitFilter4()
	{
	GetAlignParams();
	SetInvertMinusBundles(true);

	const char *OutFileName = ValueOpt("out");
	const char *TraceFileName = ValueOpt("trace");
	const char *SeqFileNameA = ValueOpt("seq1");
	const char *SeqFileNameB = ValueOpt("seq2");

	if (OutFileName == 0)
		OutFileName = "align4.rev";

	if (SeqFileNameA == 0)
		SeqFileNameA = "seq1.rev";

	if (SeqFileNameB == 0)
		SeqFileNameB = "seq2.rev";

	IntOpt("endholesize", &g_EndHoleSize);

	g_ptrSeqRevFileA = new RevFile;
	g_ptrSeqRevFileB = new RevFile;
	g_ptrOutRevFile = new RevFile;

	g_ptrSeqRevFileA->Open(SeqFileNameA);
	g_ptrSeqRevFileB->Open(SeqFileNameB);

	const RevFile &In = GetFilterInputRevFile();
	const std::vector<GenInfo> &Genomes = In.GetGenomes();

	g_ptrOutRevFile->Create(OutFileName, Genomes);

	if (TraceFileName != 0)
		{
		g_ptrTraceRevFile = new RevFile;
		g_ptrTraceRevFile->Create(TraceFileName, Genomes);
		}

	SetAfterAllChrPairsCallback(AfterAllChrPairs);
	}

void Filter4(const AIVec &AIs, const IntVec &/*AIIndexes*/)
	{
#if	TRACE
	Log("\n");
	Log("Filter, input AIs:\n");
	LogAIs(AIs);
	Log("\n");
#endif

	const unsigned AICount = (unsigned) AIs.size();
	if (AICount == 0)
		return;

	unsigned BundleId = GetFilterBundleId();
	bool Inverting = FilterIsInverting();
	Progress("Bundle %u, %u AIs %s",
	  BundleId,
	  AICount,
	  Inverting ? " (Minus)" : "");

	SetChrLengths();

// Re-align input hits.
	AIVec RealignedAIs;
	StdStrVec RealignedPaths;
	FilterAwareAlignAIsLocal(AIs, RealignedAIs, RealignedPaths, ALIGN_STYLE_PlusOnly);

	if (RealignedAIs.size() == 0)
		{
		Progress("No realigned hits");
		return;
		}

// Save realigned hits to trace file.
	WriteAlns(g_ptrTraceRevFile, RealignedAIs, RealignedPaths,
	  ALN_ATTR_TraceType, TraceType_RealignedHit);

// Find holes in realigned hits.
	IntVec HitToLoHole;
	IntVec HitToHiHole;
	IntVec HoleToLoHit;
	IntVec HoleToHiHit;
	AIVec RealignedHoleAIs;
	MakeHoles4(RealignedAIs, HitToLoHole, HitToHiHole, HoleToLoHit, HoleToHiHit,
	  RealignedHoleAIs);

// Align holes between realigned hits.
	StdStrVec RealignedHolePaths;
	FilterAwareAlignAIsGlobal(RealignedHoleAIs, RealignedHolePaths);

// Save first round hole hits to trace file
	WriteAlns(g_ptrTraceRevFile, RealignedHoleAIs, RealignedHolePaths,
	  ALN_ATTR_TraceType, TraceType_RealignedHole);

// Make monotonic chains from hits & holes.
	IntVecVec MonoChains;
	const unsigned RealignedHitCount = (unsigned) RealignedAIs.size();
	const unsigned RealignedHoleCount = (unsigned) RealignedHoleAIs.size();
	MakeChains4(RealignedHitCount, RealignedHoleCount, HoleToLoHit,
	  HoleToHiHit, MonoChains);

// Construct global alignment for each chain and extract optimal local
// alignments, giving chain hits (a.k.a. local chain alignments).
	AIVec LocalChainAIs;
	StdStrVec LocalChainPaths;
	AlignChains4(MonoChains, RealignedAIs, RealignedPaths, RealignedHoleAIs,
	  RealignedHolePaths, HitToLoHole, HitToHiHole, LocalChainAIs, LocalChainPaths);

// Make holes in re-aligned bundle, i.e. between local chain alignments.
	AIVec ChainHoleAIs;
	MakeHoles4(LocalChainAIs, HitToLoHole, HitToHiHole, HoleToLoHit,
	  HoleToHiHit, ChainHoleAIs);

// Write realigned holes to trace file.
	WriteAIs(g_ptrTraceRevFile, ChainHoleAIs, ALN_ATTR_TraceType,
	  TraceType_RealignedHole);

// Look for minus hits in new holes.
	AIVec HoleMinusAIs;
	StdStrVec HoleMinusPaths;
	FilterAwareAlignAIsLocal(ChainHoleAIs, HoleMinusAIs, HoleMinusPaths,
	  ALIGN_STYLE_MinusOnly);

// Write final alignments.
	RevFile &Out = GetOutRevFile();

	WriteAlns(&Out, LocalChainAIs, LocalChainPaths, ALN_ATTR_TraceType,
	  TraceType_ChainHit);

	WriteAlns(g_ptrTraceRevFile, LocalChainAIs, LocalChainPaths,
	  ALN_ATTR_TraceType, TraceType_ChainHit);

	WriteAlns(&Out, HoleMinusAIs, HoleMinusPaths, ALN_ATTR_TraceType,
	  TraceType_HoleMinusHit);

	WriteAlns(g_ptrTraceRevFile, HoleMinusAIs, HoleMinusPaths, ALN_ATTR_TraceType,
	  TraceType_HoleMinusHit);

	unsigned PlusHitCount = (unsigned) LocalChainAIs.size();
	unsigned MinusHitCount = (unsigned) HoleMinusAIs.size();

	Progress("%u plus hits, %u minus hits", PlusHitCount, MinusHitCount);
	}
