#ifndef ALNAI_H
#define ALNAI_H

#include <string>
#include <stack>
#include <ctype.h>
#include <math.h>
#include "types.h"
#include "utils.h"
#include "align_params.h"
#include "scoremx.h"

// Compress M, D and I trace-back matrices into 4 bits
enum
	{
	BIT_MM = 0x00,
	BIT_DM = 0x01,
	BIT_IM = 0x02,
	BIT_xM = 0x03,

	BIT_DD = 0x00,
	BIT_MD = 0x04,
	//  ID not allowed
	BIT_xD = 0x04,

	BIT_II = 0x00,
	BIT_MI = 0x08,
	//  DI not allowed
	BIT_xI = 0x08,
	};

score_t NWSimple(const unsigned char *A, unsigned uLengthA, const unsigned char *B,
  unsigned uLengthB, std::string &Path);

score_t NWBanded(const unsigned char *A, unsigned LA, const unsigned char *B,
  unsigned LB, unsigned r, std::string &Path);

score_t NWEnv(const unsigned char *A, unsigned LA, const unsigned char *B,
  unsigned LB, const IntVec &StartB, const IntVec &EndB, std::string &Path);

score_t MMSimple(const unsigned char *A_, unsigned LA_, const unsigned char *B_,
  unsigned LB_, std::string &Path);

score_t MMEnv(const unsigned char *A_, unsigned LA_, const unsigned char *B_,
  unsigned LB_, const IntVec &StartB, const IntVec &EndB, std::string &Path);

score_t SWSimple(const unsigned char *A, unsigned LA, const unsigned char *B,
  unsigned LB, unsigned *ptrStartA, unsigned *ptrStartB, std::string &Path);

score_t TraceBack(const unsigned char *A, unsigned uLengthA, const unsigned char *B,
  unsigned uLengthB, const score_t *DPMb_, const score_t *DPDb_, const score_t *DPIb_,
  std::string &Path);

void SWTraceBack(const unsigned char *A, unsigned LA, const unsigned char *B,
  unsigned LB, unsigned Topi, unsigned Topj, score_t TopScore, const score_t *DPM_,
  const score_t *DPD_, const score_t *DPI_, std::string &Path, unsigned *ptrStartA,
  unsigned *ptrStartB);

score_t TraceBackBanded(const unsigned char *A, unsigned LA, const unsigned char *B,
  unsigned LB, unsigned r, score_t **DPM, score_t **DPD, score_t **DPI, std::string &Path);

score_t TraceBackEnv(const unsigned char *A, unsigned LA, const unsigned char *B,
  unsigned LB, const IntVec &StartB, const IntVec &EndB, score_t **DPM, score_t **DPD,
  score_t **DPI, std::string &Path);

void Rangej(unsigned i, unsigned r, unsigned LA, unsigned LB,
  unsigned *ptrMinj, unsigned *ptrMaxj);

void Band(unsigned i, unsigned LA, unsigned LB, unsigned r,
   unsigned *ptrMidj, unsigned *ptrMinJ, unsigned *ptrMaxJ);

const char *ScoreToStr(score_t s);

score_t ScorePath(const unsigned char *A, unsigned LA, const unsigned char *B, unsigned LB,
  const char *Path);

score_t ScorePathLocal(const unsigned char *A, unsigned LA, const unsigned char *B,
  unsigned LB, unsigned StartA, unsigned StartB, const char *Path);

bool AcceptAln(score_t Score, unsigned Length);

void AlignNWGlobal(const char *Seq1, coord_t Length1, const char *Seq2,
  coord_t Length2, unsigned Radius, AIVec &OutAIs, STRVEC &Paths);

void AlignNWLocalAI(const char *Seq1, coord_t Length1, const char *Seq2,
  coord_t Length2, unsigned Radius, AIVec &Hits, StdStrVec &Paths);

void LogAlnLocal(const char *A, unsigned LA, const char *B, unsigned LB, unsigned StartA,
  unsigned StartB, const char *Path_, const char *Prefix);

score_t TraceBackGAG(const unsigned char *A, unsigned LA, const unsigned char *B,
  unsigned LB, const score_t *DPM_, const score_t *DPD_, const score_t *DPI_,
  const score_t *DPU_, std::string &Path);

score_t NWSimpleGAG(const unsigned char *A, unsigned LA, const unsigned char *B,
  unsigned LB, std::string &Path);

score_t ScorePathGAG(const unsigned char *A, unsigned LA, const unsigned char *B,
  unsigned LB, const char *Path);

void NWMuscleTraceBack(char **TraceBack, unsigned LengthA, unsigned LengthB,
  char LastEdge, std::string &Path);

score_t NWMuscle(const unsigned char *PA, unsigned uLengthA, const unsigned char *PB,
  unsigned uLengthB, std::string &Path);

// Macros to simulate 2D matrices
#define DPM(i, j)	DPM_[(j)*PCA + (i)]
#define DPD(i, j)	DPD_[(j)*PCA + (i)]
#define DPI(i, j)	DPI_[(j)*PCA + (i)]
#define DPU(i, j)	DPU_[(j)*PCA + (i)]

#define TBM(i, j)	TBM_[(j)*PCA + (i)]
#define TBD(i, j)	TBD_[(j)*PCA + (i)]
#define TBI(i, j)	TBI_[(j)*PCA + (i)]

#define DPMb(i, j)	DPMb_[BandIndex((i), (j), LA, LB, r)]
#define DPDb(i, j)	DPDb_[BandIndex((i), (j), LA, LB, r)]
#define DPIb(i, j)	DPIb_[BandIndex((i), (j), LA, LB, r)]
#define DPUb(i, j)	DPUb_[BandIndex((i), (j), LA, LB, r)]

#define SUBST(cA, cB)		(DNAScoreMx[(unsigned char) (cA)][(unsigned char) (cB)])

#define TERMGAP		GAPEXTEND

const score_t MINUS_INFINITY = -999999;

#endif // ALNAI_H
