#ifndef BMGRAPH_H
#define BMGRAPH_H

#include "bmtypes.h"

class BMGraph
{
	friend class BMSolver;

	private:

	int      m,                      /* internal number of arcs */
	         n,                      /* internal number of nodes */
	         node_min,               /* minimal no of node  */
	         node_max,               /* maximal no of nodes */
	         *arc_first,             /* internal array for holding
	                                    - node degree
	                                    - position of the first outgoing arc */
	         *arc_tail;              /* internal array: tails of the arcs */

	node     *nodes;                 /* pointers to the node structure */

	arc      *arcs,                  /* pointers to the arc structure */
	         *arc_current;

	price_t  m_c;                    /* maximum arc cost */

	int      *cap;                   /* array of capacities */

	excess_t total_p,                /* total supply */
	         total_n;                /* total demand */

	int      pos_current;            /* current edge position for filling edges */

	bool     Initialized;

	void parse_bm(int *n_ad, int *m_ad, node **nodes_ad, 
	              arc **arcs_ad, int *node_min_ad, price_t *m_c_ad, 
	              int **cap_ad);
	public:

	BMGraph();
	BMGraph(int TotalNodes, int TotalEdges);
	~BMGraph();

	void Init(int TotalNodes, int TotalEdges);
	void Destroy();
	void FillNode(int i, int excess);
	void FillEdge(int tail, int head, int low, int acap, price_t cost);
	void Solve();
	node *GetNodeArray();
	arc *GetArcArray();
	int *GetCapArray();
};
#endif
