#include "utils.h"
#include "revfile.h"
#include "progress.h"
#include "uniquebid.h"

#define TRACE	0

void ComputeBundleLoHis(const AIVec &AIs, unsigned GenomeIndex,
  std::map<UniqueBundleId, coord_t> &los, std::map<UniqueBundleId, coord_t> &his)
	{
	los.clear();
	his.clear();

#if	TRACE
	Log("\n");
	Log("ComputeBundleLoHis:\n");
#endif
	const unsigned AICount = (unsigned) AIs.size();
	ProgressStart("Compute bundle lo/hi's");
	for (unsigned i = 0; i < AICount; ++i)
		{
		if (i%10000 == 0)
			ProgressStep(i, AICount);

		const AlnInfo &AI = AIs[i];

		const GCSeg &Seg0 = AI.GetSeg(0);
		const GCSeg &Seg1 = AI.GetSeg(1);
		unsigned BundleId = AI.GetBundleId();
		if (BundleId == UINT_MAX)
			{
			Log("\n");
			LogAI(AI);
			Log("\n");
			Die("AlnInfo missing bundle index");
			}

		UniqueBundleId UB;
		UB.ChrId0 = Seg0.ChrId;
		UB.ChrId1 = Seg1.ChrId;
		UB.BundleId = BundleId;

		const GCSeg &Seg = AI.GetSeg(GenomeIndex);
		coord_t lo = Seg.lo;
		coord_t hi = Seg.hi;

#if	TRACE
		Log("AI%u ", i);
		LogAIN(AI);
		Log("UB ChrId0 %u ChrId1 %u BundleId %u\n", UB.ChrId0, UB.ChrId1, UB.BundleId);
#endif
		std::map<UniqueBundleId, coord_t>::iterator plo = los.find(UB);
		std::map<UniqueBundleId, coord_t>::iterator phi = his.find(UB);
		if (plo == los.end())
			{
			asserta(phi == his.end());
			std::map<UniqueBundleId, coord_t>::value_type Value_lo(UB, lo);
			std::map<UniqueBundleId, coord_t>::value_type Value_hi(UB, hi);

			los.insert(Value_lo);
			his.insert(Value_hi);
			}
		else
			{
			assert(phi != his.end());
			coord_t old_lo = plo->second;
			coord_t old_hi = phi->second;
			if (lo < old_lo)
				{
				los.erase(plo);
				std::map<UniqueBundleId, coord_t>::value_type Value(UB, lo);
				los.insert(Value);
				}
			if (hi > old_hi)
				{
				his.erase(phi);
				std::map<UniqueBundleId, coord_t>::value_type Value(UB, hi);
				his.insert(Value);
				}
			}
		}
	ProgressDone();
	}

void GetBundleLoHi(const UniqueBundleId &UB,
  const std::map<UniqueBundleId, coord_t> &los,
  const std::map<UniqueBundleId, coord_t> &his,
  coord_t &lo, coord_t &hi)
	{
	std::map<UniqueBundleId, coord_t>::const_iterator plo = los.find(UB);
	std::map<UniqueBundleId, coord_t>::const_iterator phi = his.find(UB);
	asserta(plo != los.end());
	asserta(phi != his.end());

	lo = plo->second;
	hi = phi->second;
	}
