#include "utils.h"
#include "revfile.h"
#include "progress.h"
#include "uniquebid.h"

#define TRACE	0

void ComputeBundleScores(const AIVec &AIs, std::map<UniqueBundleId, ai_score_t> &BundleScores)
	{
#if	TRACE
	Log("\n");
	Log("BundleScores:\n");
#endif
	const unsigned AICount = (unsigned) AIs.size();
	ProgressStart("Computing bundle scores");
	for (unsigned i = 0; i < AICount; ++i)
		{
		if (i%10000 == 0)
			ProgressStep(i, AICount);

		const AlnInfo &AI = AIs[i];

		const GCSeg &Seg0 = AI.GetSeg(0);
		const GCSeg &Seg1 = AI.GetSeg(1);
		unsigned BundleId = AI.GetBundleId();
		if (BundleId == UINT_MAX)
			{
			Log("\n");
			LogAI(AI);
			Log("\n");
			Die("AlnInfo missing bundle index");
			}

		UniqueBundleId UB;
		UB.ChrId0 = Seg0.ChrId;
		UB.ChrId1 = Seg1.ChrId;
		UB.BundleId = BundleId;

		std::map<UniqueBundleId, ai_score_t>::iterator p = BundleScores.find(UB);
		if (p == BundleScores.end())
			{
			std::map<UniqueBundleId, ai_score_t>::value_type Value(UB, AI.Score);
			BundleScores.insert(Value);
#if	TRACE
			Log("New: ChrId=%5u Bundle=%5u ChrId0=%5u  ChrId1=%5u  Score=%s\n",
			  ChrId,
			  UB.BundleId,
			  UB.ChrId0,
			  UB.ChrId1,
			  AIScoreToStr(AI.Score));
#endif
			}
		else
			{
			ai_score_t OldScore = p->second;
			ai_score_t NewScore = OldScore + AI.Score;
			std::map<UniqueBundleId, ai_score_t>::value_type Value(UB, NewScore);
			BundleScores.erase(p);
			BundleScores.insert(Value);
#if	TRACE
			Log("Add: ChrId=%5u Bundle=%5u ChrId0=%5u  ChrId1=%5u  Score=%s + %s = %s\n",
			  ChrId,
			  UB.BundleId,
			  UB.ChrId0,
			  UB.ChrId1,
			  ChrBundle.second,
			  AIScoreToStr(OldScore),
			  AIScoreToStr(AI.Score),
			  AIScoreToStr(NewScore));
#endif
			}
		}
	ProgressDone();
	}
