#include "utils.h"
#include "revfile.h"
#include "progress.h"

// NOTE: THIS IS IDENTICAL TO MergeHitsChrA.cpp EXCEPT
// BundleChrPair <-> MergeHitsChrPair
// SHOULD CONSOLIDATE.

void BundleChrPair(const RevFile &In, unsigned ChrIdA, unsigned ChrIdB,
  const AIXVec &ChrPairAIXs, RevFile &Out);

extern unsigned g_GenomeIndexA;
extern unsigned g_GenomeIndexB;

static unsigned GetChrIdB(const AlnInfo &AI)
	{
	const GCSeg &Seg = AI.GetSeg(g_GenomeIndexB);
	return Seg.ChrId;
	}

static void AssertSorted(const AIXVec &AIXs)
	{
	unsigned AICount = (unsigned) AIXs.size();
	for (unsigned AIIndex = 1; AIIndex < AICount; ++AIIndex)
		{
		const AlnInfo &PrevAI = AIXs[AIIndex - 1].AI;
		const AlnInfo &ThisAI = AIXs[AIIndex].AI;

		unsigned PrevChrIdB = GetChrIdB(PrevAI);
		unsigned ThisChrIdB = GetChrIdB(ThisAI);

		if (PrevChrIdB > ThisChrIdB)
			Die("Not sorted");
		}
	}

void BundleChrA(const RevFile &In, unsigned ChrIdA, const AIXVec &AIXs, RevFile &Out)
	{
	unsigned AICount = (unsigned) AIXs.size();
	if (AICount == 0)
		return;

	AssertSorted(AIXs);

	unsigned BundledCount = 0;
	unsigned PrevChrIdB = UINT_MAX;
	unsigned StartIndex = UINT_MAX;
	for (unsigned AIIndex = 0; AIIndex < AICount; ++AIIndex)
		{
		const AlnInfo &AI = AIXs[AIIndex].AI;
		unsigned ChrIdB = GetChrIdB(AI);
		if (ChrIdB != PrevChrIdB)
			{
			if (StartIndex != UINT_MAX)
				{
				AIXVec::const_iterator ptrStart = AIXs.begin() + StartIndex;
				AIXVec::const_iterator ptrEnd = AIXs.begin() + AIIndex;

				AIXVec ChrPairAIXs;
				ChrPairAIXs.insert(ChrPairAIXs.end(), ptrStart, ptrEnd);

				BundleChrPair(In, ChrIdA, PrevChrIdB, ChrPairAIXs, Out);

				BundledCount += (unsigned) ChrPairAIXs.size();
				}
			StartIndex = AIIndex;
			}
		PrevChrIdB = ChrIdB;
		}

	AIXVec::const_iterator ptrStart = AIXs.begin() + StartIndex;
	AIXVec::const_iterator ptrEnd = AIXs.end();

	AIXVec ChrPairAIXs;
	ChrPairAIXs.insert(ChrPairAIXs.end(), ptrStart, ptrEnd);

	BundleChrPair(In, ChrIdA, PrevChrIdB, ChrPairAIXs, Out);

	BundledCount += (unsigned) ChrPairAIXs.size();

	if (BundledCount != AICount)
		Die("Bundled count = %u, != AICount = %u", BundledCount, AICount);
	}
