#include "utils.h"
#include "align_params.h"
#include "progress.h"

#define TRACE	0

static unsigned MaxIndelLength(unsigned LAln, unsigned LA, unsigned LB)
	{
	if (LAln < LA || LAln < LB || LAln > 2*LA || LAln > 2*LB)
		{
		if (LAln != 0)
			Warning("SP specifies invalid alignment length %u, LA=%u, LB=%u",
			  LAln, LA, LB);
		return std::min(LA, LB);
		}

// Number of deletes in alignment
	unsigned DeleteCount = LAln - LB;

// Number of insertions in alignment
	unsigned InsertCount = LAln - LA;

// Max possible indel is all deletes or all inserts in one gap.
	unsigned MIL = std::max(DeleteCount, InsertCount);
#if	TRACE
	Log("MaxIndelLength(LAln=%u, LA=%u, LB=%u) = %u, DelCount=%u, InsertCount=%u\n",
	  LAln, LA, LB, DeleteCount, InsertCount);
#endif
	return MIL;
	}

unsigned CalcRadius(coord_t Length1, coord_t Length2, coord_t AlnLength)
	{
#if	TRACE
	Log("\n");
	Log("CalcRadius(Length1=%u, Length2=%u, AlnLength=%u\n",
	  (unsigned) Length1, (unsigned) Length2, (unsigned) AlnLength);
#endif
	if (Length1 == 0)
		return 1;

// Compute unbanded area
	coord_t UnbandedArea = Length1*Length2;

#if	TRACE
	Log("UnbandedArea=%u Length1=%u Length2=%u\n", UnbandedArea, Length1, Length2);
#endif

// Don't band unless needed
	if (UnbandedArea <= MAXAREA)
		{
		unsigned Radius = Length2/2;
#if	TRACE
		Log("UnbandedArea <= MAXAREA=%u, CalcRadius=%u\n", MAXAREA, Radius);
#endif
		return Radius;
		}

	unsigned MaxIndel = MaxIndelLength(AlnLength, Length1, Length2);
	unsigned Radius = (unsigned) (MaxIndel*g_BandFactor);
	if (Radius > Length2/2)
		{
#if	TRACE
		Log("Radius=%u > Length2=%u/2, Radius = %u\n", Radius, Length2, Length2/2);
#endif
		Radius = Length2/2;
		}

	unsigned Diameter = 2*Radius + 1;
	coord_t BandedArea = Diameter*Length1;
#if	TRACE
	Log("Diameter=2*Radius=%u + 1 = %u\n", Radius, Diameter);
	Log("BandedArea=%u MAXAREA=%u\n", (unsigned) BandedArea, (unsigned) MAXAREA);
#endif
	if (BandedArea > MAXAREA)
		{
		Diameter = MAXAREA/Length1;
		Radius = Diameter/2;
		if (Radius < MINRADIUS)
			Radius = MINRADIUS;
		BandedArea = Diameter*Length1;
#if	TRACE
		Log("BandedArea > MAXAREA, Radius=%u, BandedArea=%u\n",
		  Radius, (unsigned) BandedArea);
#endif
		}

	if (BandedArea > MAXAREA)
		{
		Warning("Area %u exceeds max %u even with banding alnlen %u radius %u maxindel %u bandfactor %g lengths %u, %u",
			(unsigned) BandedArea,
			(unsigned) MAXAREA,
			(unsigned) AlnLength,
			Radius,
			MaxIndel,
			g_BandFactor,
			(unsigned) Length1,
			(unsigned) Length2);
		}

#if	TRACE
	Log("CalcRadius=%u\n", Radius);
#endif
	return Radius;
	}
