#include "utils.h"
#include "intlist.h"
#include "progress.h"

const unsigned NULL_COMP = UINT_MAX;

static void EnumerateComp(const std::vector<IntVec> &AdjMx, IntList &PendingList,
  IntVec &Comps, unsigned CompIndex)
	{
	const unsigned NodeCount = (unsigned) AdjMx.size();

	assert(PendingList.size() > 0);
	unsigned NodeIndex = PendingList.front();
	if (NodeIndex >= NodeCount)
		Die("EnumerateComp[1]");

	PendingList.pop_front();

	const IntVec &ToNodeIndexes = AdjMx[NodeIndex];
	for (CIntVecIter p = ToNodeIndexes.begin(); p != ToNodeIndexes.end(); ++p)
		{
		unsigned ToNodeIndex = *p;
		if (ToNodeIndex >= NodeCount)
			Die("EnumerateComp[2]");
		unsigned ToCompIndex = Comps[ToNodeIndex];
		if (NULL_COMP == ToCompIndex)
			{
			Comps[ToNodeIndex] = CompIndex;
			PendingList.push_back(ToNodeIndex);
			}
		else
			{
			assert(ToCompIndex == CompIndex);
			}
		}
	}

void GetConnectedComponents(const IntVecVec &AdjMx_, IntVecVec &CCs,
  bool ShowProgress)
	{
	const unsigned NodeCount = (unsigned) AdjMx_.size();

// Construct undirected adjacency matrix

	IntVecVec AdjMx;
	for (unsigned NodeIndex = 0; NodeIndex < NodeCount; ++NodeIndex)
		{
		IntVec EmptyVec;
		AdjMx.push_back(EmptyVec);
		}

	for (unsigned NodeIndex = 0; NodeIndex < NodeCount; ++NodeIndex)
		{
		const IntVec &Neighbors = AdjMx_[NodeIndex];
		for_CIntVec(Neighbors, p)
			{
			unsigned ToNodeIndex = *p;
			AdjMx[NodeIndex].push_back(ToNodeIndex);
			AdjMx[ToNodeIndex].push_back(NodeIndex);
			}
		}

// Initialize to NULL_COMP, should be no
// NULL_COMPs left when finished.
	IntVec Comps;
	Comps.reserve(NodeCount);
	for (unsigned NodeIndex = 0; NodeIndex < NodeCount; ++NodeIndex)
		Comps.push_back(NULL_COMP);
	assert(Comps.size() == NodeCount);

	unsigned CompIndex = 0;
	if (ShowProgress)
		ProgressStart("Get CCs, %u nodes", NodeCount);
	for (unsigned NodeIndex = 0; NodeIndex < NodeCount; ++NodeIndex)
		{
		if (ShowProgress)
			{
			if (NodeIndex%1000 == 0)
				ProgressStep(NodeIndex, NodeCount);
			}

		const unsigned ThisCompIndex = Comps[NodeIndex];
		if (NULL_COMP != ThisCompIndex)
			continue;

		IntList PendingList;
		Comps[NodeIndex] = CompIndex;
		PendingList.push_back(NodeIndex);
		while (0 != PendingList.size())
			EnumerateComp(AdjMx, PendingList, Comps, CompIndex);
		++CompIndex;
		}
	if (ShowProgress)
		ProgressDone();

	const unsigned CompCount = CompIndex;
	CCs.clear();
	CCs.resize(CompCount);

	for (unsigned NodeIndex = 0; NodeIndex < NodeCount; ++NodeIndex)
		{
		unsigned CompIndex = Comps[NodeIndex];
		CCs[CompIndex].push_back(NodeIndex);
		}
	}
