#include "chrnameindex.h"
#include "utils.h"

void ChrNameIndex::Add(unsigned ChrIndex, const char *Name)
	{
	assert(m_Indexes.size() == m_Names.size());

	m_Names[ChrIndex] = Name;
	m_Indexes[Name] = ChrIndex;

	assert(m_Indexes.size() == m_Names.size());
	}

unsigned ChrNameIndex::Add(const char *Name)
	{
	assert(m_Indexes.size() == m_Names.size());

	unsigned ChrIndex = (unsigned) m_Names.size();
	for (;;)
		{
		std::map<unsigned, std::string>::const_iterator p = m_Names.find(ChrIndex);
		if (p == m_Names.end())
			break;
		++ChrIndex;
		}

	m_Names[ChrIndex] = Name;
	m_Indexes[Name] = ChrIndex;

	assert(m_Indexes.size() == m_Names.size());

	return ChrIndex;
	}

const char *ChrNameIndex::GetName(unsigned ChrIndex) const
	{
	std::map<unsigned, std::string>::const_iterator p = m_Names.find(ChrIndex);
	if (p == m_Names.end())
		Die("ChrNameIndex::GetName(%u), index not found", ChrIndex);
	const std::map<unsigned, std::string>::value_type &v = *p;
	return (v.second).c_str();
	}

const std::string &ChrNameIndex::GetNameString(unsigned ChrIndex) const
	{
	std::map<unsigned, std::string>::const_iterator p = m_Names.find(ChrIndex);
	if (p == m_Names.end())
		Die("ChrNameIndex::GetName(%u), index not found", ChrIndex);
	const std::map<unsigned, std::string>::value_type &v = *p;
	return v.second;
	}

void ChrNameIndex::ToFile(FILE *f)
	{
	const unsigned Magic = ChrNameIndexMagic;
	WRITEV(Magic);

	off64_t oStartPos = GetStdioFilePos(f);
	const unsigned ChrCount = (unsigned) m_Names.size();
	WRITEV(ChrCount);
	for (std::map<unsigned, std::string>::const_iterator p = m_Names.begin();
	  p != m_Names.end(); p++)
		{
		unsigned ChrIndex = p->first;
		const std::string &strName = p->second;
		WRITEV(ChrIndex);
		const char *Name = strName.c_str();
		char c;
		for (const char *q = Name; (c = *q); ++q)
			{
			WRITEV(c);
			}
		const char nul = '\0';
		WRITEV(nul);
		}

	unsigned uStartPos = (unsigned) oStartPos;
	WRITEV(uStartPos);
	}

void ChrNameIndex::FromFile(FILE *f, bool EatStartPos)
	{
	m_Names.clear();
	m_Indexes.clear();

	unsigned Magic;
	READV(Magic);
	if (Magic != ChrNameIndexMagic)
		Die("ChrNameIndex magic mismatch");

	unsigned Count;
	READV(Count);
	for (unsigned Index = 0; Index < Count; ++Index)
		{
		unsigned ChrIndex;
		READV(ChrIndex);
		std::string strName;
		for (;;)
			{
			char c;
			READV(c);
			if (0 == c)
				break;
			strName += c;
			}
		Add(ChrIndex, strName.c_str());
		}
	
	if (EatStartPos)
		{
		unsigned StartPos;	// @@TODO SHUOLD BE UINT64!!
		READV(StartPos);
		}
	}

bool ChrNameIndex::GetIndex(const char *Name, unsigned *ptrChrIndex) const
	{
	std::map<std::string, unsigned>::const_iterator p = m_Indexes.find(Name);
	if (p == m_Indexes.end())
		return false;
	*ptrChrIndex = p->second;
	return true;
	}

unsigned ChrNameIndex::GetFirstChrIndex() const
	{
	if (m_Names.size() == 0)
		Die("ChrNameIndex::GetFirstChrIndex, size=0");
	const std::map<unsigned, std::string>::value_type &v = *(m_Names.begin());
	return v.first;
	}

unsigned ChrNameIndex::GetNextChrIndex(unsigned ChrIndex) const
	{
	std::map<unsigned, std::string>::const_iterator p = m_Names.find(ChrIndex);
	if (p == m_Names.end())
		Die("ChrNameIndex::GetNextChrIndex(%u), not found", ChrIndex);
	++p;
	if (p == m_Names.end())
		Die("ChrNameIndex::GetNextChrIndex(%u), last index", ChrIndex);
	return p->first;
	}

void ChrNameIndex::LogMe() const
	{
	Log("ChrNameIndex this=0x%x Names.size=%u Indexes.size=%u\n",
	  this, (unsigned) m_Names.size(), m_Indexes.size());
	for (std::map<std::string, unsigned>::const_iterator p = m_Indexes.begin();
	  p != m_Indexes.end(); ++p)
		{
		std::map<std::string, unsigned>::value_type v = *p;
		Log(" [%s]=%u", (v.first).c_str(), v.second);
		}
	Log("\n");
	for (std::map<unsigned, std::string>::const_iterator p = m_Names.begin();
	  p != m_Names.end(); ++p)
		{
		std::map<unsigned, std::string>::value_type v = *p;
		Log(" [%u]=%s", v.first, (v.second).c_str());
		}
	Log("\n");
	}
