#include "chrpairindex.h"
#include "utils.h"
#include <assert.h>

using namespace std;

void ChrPairIndex::FromFile(FILE *f)
	{
	clear();

	unsigned Magic;
	READV(Magic);
	if (Magic != ChrPairIndexMagic)
		Die("ChrPairIndex magic mismatch");

	unsigned PairCount;
	READV(PairCount);
	resize(PairCount);

	for (unsigned PairIndex = 0; PairIndex < PairCount; ++PairIndex)
		{
		ChrPairIndexEntry E;
		READV(E);
		at(PairIndex) = E;
		}

	assert(size() == PairCount);
	}

void ChrPairIndex::ToFile(FILE *f)
	{
	Sort();

	const unsigned Magic = ChrPairIndexMagic;
	WRITEV(Magic);

	unsigned PairCount = (unsigned) size();
	WRITEV(PairCount);
	for (const_iterator p = begin(); p != end(); ++p)
		{
		const ChrPairIndexEntry &E = *p;
		WRITEV(E);
		}
	}

void ChrPairIndex::Add(unsigned ChrIndex1, unsigned ChrIndex2, off_t Pos)
	{
	ChrPairIndexEntry E;
	E.ChrIndex1 = ChrIndex1;
	E.ChrIndex2 = ChrIndex2;
	E.FilePos = Pos;

	push_back(E);
	}

static bool IndexLessThan(const ChrPairIndexEntry &E1, const ChrPairIndexEntry &E2)
	{
	if (E1.ChrIndex1 == E2.ChrIndex1)
		return E1.ChrIndex2 < E2.ChrIndex2;
	return E1.ChrIndex1 < E2.ChrIndex1;
	}

// Return true if chromosome pair is found, false otherwise
// if found, return file position in *ptrPos.
bool ChrPairIndex::Lookup(unsigned ChrIndex1, unsigned ChrIndex2,
  off_t *ptrPos) const
	{
	if (size() == 0)
		return false;

	ChrPairIndexEntry E;
	E.ChrIndex1 = ChrIndex1;
	E.ChrIndex2 = ChrIndex2;

// Binary search of index vector
	ChrPairIndex::const_iterator p = lower_bound(begin(), end(), E, IndexLessThan);

// I don't understand exactly what lower_bound() might return,
// this is defensive and might not be necessary.
	if (p == end())
		return false;

	const ChrPairIndexEntry &E2 = *p;
	if (E.ChrIndex1 != E2.ChrIndex1 || E.ChrIndex2 != E2.ChrIndex2)
		return false;

	*ptrPos = (off_t) E2.FilePos;
	return true;
	}

void ChrPairIndex::Sort()
	{
	sort(begin(), end(), IndexLessThan);
	}
