#include <math.h>
#include "utils.h"
#include "pdf.h"
#include "myrand.h"
#include "evomodel.h"

void GetTruncNormRD(double Mean, double StdDev, RandDist &RD);
void NCESort(vector<double> &Probs);

void GetRandomProbs(unsigned N, double Mean, vector<double> &Probs)
	{
	RandDist RD;
	GetTruncNormRD(Mean, RD);

	Probs.clear();
	for (unsigned i = 0; i < N; ++i)
		Probs.push_back(RD.Get());
	}

static void LogBins(const vector<double> &v)
	{
	vector<unsigned> Bins(10);
	for (unsigned i = 0; i < 10; ++i)
		Bins[i] = 0;

	const unsigned N = SIZE(v);
	unsigned MaxCount = 0;
	double Sum = 0;
	double Sum2 = 0;
	for (unsigned i = 0; i < N; ++i)
		{
		double p = v[i];
		if (p > 1.0)
			Die("LogBins: > 1.0");
		Sum += p;
		Sum2 += p*p;
		unsigned k = (unsigned) (p*10 + 0.5);
		if (k >= 10)
			k = 9;
		unsigned Count = Bins[k] + 1;
		Bins[k] = Count;
		if (Count > MaxCount)
			MaxCount = Count;
		}

	double Mean = Sum/N;
	double StdDev = sqrt(Sum2/N - Mean*Mean);

	Log("\n");
	Log("Mean %.4f StdDev %.4f\n", Mean, StdDev);
	for (unsigned i = 0; i < 10; ++i)
		{
		unsigned Count = Bins[i];
		Log("%6.2f- %6.2f  %5u  ", i*0.1, (i+1)*0.1, Count);
		unsigned L = (Count*32)/MaxCount;
		for (unsigned j = 0; j < L; ++j)
			Log("*");
		Log("\n");
		}
	}

void Clump(const vector<double> &Probs, const vector<unsigned> &ClumpLengths,
  vector<vector<double> > &Clumps, bool RandomizeClumpOrder)
	{
	Clumps.clear();

	unsigned ClumpCount = SIZE(ClumpLengths);
	if (ClumpCount == 0)
		Die("No clumps");

	unsigned N = 0;
	for (unsigned i = 0; i < ClumpCount; ++i)
		N += ClumpLengths[i];

	vector<double> v(Probs);
	sort(v.begin(), v.end());

// Divide by 200 not 100 because two bases for each swap
	unsigned SwapCount = (unsigned) (N*P_ClumpShufflePct/200.0);
	for (unsigned i = 0; i < SwapCount; ++i)
		{
		unsigned k1 = RandInt(N);
		unsigned k2 = RandInt(N);
		swap(v[k1], v[k2]);
		}

	vector<unsigned> ClumpOrder;
	for (unsigned i = 0; i < ClumpCount; ++i)
		ClumpOrder.push_back(i);

	if (RandomizeClumpOrder)
		RandomizeVec(ClumpOrder);

	Clumps.resize(ClumpCount);

	unsigned k = 0;
	for (unsigned i = 0; i < ClumpCount; ++i)
		{
		unsigned ClumpIndex = ClumpOrder[i];
		const unsigned L = ClumpLengths[ClumpIndex];
		vector<double> &Clump = Clumps[ClumpIndex];
		for (unsigned j = 0; j < L; ++j)
			Clump.push_back(v[k++]);

		RandomizeVec(Clump);
		}
	asserta(k == N);
	}

void TestClump()
	{
	InitRand();

	vector<unsigned> ClumpLengths;
	for (unsigned i = 0; i < 8; ++i)
		ClumpLengths.push_back(32);
	unsigned ClumpCount = SIZE(ClumpLengths);

	unsigned N = 0;
	for (unsigned i = 0; i < ClumpCount; ++i)
		N += ClumpLengths[i];

	double Mean = 0.5;
	vector<double> Probs;
	GetRandomProbs(N, Mean, Probs);

	vector<vector<double> > Clumps;
	Clump(Probs, ClumpLengths, Clumps, false);

	Log("\n");
	Log("Probs:");
	LogBins(Probs);

	for (unsigned i = 0; i < ClumpCount; ++i)
		{
		Log("\n");
		Log("Clump %2u L %u:", i, ClumpLengths[i]);
		LogBins(Clumps[i]);
		}

	NCESort(Probs);
	}
