#include "revfile.h"

static std::vector<GenInfo> g_Genomes;
static IntVecVec g_FileGenIndexToCombinedVec;
static IntVecVec g_CombinedGenIndexToFileVec;

void LogCombinedGenomes()
	{
	Log("\n");
	const unsigned CommonGenCount = SIZE(g_Genomes);
	Log("Common index  Name\n");
	Log("============  ====\n");
	FORIX(CommonGen)
		{
		const GenInfo &GI = g_Genomes[CommonGenIndex];
		Log("%12u  %s\n", CommonGenIndex, GI.GetName().c_str());
		}
	Log("\n");
	}

static unsigned GetCombinedIndex(const std::string &Name)
	{
	const unsigned GenCount = SIZE(g_Genomes);
	FORIX(Gen)
		{
		if (g_Genomes[GenIndex].GetName() == Name)
			return GenIndex;
		}
	return UINT_MAX;
	}

static unsigned AddNew(const GenInfo &GI)
	{
	unsigned CombinedGenIndex = SIZE(g_Genomes);
	g_Genomes.push_back(GI);
	return CombinedGenIndex;
	}

const std::vector<GenInfo> &CombineGenomes(
  const std::vector<const RevFile *> &RevFiles)
	{
	g_Genomes.clear();
	g_FileGenIndexToCombinedVec.clear();
	g_CombinedGenIndexToFileVec.clear();

	const unsigned FileCount = SIZE(RevFiles);

	g_FileGenIndexToCombinedVec.resize(FileCount);
	g_CombinedGenIndexToFileVec.resize(FileCount);

	FORIX(File)
		{
		const RevFile &Rev = *RevFiles[FileIndex];
		const std::vector<GenInfo> &Genomes = Rev.GetGenomes();
		const unsigned GenCount = SIZE(Genomes);
		IntVec &FileGenIndexToCombined = g_FileGenIndexToCombinedVec[FileIndex];
		FileGenIndexToCombined.resize(GenCount, UINT_MAX);
		for (unsigned FileGenIndex = 0; FileGenIndex < GenCount; ++FileGenIndex)
			{
			const GenInfo &GI = Genomes[FileGenIndex];
			unsigned CombinedGenIndex = GetCombinedIndex(GI.GetName());
			if (CombinedGenIndex != UINT_MAX)
				{
				if (!(GI == g_Genomes[CombinedGenIndex]))
					Die("Genome %s differs between files", GI.GetName().c_str());
				}
			else
				CombinedGenIndex = AddNew(GI);
			FileGenIndexToCombined[FileGenIndex] = CombinedGenIndex;
			}
		}

	const unsigned CombinedGenCount = SIZE(g_Genomes);
	FORIX(File)
		{
		const RevFile &Rev = *RevFiles[FileIndex];

		IntVec &CombinedGenIndexToFile = g_CombinedGenIndexToFileVec[FileIndex];
		IntVec &FileGenIndexToCombined = g_FileGenIndexToCombinedVec[FileIndex];

		CombinedGenIndexToFile.resize(CombinedGenCount, UINT_MAX);

		const std::vector<GenInfo> &Genomes = Rev.GetGenomes();
		const unsigned GenCount = SIZE(Genomes);
		for (unsigned FileGenIndex = 0; FileGenIndex < GenCount; ++FileGenIndex)
			{
			unsigned CombinedGenIndex = FileGenIndexToCombined[FileGenIndex];
			CombinedGenIndexToFile[CombinedGenIndex] = FileGenIndex;
			}
		}
	return g_Genomes;
	}

unsigned CombinedGenIndexToFile(unsigned FileIndex, unsigned CombinedGenIndex)
	{
	unsigned FileCount = SIZE(g_FileGenIndexToCombinedVec);
	asserta(FileCount == SIZE(g_CombinedGenIndexToFileVec));

	const IntVec &CombinedGenIndexToFile = g_CombinedGenIndexToFileVec[FileIndex];
	unsigned CombinedGenCount = SIZE(CombinedGenIndexToFile);
	asserta(CombinedGenIndex < CombinedGenCount);

	return CombinedGenIndexToFile[CombinedGenIndex];
	}

unsigned FileGenIndexToCombined(unsigned FileIndex, unsigned FileGenIndex)
	{
	unsigned FileCount = SIZE(g_FileGenIndexToCombinedVec);
	asserta(FileCount == SIZE(g_CombinedGenIndexToFileVec));

	const IntVec &FileGenIndexToCombined = g_FileGenIndexToCombinedVec[FileIndex];
	unsigned FileGenCount = SIZE(FileGenIndexToCombined);
	asserta(FileGenIndex < FileGenCount);

	return FileGenIndexToCombined[FileGenIndex];
	}

void GetCommonGenomes(unsigned FileIndexA, unsigned FileIndexB,
  IntVec &CombinedGenIndexes)
	{
	CombinedGenIndexes.clear();

	unsigned FileCount = SIZE(g_FileGenIndexToCombinedVec);
	asserta(FileCount == SIZE(g_CombinedGenIndexToFileVec));
	asserta(FileIndexA < FileCount);
	asserta(FileIndexB < FileCount);

	const IntVec &FileGenIndexToCombinedA = g_FileGenIndexToCombinedVec[FileIndexA];
	const IntVec &FileGenIndexToCombinedB = g_FileGenIndexToCombinedVec[FileIndexB];

	unsigned CountA = SIZE(FileGenIndexToCombinedA);
	unsigned CountB = SIZE(FileGenIndexToCombinedB);

	for (unsigned a = 0; a < CountA; ++a)
		{
		unsigned CombinedGenIndexA = FileGenIndexToCombinedA[a];
		for (unsigned b = 0; b < CountB; ++b)
			{
			if (CombinedGenIndexA == FileGenIndexToCombinedB[b])
				CombinedGenIndexes.push_back(CombinedGenIndexA);
			}
		}
	}

void ConvertGenIndexesToCombined(AlnInfo &AI, unsigned FileIndex)
	{
	assert(FileIndex < SIZE(g_FileGenIndexToCombinedVec));
	const IntVec &FileGenIndexToCombined = g_FileGenIndexToCombinedVec[FileIndex];
	const unsigned FileGenCount = SIZE(FileGenIndexToCombined);
	const unsigned SegCount = SIZE(AI.Segs);
	FORIX(Seg)
		{
		GCSeg &Seg = AI.Segs[SegIndex];
		unsigned FileGenIndex = Seg.GenomeIndex;
		asserta(FileGenIndex < FileGenCount);
		Seg.GenomeIndex = FileGenIndexToCombined[FileGenIndex];
		}
	}
