#include "utils.h"
#include "revfile.h"
#include "getopts.h"
#include "progress.h"

const unsigned MAX_LINE = 4096;

bool GetSeq(FILE *f, std::string &Seq, std::string &Label);
void TruncLabel(std::string &Label);

static void CopySeqToRev(const char *FASTAFileName, const char *GenomeName,
  RevFile &Rev)
	{
	Progress("File %s, Genome %s", FASTAFileName, GenomeName);

	FILE *f = OpenStdioFile(FASTAFileName);

	std::string Seq;
	std::string Label;
	unsigned ChrId = 0;
	while (GetSeq(f, Seq, Label))
		{
		coord_t Length = (coord_t) Seq.size();
		if ((size_t) Length != Seq.size())
			Die("Seq length overflowed size_t");

		TruncLabel(Label);

		Progress("Adding %s  Chr [%5u]  Size [%10u] %s",
		  GenomeName, ChrId, Length, Label.c_str());

		Rev.AddSeq(GenomeName, Label, Seq.c_str(), Length);

		++ChrId;
		}

	CloseStdioFile(f);
	}

static void FromText(const char *TextFileName, RevFile &Rev)
	{
	FILE *f = OpenStdioFile(TextFileName);

	char Line[MAX_LINE];
	for (;;)
		{
		bool EndOfFile = ReadLineStdioFile(f, Line, sizeof(Line));
		if (EndOfFile)
			break;

		char GenomeName[MAX_LINE];
		char FASTAFileName[MAX_LINE];

		int n = sscanf(Line, "%s %s", GenomeName, FASTAFileName);
		if (n != 2)
			Die("Expected 2 fields, got: %s", Line);

		CopySeqToRev(FASTAFileName, GenomeName, Rev);
		}
	}

void CvtFASTA2Rev()
	{
	const char *FASTAFileName = ValueOpt("fromfasta");
	const char *FASTATextFileName = ValueOpt("fromfastafiles");
	const char *RevFileName = RequiredValueOpt("torev");

	if (FASTAFileName == 0 && FASTATextFileName == 0)
		Die("Must specify -fromfasta and / or -fromfastafiles");

	RevFile Rev;
	Rev.Create(RevFileName);

	if (FASTAFileName != 0)
		{
		const char *GenomeName = RequiredValueOpt("genome");
		CopySeqToRev(FASTAFileName, GenomeName, Rev);
		}

	if (FASTATextFileName != 0)
		FromText(FASTATextFileName, Rev);

	Rev.Close();
	}
