#include <stdio.h>
#include <string>
#include "utils.h"
#include "revfile.h"
#include "progress.h"
#include "getopts.h"

#define BUFFER_SIZE 65535

extern bool GetSeq(FILE *f, std::string &Seq, std::string &Label);

void Mfa2Rev(const char *GenomeName, const char *MfaFileName, const char *RevFileName)
{
	FILE *f = OpenStdioFile(MfaFileName);

	RevFile OutFile;
	OutFile.Create(RevFileName);
	
	off64_t TotalSize = GetStdioFileSize(f);

	std::string Seq;
	std::string Label;
	unsigned ChrId = 0;
	Progress("Converting file '%s' to file '%s' with genome name '%s'", MfaFileName, RevFileName, GenomeName);
	while (GetSeq(f, Seq, Label))
	{
		coord_t Length = (coord_t) Seq.size();
		if ((size_t) Length != Seq.size())
			Die("Seq length overflowed size_t");

		Progress("[%5.1f%%] Adding %s  Chr [%5u]  Size [%10u] %s",
		  GetStdioFilePos(f)*100.0/TotalSize,
		  GenomeName, ChrId, Length, Label.c_str());

		OutFile.AddSeq(GenomeName, Label, Seq.c_str(), Length);

		++ChrId;
		}

	CloseStdioFile(f);
	OutFile.Close();
}

void CvtMFA2Rev()
{
	const char *GenomeName = RequiredValueOpt("genome");
	const char *MfaFileName = RequiredValueOpt("frommfa");
	const char *RevFileName = RequiredValueOpt("torev");

	Mfa2Rev(GenomeName, MfaFileName, RevFileName);
}

