#include "revfile.h"
#include "utils.h"
#include "gff.h"
#include "getopts.h"
#include "progress.h"

void CvtRevGFF2FASTA()
	{
	const char *RevFileName = RequiredValueOpt("fromrev");
	const char *GFFFileName = RequiredValueOpt("gff");
	const char *FASTAFileName = RequiredValueOpt("tofasta");
	const char *GenomeName = RequiredValueOpt("genome");

	RevFile In;
	In.Open(RevFileName);

	unsigned GenIndex = In.GetGenomeIndex(GenomeName);

	FILE *fGFF = OpenStdioFile(GFFFileName);
	FILE *fFASTA = CreateStdioFile(FASTAFileName);

	GFFRecord Rec;
	unsigned RecordCount = 0;
	ProgressStart("Processing");
	off64_t FileSize = GetStdioFileSize(fGFF);
	while (GetNextGFFRecord(fGFF, Rec))
		{
		if ((RecordCount++)%100 == 0)
			{
			off64_t FilePos = GetStdioFilePos(fGFF);
			double Fract = (double) FilePos / (double) FileSize;
			unsigned Mills = (unsigned) (Fract*10000);
			ProgressStep(Mills, 10000);
			}

		const std::string ChrName(Rec.SeqName);
		unsigned ChrId = In.GetChrId(GenIndex, ChrName);

		asserta(Rec.Start > 0);
		asserta(Rec.End >= Rec.Start);

		coord_t lo = Rec.Start - 1;
		coord_t hi = Rec.End - 1;
		unsigned Length = (unsigned) (hi - lo + 1);
		unsigned From = (unsigned) (lo - 1);

		char *Seq = new char[Length];
		In.GetSubSeq(GenIndex, ChrId, Seq, From, Length);

		fprintf(fFASTA, ">%s;%"COORD_T_PRINTF"-%"COORD_T_PRINTF";%s;%c;%s\n",
		  Rec.SeqName,
		  lo,
		  hi,
		  Rec.Feature,
		  Rec.Strand,
		  Rec.Attrs);

		const unsigned BlockLength = 60;
		char Line[BlockLength+1];
		unsigned BlockCount = (Length + BlockLength - 1)/BlockLength;
		unsigned LettersLeft = Length;
		for (unsigned i = 0; i < BlockCount; ++i)
			{
			unsigned LettersToWrite = LettersLeft;
			if (LettersToWrite > BlockLength)
				LettersToWrite = BlockLength;
			memcpy(Line, Seq + i*BlockLength, LettersToWrite);
			Line[LettersToWrite] = 0;
			fprintf(fFASTA, "%s\n", Line);
			LettersLeft -= LettersToWrite;
			}
		delete[] Seq;
		}

	In.Close();
	CloseStdioFile(fGFF);
	CloseStdioFile(fFASTA);
	}
