#include "utils.h"
#include "revfile.h"
#include "getopts.h"

#define ReadData(v)		ReadStdioFile(f, &v, (unsigned) sizeof(v))

void DumpRev()
	{
	std::string RevFileName = RequiredValueOpt("dumprev");
	g_LogOnly = FlagOpt("logonly");

	FILE *f = OpenStdioFile(RevFileName.c_str());

	RevFileHdr Hdr;
	ReadData(Hdr);

	Dump("\n");
	Dump("Header:\n");
	Dump("Magic          %0x\n", (unsigned) Hdr.Magic);
	Dump("GenomeCount    %s\n", uint32tostr(Hdr.GenomeCount));
	Dump("AlnInfoCount   %s\n", uint32tostr(Hdr.AlnInfoCount));
	Dump("AlnCount       %s (non-compressed only)\n", uint32tostr(Hdr.AlnCount));
	Dump("GenomesPos     %s\n", uint64tostr(Hdr.GenomesPos));
	Dump("AlnInfoTOCPos  %s\n", uint64tostr(Hdr.AlnInfoTOCPos));
	Dump("AlnTOCPos      %s\n", uint64tostr(Hdr.AlnTOCPos));
	Dump("HashTablePos   %s\n", uint64tostr(Hdr.HashTablePos));
	Dump("HashTableBytes %s\n", uint32tostr(Hdr.HashTableBytes));
	Dump("HashTableSize  %s\n", uint32tostr(Hdr.HashTableEntryCount));
	Dump("SeqCount       %s\n", uint64tostr(Hdr.SeqCount));
	Dump("SeqTOCPos      %s\n", uint64tostr(Hdr.SeqTOCPos));
	Dump("EndMagic       %0x\n", (unsigned) Hdr.EndHdrMagic);

	SeekStdioFile(f, (off64_t) Hdr.GenomesPos);

	const unsigned GenomeCount = (unsigned) Hdr.GenomeCount;
	for (unsigned GenomeIndex = 0; GenomeIndex < GenomeCount; ++GenomeIndex)
		{
		off64_t Pos = GetStdioFilePos(f);

		unsigned StartMagic;
		ReadData(StartMagic);

		Dump("\n");
		Dump("\n");
		Dump("Genome %u Pos %u\n", GenomeIndex, (unsigned) Pos);
		Dump("StartMagic  %0x\n", StartMagic);
		if (StartMagic != GenInfoStartMagic)
			Dump(" --warning-- magic should be 0x%x\n", GenInfoStartMagic);

		unsigned NameLength;
		ReadData(NameLength);
		std::string Name;
		for (unsigned i = 0; i < NameLength; ++i)
			{
			char c;
			ReadData(c);
			Name.push_back(c);
			}

		Dump("Name %s\n", Name.c_str());
		unsigned ChrCount;
		ReadData(ChrCount);
		Dump("ChrCount %u\n", ChrCount);

		for (unsigned ChrId = 0; ChrId < ChrCount; ++ChrId)
			{
			off64_t Pos = GetStdioFilePos(f);

			unsigned Id;
			coord_t Length;
			ReadData(Id);
			ReadData(Length);

			unsigned NameLength;
			ReadData(NameLength);
			std::string Name;
			for (unsigned i = 0; i < NameLength; ++i)
				{
				char c;
				ReadData(c);
				Name.push_back(c);
				}
			Dump("  Pos %5u  Chr %3u  Id %5u  NameLength %3u  Length %10u  Name %s\n",
			  (unsigned) Pos, ChrId, Id, NameLength, (unsigned) Length, Name.c_str());
			}

		Pos = GetStdioFilePos(f);

		unsigned EndMagic;
		ReadData(EndMagic);
		Dump("EndMagic  %0x  Pos %u\n", EndMagic, (unsigned) Pos);
		if (EndMagic != GenInfoEndMagic)
			Dump("--warning-- magic should be 0x%x\n", GenInfoEndMagic);
		}

	Dump("\n");
	Dump("AlnInfoTOC:\n");
	Dump("       Pos  Index  AlnIndex  Seg  Att      -> Pos\n");
	Dump("==========  =====  ========  ===  ===  ==========\n");
	SeekStdioFile(f, (off64_t) Hdr.AlnInfoTOCPos);
	const unsigned AlnInfoCount = Hdr.AlnInfoCount;
	for (unsigned AIIndex = 0; AIIndex < AlnInfoCount; ++AIIndex)
		{
		unsigned AtPos = (unsigned) GetStdioFilePos(f);
		DiskAlnInfoTOCEntry AITE;
		ReadData(AITE);
		DiskAlnInfoTOCEntryC &AITEC = (DiskAlnInfoTOCEntryC &) AITE;

		Dump("%10u  %5u", AtPos, AIIndex);
		if (AITEC.BitFields & BIT_AI_IS_COMPRESSED)
			{
			Dump(" chr0=%u chr1=%u len0=%u dlen1=%d bits=%04x\n",
			  AITEC.ChrId0, AITEC.ChrId1, AITEC.Length0, AITEC.DiffLength1, AITEC.BitFields);
			continue;
			}

		if (AITE.AlnIndex == UINT_MAX)
			Dump("         -");
		else
			Dump("  %8u", AITE.AlnIndex);
		Dump("  %3u  %3u  %10s\n",
		  AITE.SegCount, AITE.AttrCount, uint64tostr(AITE.FilePos));
		}

	Dump("\n");
	Dump("AlnInfos:\n");
	for (unsigned AIIndex = 0; AIIndex < AlnInfoCount; ++AIIndex)
		{
		off64_t TOCPos = (off64_t) Hdr.AlnInfoTOCPos + AIIndex*sizeof(DiskAlnTOCEntry);
		SeekStdioFile(f, TOCPos);

		DiskAlnInfoTOCEntry DATE;
		ReadData(DATE);

		Dump("\n");
		Dump(
"Ix=%5u  TOCPos=%7u  FilePos=%7u  AlnIndex=%5u  AttrCount=%1u  SegCount=%1u Score=%11.6g\n",
		  AIIndex, (unsigned) TOCPos, (unsigned) DATE.FilePos, DATE.AlnIndex,
		  DATE.AttrCount, DATE.SegCount, DATE.Score);

		SeekStdioFile(f, (off64_t) DATE.FilePos);

		for (unsigned SegIndex = 0; SegIndex < DATE.SegCount; ++SegIndex)
			{
			DiskGCSeg D;
			ReadStdioFile(f, &D, sizeof(D));
			Dump("  Gen=%u Chr=%u lo=%u hi=%u %c\n",
			  D.GenomeIndex, D.ChrId, (unsigned) D.lo, (unsigned) D.hi, D.Plus ? '+' : '-');
			}

		for (unsigned AttrIndex = 0; AttrIndex < DATE.AttrCount; ++AttrIndex)
			{
			DiskAlnAttr D;
			ReadStdioFile(f, &D, sizeof(D));
			Dump("  Attr=%u Value=%u", (unsigned) D.Attr, (unsigned) D.Value);
			}
		if (DATE.AttrCount > 0)
			Dump("\n");
		}

	Dump("\n");
	Dump("AlnTOC:\n");
	Dump("       Pos  Index  AIIndex   Rows   Cols     -> Pos\n");
	Dump("==========  =====  =======  ===== ======  ==========\n");
	SeekStdioFile(f, (off64_t) Hdr.AlnTOCPos);
	const unsigned AlnCount = Hdr.AlnCount;
	for (unsigned AlnIndex = 0; AlnIndex < AlnCount; ++AlnIndex)
		{
		unsigned AtPos = (unsigned) GetStdioFilePos(f);
		DiskAlnTOCEntry ATE;
		ReadData(ATE);
		Dump("%10u  %5u  %7u  %6u  %4u  %10s\n",
		  AtPos, AlnIndex, ATE.AlnInfoIndex, ATE.RowCount,
		  ATE.ColCount, uint64tostr(ATE.FilePos));
		}

	Dump("\n");
	Dump("Alns:\n");
	for (unsigned AlnIndex = 0; AlnIndex < AlnCount; ++AlnIndex)
		{
		off64_t TOCPos = (off64_t) Hdr.AlnTOCPos + AlnIndex*sizeof(DiskAlnTOCEntry);
		SeekStdioFile(f, TOCPos);

		DiskAlnTOCEntry DATE;
		ReadData(DATE);
		SeekStdioFile(f, (off64_t) DATE.FilePos);

		Dump("\n");
		Dump("TOCPos=%u Pos=%u\n", (unsigned) TOCPos, (unsigned) DATE.FilePos);
		unsigned Cols = DATE.ColCount;
		if (Cols > 100)
			Cols = 100;

		char *Row = myalloc(char, DATE.ColCount);
		for (unsigned RowIndex = 0; RowIndex < DATE.RowCount; ++RowIndex)
			{
			ReadStdioFile(f, Row, DATE.ColCount);
			for (unsigned ColIndex = 0; ColIndex < Cols; ++ColIndex)
				Dump("%c", Row[ColIndex]);
			if (DATE.ColCount > 100)
				Dump("...");
			Dump("\n");
			}
		myfree(Row);
		}

	Dump("\n");
	Dump("Hash table:\n");
	SeekStdioFile(f, (off64_t) Hdr.HashTablePos);
	if (Hdr.HashTableBytes != Hdr.HashTableEntryCount*sizeof(HashTableEntry))
		Die("Hash table size conflict");

	HashTableEntry *Hash = myalloc(HashTableEntry, Hdr.HashTableEntryCount);
	ReadStdioFile(f, (off64_t) Hdr.HashTablePos, Hash, Hdr.HashTableBytes);

	Dump("  Entry    Gen      Chr         Pos    Count\n");
	Dump("=======  =====  =======  ==========  =======\n");
	for (unsigned TableIndex = 0; TableIndex < Hdr.HashTableEntryCount; ++TableIndex)
		{
		const HashTableEntry *H = Hash + TableIndex;
		if (H->GenomeIndex == NullGenomeIndex)
			Dump("%7u      -\n", TableIndex);
		else
			Dump("%7u  %5u  %7u  %10u  %7u\n",
			  TableIndex, H->GenomeIndex, H->ChrId, (unsigned) H->FileOffset, H->Count);
		}

	Dump("\n");
	Dump("AlnIdLists:\n");

	for (unsigned TableIndex = 0; TableIndex < Hdr.HashTableEntryCount; ++TableIndex)
		{
		const HashTableEntry *H = Hash + TableIndex;
		if (H->GenomeIndex == NullGenomeIndex)
			continue;
		Dump("\n");
		Dump("Gen=%u,Chr=%u Pos=%u AlnIds=", H->GenomeIndex, H->ChrId,
		  (unsigned) H->FileOffset);

		SeekStdioFile(f, (off64_t) H->FileOffset);
#if	OLD_REV_FILE
		for (unsigned i = 0; i < H->AlnInfoCount; ++i)
			{
			if (i%16 == 0)
				Dump("\n");
			unsigned AlnId;
			ReadData(AlnId);
			Dump(" %7u", AlnId);
			}
		Dump("\n");
#else
		for (unsigned i = 0; i < H->Count; ++i)
			{
			if (i%16 == 0)
				Dump("\n");
			uint32 Lo;
			uint32 Hi;
			ReadData(Lo);
			ReadData(Hi);
			Dump(" %7u-%7u", Lo, Hi);
			}
		Dump("\n");
#endif
		}

	Dump("\n");
	Dump("SeqTOC:\n");
	Dump("\n");
	Dump(" TOCEntPos  Gen      Chr      SeqPos      Length  Seq\n");
	Dump("==========  ===  =======  ==========  ==========  ===\n");

	off64_t TOCPos = (off64_t) Hdr.SeqTOCPos;
	for (unsigned TableIndex = 0; TableIndex < Hdr.SeqCount; ++TableIndex)
		{
		DiskSeqTOCEntry TE;

		off64_t TEPos = TOCPos + TableIndex*sizeof(TE);
		ReadStdioFile(f, TEPos, &TE, sizeof(TE));

		unsigned Length = (unsigned) TE.Length;
		off64_t SeqPos = (off64_t) TE.FilePos;

		Dump("%10u  %3u  %7u  %10u  %10u  ",
		  TEPos,
		  TE.GenIndex,
		  TE.ChrId,
		  (unsigned) SeqPos,
		  Length);

		unsigned ShowLength = Length;
		if (ShowLength > 16)
			ShowLength = 16;

		char Seq[32];
		ReadStdioFile(f, SeqPos, Seq, ShowLength);

		for (unsigned i = 0; i < ShowLength; ++i)
			Dump("%c", Seq[i]);

		if (ShowLength < Length)
			{
			Dump("...");
			unsigned EndLength = (Length >= 16 ? 16 : Length);
			unsigned EndPos = Length - EndLength;
			ReadStdioFile(f, SeqPos + EndPos, Seq, EndLength);
			for (unsigned i = 0; i < EndLength; ++i)
				Dump("%c", Seq[i]);
			}

		Dump("\n");
		}

	myfree(Hash);
	Hash = 0;
	}
