#include "utils.h"
#include "masktab.h"
#include "getopts.h"
#include "xlat.h"

void DumpTab()
	{
	const char *TabFileName = RequiredValueOpt("dumptab");
	const char *OutFileName = RequiredValueOpt("out");
	bool All = FlagOpt("all");

	MaskTable MT;
	MT.FromFile(TabFileName);

	ALPHA Alpha = MT.GetAlpha();
	unsigned WordSize = MT.GetWordSize();
	uint64 WordCount = CalcWordCount(Alpha, WordSize);

	FILE *fOut = CreateStdioFile(OutFileName);
	for (uint64 Word = 0; Word < WordCount; ++Word)
		{
		bool Mask = MT.GetBit(Word);
		if (!Mask && !All)
			continue;
		const char *Str = 0;
		if (Alpha == ALPHA_Nucleo)
			Str = NucWordToStr(Word, WordSize);
		else if (Alpha == ALPHA_Amino)
			Str = AminoWordToStr(Word, WordSize);
		else
			Die("Alphabet not supported %d", Alpha);
		if (All)
			fprintf(fOut, "%s %s\n", Str, Mask ? "T" : "F");
		else
			fprintf(fOut, "%s\n", Str);
		}
	CloseStdioFile(fOut);
	}
