#ifndef eventresult_h
#define eventresult_h

#include "event.h"

enum EVENT_RESULT
	{
	ER_Fail,
	ER_Reject,
	ER_Accept
	};

#define DEF_FAIL(EventName, Operation)		ERF_##EventName##_##Operation,
#define DEF_REJECT(EventName, Mutation)		ERR_##EventName##_##Mutation,
#define DEF_ACCEPT(EventName)				ERA_##EventName,

enum ER_TYPE
	{
#include "eventresults.h"
	};

#define DEF_FAIL(EventName, Operation)		 + 1
#define DEF_REJECT(EventName, Mutation)		 + 1
#define DEF_ACCEPT(EventName)				 + 1

const unsigned EventResultTypeCount = 
#include "eventresults.h"
	;

extern unsigned g_ERTCounts[EventResultTypeCount];

void IncEventLengthDist(EVENT_TYPE Event, gpos_t Length);

class RateMgr;
#define E(e) extern RateMgr *g_##e##RateMgr;
#include "events.h"

#define EV_FAIL(EventName, Length, Operation)				\
	{														\
	g_##EventName##RateMgr->StoreResult(ER_Fail, Length);	\
	++(g_ERTCounts[ERF_##EventName##_##Operation]);			\
	return ER_Fail;											\
	}

#define EV_REJECT(EventName, Length, Mutation)				\
	{														\
	g_##EventName##RateMgr->StoreResult(ER_Reject, Length);	\
	++(g_ERTCounts[ERR_##EventName##_##Mutation]);			\
	if (m_LogEvents)										\
		Log("@EV REJECT %10u %16.16s %10.10s %10"PF"\n",	\
		  m_EventIndex, #EventName, #Mutation,  Length);	\
	return ER_Reject;										\
	}

#define EV_ACCEPT(EventName, Pos, Length)					\
	{														\
	g_##EventName##RateMgr->StoreResult(ER_Accept, Length);	\
	++(g_ERTCounts[ERA_##EventName]);						\
	m_LastAcceptedEvent = EVENT_TYPE_##EventName;			\
	m_LastAcceptedEventPos = Pos;							\
	if (m_LogEvents)										\
		Log("@EV ACCEPT %10u %16.16s %10"PF" %10"PF" %10"PF"\n",	\
		  m_EventIndex, #EventName, Pos, Length, GetEvolvedLength());			\
	return ER_Accept;										\
	}

// No-length versions
#define EV_FAILN(EventName, Operation)						\
	{														\
	++(g_ERTCounts[ERF_##EventName##_##Operation]);			\
	return ER_Fail;											\
	}

#define EV_REJECTN(EventName, Mutation)						\
	{														\
	++(g_ERTCounts[ERR_##EventName##_##Mutation]);			\
	if (m_LogEvents)										\
		Log("@EV REJECT %10u %16.16s %16.16s\n",			\
		  m_EventIndex, #EventName, #Mutation);				\
	return ER_Reject;										\
	}

#define EV_ACCEPTN(EventName, Pos)							\
	{														\
	++(g_ERTCounts[ERA_##EventName]);						\
	m_LastAcceptedEvent = EVENT_TYPE_##EventName;			\
	m_LastAcceptedEventPos = Pos;							\
	if (m_LogEvents)										\
		Log("@EV ACCEPT %10u %16.16s %10"PF"            %10"PF"\n",	\
		  m_EventIndex, #EventName, Pos, GetEvolvedLength());\
	return ER_Accept;										\
	}

#define EV_GETLENGTH(EventName, LengthVar)					\
	gpos_t LengthVar = g_##EventName##RateMgr->GetLength();

#endif // eventresult_h
