#ifndef evo_h
#define	evo_h

#define	LOGMBD		0

#if	defined(DEBUG) && !defined(_DEBUG)
#define	_DEBUG	1
#endif

#if	defined(_DEBUG) && !defined(DEBUG)
#define	DEBUG	1
#endif

#if	defined(_MSC_VER)
#pragma warning(disable: 4800)	// int->bool conversion
#endif

//#define	_SECURE_SCL					0
//#define	_HAS_ITERATOR_DEBUGGING		0

#include <math.h>
#include <list>
#include <vector>
#include <set>
#include <string>
#include "utils.h"
#include "getopts.h"
#include "eventresult.h"
#include "event.h"
#include "evolver.h"
#include "progress.h"
#include "intlist.h"
#include "revfile.h"
#include "codon.h"
#include "geninfo.h"
#include "evoparams.h"
#include "readfasta.h"
#include "ratemgr.h"
#include "myrand.h"

struct MEInfo;

using namespace std;

const gpos_t ONE_HUNDRED = 100;
const gpos_t ONE_THOUSAND = 1000;
const gpos_t ONE_MILLION = ONE_THOUSAND*ONE_THOUSAND;
const gpos_t POS_NOT_USED = GPOS_T_MAX;
const gpos_t LENGTH_NOT_USED = GPOS_T_MAX;
const gpos_t USE_MOTIF_LENGTH = GPOS_T_MAX - 1;

#if	!LOGMDB
#define	LogMDB(Pos, Str)	/*empty*/
#endif

uint32 coord64_t_to_uint32(gpos_t Pos);
const char *BaseCountToStr(gpos_t Count);
unsigned GetUngappedSeqLength(const std::string &Seq);
gpos_t GetOverlapLength(gpos_t loA, gpos_t hiA, gpos_t loB, gpos_t hiB);
void Normalize(vector<double> &Vect);
void Normalize(vector<double> &Vect, double Value);
void LogAIRows(const AlnInfo &AI, const StdStrVec &Rows, const RevFile *Rev = 0);
bool ReadEvoParamsLine(FILE *f, string &Line);
void ReadDist(FILE *f, vector<pair<gpos_t, double> > &Rates);
void ReadCCAcceptProbs(FILE *f);
void ReadEventDist(FILE *f, const string &Event);
void LogRateParams();
void LogNCCECWeights();
void LogDists();
void ReadMESeqs(const char *FileName, const vector<MEInfo> &MEs);
void LogMESeqs();
gpos_t GetRandLengthRR(const RateRange &RR);
const string &GetMESeq(const string &Name);
void ValidateSegLengths(const AlnInfo &AI, const StdStrVec &Rows);
void RevComp(const std::string &Seq, std::string &RevCompSeq);
void RevComp(std::string &RevCompSeq);
char CompLetter(char c);
double GetAASubstProb(unsigned AminoIndexBefore, unsigned AminoIndexAfter);
void MakeChrPosMap(const GenInfo &Genome, map<string, gpos_t> &ChrNameToStartPos,
  map<gpos_t, string> &ChrStartPosToName);
void LogChrPosMap(const map<string, gpos_t> &ChrNameToStartPos,
  const map<gpos_t, string> &ChrStartPosToName);
void ReadModel(const char *FileName);
void SetEvoParam(const string &Name, const string &Value);
void ParseStrVec(const std::string &In, std::vector<std::string> &Out, char c);
void LogMatchCounts();
void ParseStrVecWS(const std::string &In, std::vector<std::string> &Out);
double GetMicroInsertAcceptProb(uint8 CC, gpos_t Length);
void PosToChrPos(gpos_t Pos, const map<gpos_t, string> &ChrStartPosToName,
  string &ChrName, gpos_t &ChrPos);
void FixChrLengths(FILE *f);
gpos_t ChrPosToPos(const string &ChrName, gpos_t ChrPos,
  const map<string, gpos_t> &ChrNameToStartPos);
unsigned GetMicrosatMotifLength(const char *Attrs);
void WriteFasta(FILE *f, const char *SeqData, unsigned Length);
void DumpChrNames();
void FASTA2Evo();
void ReadAAMx(FILE *f);
void ReadNucMx(FILE *f);
void LogAAMx();
void LogNucMx();
void LogCCAcceptProbs();
void SetCCName(uint8 CC, const string &Name);
void ClearInvalidCounts();
double ComputeBasesAffected(gpos_t Llo, gpos_t Lhi, double flo, double fhi);
unsigned GetInvalidLetterCount();
void SkipInvalid(unsigned char c);
void WarnInvalids();
bool isacgt(char c);
void SeqTree();
void Macro();
void MicroEvolve();
void EvoToGFF();
void EvoToFASTA();
void DumpEvoHdr();
void TestRand();
void TestMicro();
void TestMacro();
void TestMacroEvolve();
void Unwind();
void EvoToRev();
void UnwindAnnots();
void RandomizeNs();
double AccelProb(double p, double f);
void LogEvoParams();
void SetModel(const char *FileName);
void LogModel();
void LogERCounts(bool DoMicro);
void LogERDetails(bool DoMicro);
void LogLengthCounts();
void GenerateNCCEs();
void SizeOf();
void Evo();

extern bool g_ValidateEvents;
extern bool g_LogEvents;
extern bool g_NoEvents;
extern gpos_t g_WrittenBaseCount;
extern gpos_t g_LeafBaseCount;
extern double g_DurationFactor;
extern bool g_ValAlnSeqs;

#endif	// evo_h
