#include "evolver.h"
#include "intlist.h"
#include "progress.h"

bool isacgt(char c);

void Evolver::Validate(bool ShowProgress) const
	{
	if (ShowProgress)
		Progress("[%u] Validate", m_EventIndex);
	BaseList_Validate();

	gpos_t EvolvedLength = 0;
	FOR_EVBASES(p)
		{
		++EvolvedLength;

		const BaseInfo &Base = GetBase(p);

		char Letter = Base.GetLetter();
		if (!isacgt(Letter) && Letter != '-')
			Die("Invalid letter '%c'", Letter);

		if (Base.IsCoding())
			{
			if (Base.Codex == 0 || Base.Codex < -3 || Base.Codex > 3)
				Die("CDS base pos=%"PF" has Codex=%d",
				  p, Base.Codex);
			}

		if (Base.IsStartCodon() && (Base.Codex == 1 || Base.Codex == -1))
			{
			map<gpos_t, unsigned>::const_iterator q = m_GeneIndexes.find(p);
			if (q == m_GeneIndexes.end())
				Die("Start codon at pos %"PF" not in m_GeneIndexes", p);
			}

		if (Base.IsSplice())
			if (!Base.IsFullyConstrained())
				Die("Splice base Index=%"PF" not fully constrained", p);

		if (Base.IsExon() && !Base.IsConstrained())
			Die("Exon base Index=%"PF" not constrained", p);
		}
	if (EvolvedLength != m_EvolvedLength)
		Die("Evolved length problem, measured %"PF" cached %"PF,
		  EvolvedLength, m_EvolvedLength);

	ValidateGenes(ShowProgress);
	ValidateCDSs();
	}

void Evolver::ValidateCDSs() const
	{
	int8 LastCodex = 0;
	bool LastBaseWasCDS = false;
	FOR_EVBASES(p)
		{
		const BaseInfo &Base = GetBase(p);
		bool IsCoding = Base.IsCoding();
		if (IsCoding)
			{
			if (Base.Codex >= 0)
				asserta(Base.Codex >= 1 && Base.Codex <= 3);
			else
				asserta(Base.Codex <= -1 && Base.Codex >= -3);
			if (LastBaseWasCDS)
				{
				int8 ExpectedCodex = IncCodex(LastCodex);
				if (Base.Codex != ExpectedCodex)
					{
					LogNeighborsAsTable(p, 20);
					Die("Expected codex %d at pos %"PF", got %d",
					  ExpectedCodex, p, Base.Codex);
					}
				}
			LastCodex = Base.Codex;
			}
		LastBaseWasCDS = IsCoding;
		}
	}

void DieWrap(const std::string &Msg)
	{
	Die("%s", Msg.c_str());
	}

void Evolver::PrintAndDie(const char *Format, ...) const
	{
	string Msg;
	va_list ArgList;
	va_start(ArgList, Format);
	VStrPrintf(Msg, Format, ArgList);
	va_end(ArgList);

	if (m_LastAcceptedEvent != EVENT_TYPE_Invalid)
		ProgressNoRes("Event %u, last accepted event %u %s, pos %"PF,
		  m_EventIndex,
		  m_LastAcceptedEventIndex,
		  EventTypeToStr(m_LastAcceptedEvent),
		  m_LastAcceptedEventPos);

	if (m_CurrentEvent != EVENT_TYPE_Invalid)
		{
		if (m_CurrentEventPos == GPOS_T_MAX)
			ProgressNoRes("Current event %u %s",
			  m_EventIndex, EventTypeToStr(m_CurrentEvent));
		else
			ProgressNoRes("Current event %u %s, pos %"PF,
			  m_EventIndex, EventTypeToStr(m_CurrentEvent), m_CurrentEventPos);
		}
	DieWrap(Msg);
	}
