#include "filter4.h"

#define TRACE	0

// See comments in filwrap4.cpp.

void FilterAwareOutputAlnInfo(RevFile &Out, const AlnInfo &UserAI)
	{
	if (!FilterIsInverting())
		{
		Out.AddAlnInfo(UserAI);
		return;
		}

	AlnInfo TrueAI;
	FilterInvertAI(UserAI, TrueAI);

	Out.AddAlnInfo(TrueAI);
	}

void FilterAwareOutputAlnRows(RevFile &Out, const AlnInfo &UserAI,
  const StdStrVec &UserRows)
	{
	if (!FilterIsInverting())
		{
		Out.AddAln(UserAI, UserRows);
		return;
		}

	AlnInfo TrueAI;
	FilterInvertAI(UserAI, TrueAI);

// The rows should be correct, don't need to reverse-complement.
	Out.AddAln(TrueAI, UserRows);
	}

void FilterAwareGetSeqData(const RevFile &Rev, unsigned InGenomeIndex,
  unsigned ChrId, coord_t Pos, unsigned Length, char *Seq, bool Plus)
	{
	const RevFile &In = GetFilterInputRevFile();
	const std::string &GenName = In.GetGenomeName(InGenomeIndex);
	unsigned RevGenomeIndex = Rev.GetGenomeIndex(GenName);

	coord_t TruePos = Pos;
	bool Inv = (FilterIsInverting() && InGenomeIndex == GetFilterInvGenomeIndex());
	if (Inv)
		{
		const ChrInfo &CI = GetFilterChrInfoB();
		coord_t ChrLength = CI.GetLength();
		if (Pos >= ChrLength)
			Die("FilterAwareGetSeqData, Pos >= ChrLength (1)");
		coord_t InvLength = Pos + Length;
		if (InvLength > ChrLength)
			Die("FilterAwareGetSeqData, InvLength > ChrLength");
		TruePos = ChrLength - InvLength;
#if	TRACE
		{
		Log("\n");
		Log("FilterAwareGetSeqData Inv Gen=%u Chr=%u Len=%u Pos=%u ChrLen=%u TruePos=%d\n",
		  InGenomeIndex,
		  ChrId,
		  (unsigned) Pos,
		  Length,
		  (unsigned) ChrLength,
		  (unsigned) TruePos);
		}
#endif
		if (Pos >= ChrLength)
			Die("FilterAwareGetSeqData, Pos >= ChrLength (2)");
		}

#if	TRACE
	{
	Log("\n");
	Log("FilterAwareGetSeqData Gen=%u Chr=%u Pos=%u TruePos=%u Len=%u Inv=%c\n",
	  InGenomeIndex,
	  ChrId,
	  (unsigned) Pos,
	  (unsigned) TruePos,
	  Length,
	  Inv ? 'T' : 'F');
	}
#endif
	Rev.GetSubSeq(RevGenomeIndex, ChrId, Seq, TruePos, Length);
#if	TRACE
	Log("  Seq=%*.*s\n", Length, Length, Seq);
#endif

	bool RC = !Plus;
	if (Inv)
		RC = !RC;

	if (RC)
		{
		RevComp(Seq, Length);
#if	TRACE
		Log("  RevCompSeq=%*.*s\n", Length, Length, Seq);
#endif
		}
	}
