#include <set>
#include "gff2.h"
#include "getopts.h"
#include "ivtree.h"
#include "lohi.h"
#include "progress.h"

coord_t GetOverlapLength(coord_t loA, coord_t hiA, coord_t loB, coord_t hiB);

void FixCpGs()
	{
	const char *InFileName = RequiredValueOpt("fixcpgs");
	const char *CDSFileName = RequiredValueOpt("cds");
	const char *OutFileName = RequiredValueOpt("out");

	FILE *fIn = OpenStdioFile(InFileName);
	FILE *fAnnots = OpenStdioFile(CDSFileName);
	FILE *fOut = CreateStdioFile(OutFileName);

	IvTree<LoHi> CDSIx;
	set<string> InvalidFeatures;

	ProgressStartFile(fAnnots, "Reading %s", CDSFileName);
	unsigned Count = 0;
	GFFRecord2 Rec;
	while (GetNextGFFRecord(fAnnots, Rec))
		{
		if (Count++%100 == 0)
			ProgressStepFile();
		if (Rec.Feature == "CDS")
			{
			LoHi LH;
			LH.Lo = Rec.Start;
			LH.Hi = Rec.End;
			CDSIx.Add(LH);
			}
		}
	ProgressDone();

	if (!InvalidFeatures.empty())
		{
		for (set<string>::const_iterator p = InvalidFeatures.begin();
		  p != InvalidFeatures.end(); ++p)
			Log(" %s", (*p).c_str());
		Log("\n");
		Warning("%u invalid features in %s", SIZE(InvalidFeatures), CDSFileName);
		}

	InvalidFeatures.clear();
	ProgressStartFile(fIn, "Processing");
	Count = 0;
	unsigned DeletedCount = 0;
	while (GetNextGFFRecord(fIn, Rec))
		{
		if (Count++%100 == 0)
			ProgressStepFile();
		if (Rec.Feature != "island")
			{
			InvalidFeatures.insert(Rec.Feature);
			continue;
			}

		gpos_t IslandLength = Rec.End - Rec.Start + 1;
		pair<gpos_t, gpos_t> Iv(Rec.Start, Rec.End);

		vector<LoHi> CDSs;
		CDSIx.GetOverlaps(Iv, CDSs);
		for (vector<LoHi>::const_iterator p = CDSs.begin(); p != CDSs.end(); ++p)
			{
			const LoHi &CDS = *p;
			if (GetOverlapLength(Rec.Start, Rec.End, CDS.Lo, CDS.Hi) > IslandLength/2)
				{
				++DeletedCount;
				goto NextRec;
				}
			}
		WriteGFFRec(fOut, Rec);
	NextRec:;
		}
	ProgressDone();

	if (!InvalidFeatures.empty())
		{
		for (set<string>::const_iterator p = InvalidFeatures.begin();
		  p != InvalidFeatures.end(); ++p)
			Log(" %s", (*p).c_str());
		Log("\n");
		Warning("%u invalid features in %s", SIZE(InvalidFeatures), InFileName);
		}

	Progress("%u deleted", DeletedCount);

	CloseStdioFile(fIn);
	CloseStdioFile(fAnnots);
	CloseStdioFile(fOut);
	}
