#include "utils.h"
#include "revfile.h"

static coord_t AddMargin(coord_t Pos, unsigned Margin)
	{
	return Pos + Margin;
	}

static coord_t SubMargin(coord_t Pos, unsigned Margin)
	{
	if (Pos >= Margin)
		return Pos - Margin;
	return 0;
	}

void GetBendpoints(const std::vector<AlnInfo> &AIs, unsigned GenomeIndex,
  std::vector<AIBP> &BPs, unsigned Margin)
	{
	const unsigned AICount = (unsigned) AIs.size();

	BPs.clear();
	BPs.reserve(2*AICount);

	for (unsigned i = 0; i < AICount; ++i)
		{
		const AlnInfo &AI = AIs[i];
		unsigned AIIndex = i;

	// Note: a given genome index may occur more than once
		const unsigned SegCount = (unsigned) AI.Segs.size();
		for (unsigned SegIndex = 0; SegIndex < SegCount; ++SegIndex)
			{
			const GCSeg &Seg = AI.Segs[SegIndex];
			if (Seg.GenomeIndex != GenomeIndex)
				continue;

			coord_t OldLo = Seg.lo;
			coord_t OldHi = Seg.hi;
			coord_t NewLo = AddMargin(OldLo, Margin);
			coord_t NewHi = SubMargin(OldHi, Margin);

			if (NewLo > NewHi)
				{
				coord_t Mid = (OldHi + OldLo)/2;
				NewLo = Mid;
				NewHi = Mid;
				}

			AIBP BP;
			BP.AlnInfoIndex = AIIndex;
			BP.ChrId = Seg.ChrId;
			BP.SegIndex = SegIndex;

			BP.Pos = NewLo;
			BP.IsLo = true;
			BPs.push_back(BP);

			BP.Pos = NewHi;
			BP.IsLo = false;
			BPs.push_back(BP);
			}
		}

	std::sort(BPs.begin(), BPs.end());
	}
