#include "utils.h"
#include "revfile.h"
#include "progress.h"
#include "getopts.h"
#include <string>

bool IsStopCodon(const string &Codon);

void GetCodonSubs()
	{
	const char *FileName = RequiredValueOpt("getcodonsubs");
	const char *OutFileName = RequiredValueOpt("out");

	RevFile Rev;
	Rev.Open(FileName);

	FILE *f = CreateStdioFile(OutFileName);

	const unsigned AICount = Rev.GetAlnInfoCount();
	unsigned ErrCount = 0;
	ProgressStart("Processing");
	for (unsigned AIIndex = 0; AIIndex < AICount; ++AIIndex)
		{
		if (ErrCount > 10)
			Die("> 10 errors");

		if (AIIndex%100 == 0)
			ProgressStep(AIIndex, AICount);

		unsigned AlnIndex = Rev.GetAlnIndex(AIIndex);
		if (AlnIndex == UINT_MAX)
			continue;

		AlnInfo AI;
		Rev.GetAlnInfo(AIIndex, AI);

		StdStrVec Rows;
		Rev.GetAlnRows(AlnIndex, Rows);

		if (Rows.size() != 2)
			{
			++ErrCount;
			Warning("Aln %u has %u rows, skipped", AlnIndex, (unsigned) Rows.size());
			continue;
			}

		const unsigned L = SIZE(Rows[0]);
		if (SIZE(Rows[1]) != L)
			Die("Not aligned");

		unsigned Frame = AI.GetAttr(ALN_ATTR_Frame);
		if (Frame == UINT_MAX)
			{
			++ErrCount;
			Warning("Aln %u does not have frame", AlnIndex);
			continue;
			}
		unsigned Seg0 = AI.GetSegIndex(0);
		unsigned Seg1 = AI.GetSegIndex(1);
		unsigned i = UINT_MAX;
		if (Frame == 0)
			i = 0;
		else if (Frame == 1)
			i = 1;
		else if (Frame == 2)
			i = 2;
		else
			{
			++ErrCount;
			Warning("Invalid frame %u in aln %u", Frame, AlnIndex);
			continue;
			}
		for (; i + 2 < L; i += 3)
			{
			string Codon0 = Rows[Seg0].substr(i, 3);
			string Codon1 = Rows[Seg1].substr(i, 3);

			if (Codon0.find('-') != string::npos || Codon1.find('-') != string::npos)
				continue;

#if	0
// This warning happens for unknown reasons; disabled...
			if ((IsStopCodon(Codon0) || IsStopCodon(Codon1)) && i + 3 != L)
				{
				++ErrCount;
				Warning("Stop codon aln %u, col %u (%s-%s)",
				  AlnIndex, i, Codon0.c_str(), Codon1.c_str());
				}
#endif

			fprintf(f, "%s %s\n", Codon0.c_str(), Codon1.c_str());
			}
		}
	ProgressDone();
	}
