#define TRACE 0

#include <algorithm>
#include <vector>
#include "utils.h"
#include "lohi.h"

using namespace std;

// Input is a set of possibly overlapping intervals.
// Output is ranges that are not covered.
// Index is the value to be assigned to LoHi.Index.
void GetLoHiGaps(const vector <LoHi> &LHs, vector<LoHi> &Gaps,
  unsigned Index)
	{
#if	TRACE
	Log("\n");
	Log("GetLoHiGaps\n");
#endif

	Gaps.clear();

	const unsigned N = SIZE(LHs);
	vector<LoHiBendpoint> BPs;
	for (unsigned i = 0; i < N; ++i)
		{
		const LoHi &LH = LHs[i];

		LoHiBendpoint BP;
		BP.Pos = LH.Lo;
		BP.IsLo = true;
		BP.Index = i;
		BPs.push_back(BP);

		BP.Pos = LH.Hi;
		BP.IsLo = false;
		BPs.push_back(BP);
		}

	sort(BPs.begin(), BPs.end());

	asserta(SIZE(BPs) == 2*N);
	unsigned CoverageCount = 0;
	unsigned StartGap = UINT_MAX;
	for (unsigned i = 0; i < 2*N; ++i)
		{
		const LoHiBendpoint &BP = BPs[i];
#if	TRACE
		Log("Pos %u IsLo %c CovCount %u\n",
		  BP.Pos, tof(BP.IsLo), CoverageCount);
#endif
		if (BP.IsLo)
			{
			if (CoverageCount == 0 && StartGap < BP.Pos)
				{
				LoHi Gap;
				Gap.Lo = StartGap;
				Gap.Hi = BP.Pos - 1;
				Gap.Index = Index;
				Gaps.push_back(Gap);
				}
			++CoverageCount;
			}
		else
			{
			asserta(CoverageCount > 0);
			--CoverageCount;
			if (CoverageCount == 0)
				StartGap = BP.Pos + 1;
			}
		}
	asserta(CoverageCount == 0);
	}
