#include "evolver.h"

#define TRACE	0

void Evolver::GetContigRanges(gpos_t Pos_, gpos_t Length, vector<gpos_t> &Starts,
  vector<gpos_t> &Lengths, bool *ptrExists) const
	{
	gpos_t Pos = Pos_;
#if	TRACE
	Log("\n");
	Log("Evo::GetContigRanges(Pos=%"PF", Length=%"PF")\n", Pos, Length);
#endif
	if (ptrExists != 0)
		*ptrExists = true;

	Starts.clear();
	Lengths.clear();
	if (Length == 0)
		{
		if (ptrExists == 0)
			Die("Evolver::GetContigRanges, range does not exist");
		*ptrExists = false;
		return;
		}

	vector<gpos_t> PosVec;
	gpos_t SumLengths = 0;
	for (;;)
		{
#if	TRACE
		Log(" loop: Pos=%"PF" SumLengths=%"PF"\n", Pos, SumLengths);
#endif
		PosVec.push_back(Pos);
		++SumLengths;
		if (SumLengths == Length)
			break;
		Pos = GetNextPos(Pos);
		if (Pos == GPOS_T_MAX)
			{
#if	TRACE
			Log("** Successor doesn't exist\n");
#endif
			if (ptrExists == 0)
				Die("Range does not exist");
			*ptrExists = false;
			return;
			}
		}

	sort(PosVec.begin(), PosVec.end());
#if	TRACE
	{
	Log("Sorted PosVec:\n");
	const unsigned N = SIZE(PosVec);
	for (unsigned i = 0; i < N; ++i)
		{
		if (i%16 == 0)
			Log("\n");
		Log(" %"PF, PosVec[i]);
		}
	Log("\n");
	}
#endif
	gpos_t Start = PosVec[0];
	gpos_t RangeLength = 1;
	const unsigned N = SIZE(PosVec);
	for (unsigned i = 1; i < N; ++i)
		{
		assert(PosVec[i] > PosVec[i-1]);
#if	TRACE
		Log("PosVec[%u]=%"PF", PosVec[%u]=%"PF"\n",
		  i-1, PosVec[i-1], i, PosVec[i]);
#endif
		if (PosVec[i] != PosVec[i-1] + 1)
			{
#if	TRACE
			Log("  !=, new start\n");
			Log("Add range %"PF", %"PF"\n", Start, RangeLength);
#endif
			Starts.push_back(Start);
			Lengths.push_back(RangeLength);

			Start = PosVec[i];
			RangeLength = 1;
#if	TRACE
			Log("Start=%"PF", RangeLength=%"PF"\n", Start, RangeLength);
#endif
			}
		else
			{
#if	TRACE
			Log("  ==, RangeLength=%u\n", RangeLength);
#endif
			++RangeLength;
			}
		}

#if	TRACE
	Log("End of loop, RangeLength=%u\n", RangeLength);
#endif
	if (RangeLength > 0)
		{
#if	TRACE
		Log("Add range %"PF", %"PF"\n", Start, RangeLength);
#endif
		Starts.push_back(Start);
		Lengths.push_back(RangeLength);
		}
#if	DEBUG
	{
	const unsigned N = SIZE(Starts);
	asserta(N == SIZE(Lengths));
	gpos_t SumLengths = 0;
	for (unsigned i = 0; i < N; ++i)
		SumLengths += Lengths[i];
	asserta(SumLengths == Length);
	}
#endif
	}
