#include "filter4.h"
#include "revfile.h"
#include "getopts.h"

/***
Get genome indexes 1 & 2, which are required, and chr
ids 1 and 2, which are optional, from command line.

If genomes are not specified, they default to 0 and 1.

If chr ids are not specified, this means all chromosomes.

Genome indexes may be specified directly by -genix1/2 or
indirectly by -genome1/2.

Chr ids may be specified directly by -chrid1/2, indiractly
by -chrname1/2, or not at all (=all chrs).

Return true if all chr pairs, false if chr pair is specified.
***/

bool GetRevChrPairOpts(const RevFile &Rev, unsigned *ptrGenomeIndexA,
  unsigned *ptrGenomeIndexB, unsigned *ptrChrIdA, unsigned *ptrChrIdB)
	{
	*ptrGenomeIndexA = 0;
	*ptrGenomeIndexB = 1;

	bool GenIx1Found = IntOpt("genomeindex1", ptrGenomeIndexA);
	if (GenIx1Found)
		*ptrGenomeIndexB = RequiredIntOpt("genomeindex2");
	else
		{
		const char *GenomeName1 = ValueOpt("genome1");
		if (GenomeName1 != 0)
			{
			const char *GenomeName2 = RequiredValueOpt("genome2");
			*ptrGenomeIndexA = Rev.GetGenomeIndex(GenomeName1);
			*ptrGenomeIndexA = Rev.GetGenomeIndex(GenomeName2);
			}
		}

	bool ChrIdAFound = IntOpt("chrid1", ptrChrIdA);
	if (ChrIdAFound)
		{
		bool ChrIdBFound = IntOpt("chrid2", ptrChrIdB);
		if (!ChrIdBFound)
			Die("-chrid1 specified, requires -chrid2");
		return false;
		}

	const char *ChrName1 = ValueOpt("chrname1");
	if (0 == ChrName1)
		return true;

	const char *ChrName2 = ValueOpt("chrname2");
	if (0 == ChrName2)
		Die("-chrname1 specified, requires -chrname2");

	*ptrChrIdA = Rev.GetChrId(*ptrGenomeIndexA, ChrName1);
	*ptrChrIdB = Rev.GetChrId(*ptrGenomeIndexB, ChrName2);
	return false;
	}
