#define TRACE	0

#include "utils.h"
#include "alninfo.h"
#include <vector>

using namespace std;

// Input is an alignment row.
// OffsetToCol[Offset] is the column in which chromosome
// position Seg.lo+Offset is found.
// ColToOffset[Col] is UINT_MAX if the column contains a gap,
// otherwise the offset relative to Seg.lo of the letter in
// that column.
void GetRowOffsetMap(const StdStr &Row, vector<unsigned> &OffsetToCol,
  vector<unsigned> &ColToOffset, bool Plus)
	{
	const unsigned ColCount = SIZE(Row);
	ColToOffset.reserve(ColCount);
	OffsetToCol.reserve(ColCount);
	if (Plus)
		{
		for (unsigned Col = 0; Col < ColCount; ++Col)
			{
			char c = Row[Col];
			if (c == '-')
				ColToOffset.push_back(UINT_MAX);
			else
				{
				ColToOffset.push_back(SIZE(OffsetToCol));
				OffsetToCol.push_back(Col);
				}
			}
		}
	else
		{
		for (int Col = (int) ColCount - 1; Col >= 0; --Col)
			{
			char c = Row[Col];
			if (c == '-')
				ColToOffset.push_back(UINT_MAX);
			else
				{
				ColToOffset.push_back(SIZE(OffsetToCol));
				OffsetToCol.push_back((unsigned) Col);
				}
			}
		reverse(ColToOffset.begin(), ColToOffset.end());
		}
	}

void GetRangeSeg(const StdStr &Row, unsigned ColFrom, unsigned ColTo,
  const GCSeg &Seg, GCSeg &RangeSeg)
	{
#if	TRACE
	Log("\n");
	Log("GetRangeSeg(ColFrom=%u, ColTo=%u)\n", ColFrom, ColTo);
	Log("Row=%.128s\n", Row.c_str());
	Log("Seg=");
	Seg.LogMe(true);
#endif
	const unsigned ColCount = SIZE(Row);
	asserta(ColFrom <= ColTo);
	asserta(ColTo < ColCount);

	RangeSeg.GenomeIndex = Seg.GenomeIndex;
	RangeSeg.ChrId = Seg.ChrId;
	RangeSeg.lo = COORD_T_MAX;
	RangeSeg.hi = COORD_T_MAX;
	RangeSeg.Plus = Seg.Plus;

	coord_t Pos = (Seg.Plus ? Seg.lo : Seg.hi);
	for (unsigned Col = 0; Col < ColCount; ++Col)
		{
		char c = Row[Col];
		if (c == '-')
			continue;

		if (Col >= ColFrom)
			{
			if (RangeSeg.lo == COORD_T_MAX)
				{
				RangeSeg.lo = Pos;
				RangeSeg.hi = Pos;
				}
			else
				{
				if (Seg.Plus)
					++RangeSeg.hi;
				else
					{
					asserta(RangeSeg.lo > 0);
					--RangeSeg.lo;
					}
				}
			}
		if (Col >= ColTo)
			{
#if	TRACE
			Log("RangeSeg=");
			RangeSeg.LogMe(true);
#endif
			return;
			}
		if (Seg.Plus)
			++Pos;
		else
			{
			asserta(Pos > 0);
			--Pos;
			}
		}
	}

void GetColRange(const StdStr &Row, const GCSeg &Seg, coord_t lo, coord_t hi,
  unsigned &ColFrom, unsigned &ColTo)
	{
	asserta(lo >= Seg.lo);
	asserta(hi <= Seg.hi);
	asserta(lo <= hi);

	const unsigned ColCount = SIZE(Row);
	ColFrom = UINT_MAX;
	ColTo = UINT_MAX;

	if (Seg.Plus)
		{
		coord_t Pos = Seg.lo;
		for (unsigned Col = 0; Col < ColCount; ++Col)
			{
			char c = Row[Col];
			if (c == '-')
				continue;

			if (Pos == lo)
				{
				asserta(ColFrom == UINT_MAX);
				ColFrom = Col;
				}

			if (Pos == hi)
				{
				asserta(ColFrom != UINT_MAX);
				asserta(ColTo == UINT_MAX);
				ColTo = Col;
				return;
				}
			++Pos;
			}
		}
	else
		{
		asserta(!Seg.Plus);
		coord_t Pos = Seg.hi;
		for (unsigned Col = 0; Col < ColCount; ++Col)
			{
			char c = Row[Col];
			if (c == '-')
				continue;

			if (Pos == hi)
				{
				asserta(ColFrom == UINT_MAX);
				ColFrom = Col;
				}

			if (Pos == lo)
				{
				asserta(ColFrom != UINT_MAX);
				asserta(ColTo == UINT_MAX);
				ColTo = Col;
				return;
				}
			--Pos;
			}
		}
	}
