#include <ctype.h>
#include <errno.h>
#include "getopts.h"
#include "utils.h"

#ifndef BUFFER_SIZE
#define BUFFER_SIZE 65535
#endif

bool GetSeq(FILE *f, std::string &Seq, std::string &Label)
{
	Seq.clear();
	Label.clear();

	Seq.reserve(1000000);
	Label.reserve(4096);

	char buf[BUFFER_SIZE];
	size_t i = 0, N = 0;
	enum {StateBegin, StateHeader, StateBody} State = StateBegin;

	for (;;)
	{
		/* Check if it's time to read new block */
		if (i == N)
		{
			if (!(N = fread(buf, 1, BUFFER_SIZE, f)))
				break;

			i = 0;
		}

		/* Process data */
		switch (State)
		{
			/* Beginning: look for '>' */
			case StateBegin:
				if (buf[i++] != '>')
					Die("FASTA header '>' needed!");
				else
				{
					/* We are now in the header */
					State = StateHeader;
					Label.clear();
				}
				continue;

			/* While in the header */
			case StateHeader:
				{
					size_t i_old = i;
					while ((i < N) && (buf[i] != '\r') && (buf[i] != '\n'))
						i++;
					
					/* Update chromosome name */
					Label.append(buf + i_old, i - i_old);

					if (i != N)
					{
						if (buf[i] == '\n')
						{
							/* End of chromosome header reached! */
							State = StateBody;
						}
						/* Skip \r and \n */
						i++;
					}
				}
				continue;

			/* While in the fasta body */
			case StateBody:
				{
					size_t i_old = i;
					if ((i < N) && (buf[i] == '>'))
					{
						/* New chromosome ahead! */
						long Offset = (long) (N - i);
						if (fseek(f, -Offset, SEEK_CUR) != 0)
							Die("fseek failed");
						return true;
					}
					while ((i < N) && (buf[i] != '\n') && (buf[i] != '\r'))
						i++;

					Seq.append(buf + i_old, i - i_old);
					if (i != N)
						i++;
				}
				continue;
		}
	}

	return (State == StateBody);
}
