#include "getopts.h"
#include "utils.h"
#include "progress.h"
#include "gff.h"
#include "bp.h"
#include <vector>

using namespace std;

void GFFCover()
	{
	const char *InFileName = RequiredValueOpt("gffcover");
	const char *OutFileName = RequiredValueOpt("out");

	FILE *fIn = OpenStdioFile(InFileName);
	FILE *fOut = CreateStdioFile(OutFileName);

	vector<BendPoint> BPs;
	GFFRecord Rec;
	while (GetNextGFFRecord(fIn, Rec))
		{
		BendPoint BP;
		BP.Pos = Rec.Start;
		BP.lo = true;
		BPs.push_back(BP);

		BP.Pos = Rec.End;
		BP.lo = false;
		BPs.push_back(BP);
		}

	sort(BPs.begin(), BPs.end());

	int CovCount = 0;
	Rec.Attrs = "";
	Rec.Frame = INT_MAX;
	Rec.Score = 0;
	Rec.Source = "gffcover";
	Rec.Feature = "cover";
	coord_t Total = 0;
	for (vector<BendPoint>::const_iterator p = BPs.begin(); p != BPs.end(); ++p)
		{
		const BendPoint &BP = *p;
		if (BP.lo)
			{
			if (CovCount == 0)
				Rec.Start = BP.Pos;
			++CovCount;
			}
		else
			{
			--CovCount;
			if (CovCount == 0)
				{
				Rec.End = BP.Pos;
				WriteGFFRec(fOut, Rec);
				Total += Rec.GetLength();
				}
			}
		}

	CloseStdioFile(fIn);
	CloseStdioFile(fOut);
	ProgressNoRes("Total %u bases covered", (unsigned) Total);
	}
