#include "ice.h"

coord_t GetOverlapLength(coord_t loA, coord_t hiA, coord_t loB, coord_t hiB);

const char *ContigTypeToStr(ContigType Type)
	{
	switch (Type)
		{
	case CT_Neutral:
		return "Neutral";
	case CT_CDS:
		return "CDS";
	case CT_UTR:
		return "UTR";
	case CT_NGE:
		return "NGE";
	case CT_NXE:
		return "NXE";
	case CT_Merged:
		return "Merged";
	case CT_Range:
		return "Range";
		}
	static char Str[32];
	sprintf(Str, "CT_%d", Type);
	return Str;
	}

void InterChrEvolver::LogMe() const
	{
	Log("\n");
	const unsigned ChrCount = GetChrCount();
	Log("Chr      Length  Name\n");
	Log("===  ==========  ====\n");
	for (unsigned ChrIndex = 0; ChrIndex < ChrCount; ++ChrIndex)
		{
		Log("%3u", ChrIndex);
		Log("  %10"PF, GetChrLength(ChrIndex));
		Log("  %s", m_ChrNames[ChrIndex].c_str());
		Log("\n");
		}
	if (SIZE(m_Contigs) > 1000)
		{
		Log("\n");
		Log("%u contigs, not logging them...\n", SIZE(m_Contigs));
		}
	else
		LogContigs(m_Contigs);
	}

void InterChrEvolver::LogContigs(const set<ContigData> &Contigs) const
	{
	Log("\n");
	Log("   Type   Gene  EvChr       EvPos          Hi  AnChr      AncPos       AncHi      Length  +  Copy\n");
	Log("=======  =====  =====  ==========  ==========  =====  ==========  ==========  ==========  =  ====\n");

	unsigned ChrIndex = UINT_MAX;
	for (ContigSetCIter p = Contigs.begin(); p != Contigs.end(); ++p)
		{
		const ContigData &Contig = *p;
		if (Contig.EvChrIndex != ChrIndex && ChrIndex != UINT_MAX)
			Log("\n");
		Log("%7.7s", ContigTypeToStr(Contig.Type));
		if (Contig.GeneIndex == UINT_MAX)
			Log("       ");
		else
			Log("  %5u", Contig.GeneIndex);
		Log("  %5u", Contig.EvChrIndex);
		Log("  %10"PF, Contig.EvPos);
		Log("  %10"PF, Contig.GetEvHi());
		Log("  %5u", Contig.AncChrIndex);
		Log("  %10"PF, Contig.AncPos);
		Log("  %10"PF, Contig.GetAncHi());
		Log("  %10"PF, Contig.Length);
		Log("  %c", pom(Contig.Plus));
		Log("  %4c", tof(Contig.Copy));
		Log("\n");
		ChrIndex = Contig.EvChrIndex;
		}
	Log("\n");
	}

void InterChrEvolver::LogRange(unsigned ChrIndex, gpos_t Pos, gpos_t Length) const
	{
	Log("\n");
	Log("Range Chr %u Pos %"PF" L %"PF" End %"PF"\n",
	  ChrIndex, Pos, Length, Pos + Length - 1);
	set<ContigData> Contigs;
	GetOverlappingContigs(ChrIndex, Pos, Length, Contigs);
	LogContigs(Contigs);
	}
