#include "ice.h"
#include "myrand.h"
#include "progress.h"
#include "evomodel.h"
#include "getopts.h"

extern unsigned g_ValEvents;
extern unsigned g_StartValEvents;

EVENT_TYPE InterChrEvolver::GetRandomEventType() const
	{
	return (EVENT_TYPE) RandWeightDist(m_IntegratedEventRates);
	}

EVENT_RESULT InterChrEvolver::TryEvent()
	{
	m_CurrentEvent = GetRandomEventType();
	for (unsigned Try = 0; Try < P_EventFailTries; ++Try)
		{
		EVENT_RESULT Result = ER_Fail;
		switch (m_CurrentEvent)
			{
		case EVENT_TYPE_InterChrCopy:
			Result = Try_InterChrCopy();
			break;

		case EVENT_TYPE_InterChrMove:
			Result = Try_InterChrMove();
			break;

		case EVENT_TYPE_ChrSplit:
			Result = Try_ChrSplit();
			break;

		case EVENT_TYPE_ChrFuse:
			Result = Try_ChrFuse();
			break;

		case EVENT_TYPE_RecipTransloc:
			Result = Try_RecipTransloc();
			break;

		case EVENT_TYPE_NonRecipTransloc:
			Result = Try_NonRecipTransloc();
			break;

		default:
			Warning("InterChrEvolver::TryEvent, invalid event %u",
			  (unsigned) m_LastAcceptedEvent);
			break;
			}

		switch (Result)
			{
		case ER_Reject:
		case ER_Accept:
			return Result;
		case ER_Fail:
			continue;
		default:
			asserta(false);
			}
		}
	return ER_Fail;
	}

void InterChrEvolver::SetEventCount()
	{
	double AvgWaitTime = GetAvgWaitTime();
	m_EventCount = (unsigned) (0.5 + m_Duration/AvgWaitTime);
	m_EventIndex = 0;
	}

void InterChrEvolver::SetEventCount2(unsigned EventCount)
	{
	m_EventCount = EventCount;
	m_Duration = GetAvgWaitTime()*m_EventCount;
	}

void InterChrEvolver::Run()
	{
	Progress("T=%.5f, %u events", m_Duration, m_EventCount);
	ProgressStart("Evolving");
	m_LastAcceptedEventIndex = UINT_MAX;
	for (; m_EventIndex < m_EventCount; ++m_EventIndex)
		{
		if (m_EventIndex%20 == 0)
			{
			double t = m_Duration*((double) m_EventIndex / (double) m_EventCount);
			char Str[100];
			sprintf(Str, "Evolving t=%.5f event=%u", t, m_EventIndex);
			SetProgressMsg(Str);
			ProgressStep(m_EventIndex, m_EventCount);
			}

		bool Val = (g_StartValEvents > 0 || g_ValEvents > 0);
		if (Val && m_EventIndex >= g_StartValEvents && 
		  (g_ValEvents == 0 || m_EventIndex%g_ValEvents == 0))
			{
			Validate("valevents");
			const char *SaveGoodFileName = ValueOpt("savegood");
			if (SaveGoodFileName != 0)
				ToFile(SaveGoodFileName);
			}

		EVENT_RESULT Result = TryEvent();
		if (Result == ER_Accept)
			{
			m_LastAcceptedEventIndex = m_EventIndex;
			m_LastAcceptedEvent = m_CurrentEvent;
			m_LastAcceptedEventPos = m_CurrentEventPos;
			}
		}

	m_EventIndex = UINT_MAX;
	m_CurrentEvent = EVENT_TYPE_Invalid;
	m_LastAcceptedEvent = EVENT_TYPE_Invalid;
	SetProgressMsg("Evolving");
	ProgressDone();
	}
