#ifndef IFACE_H
#define IFACE_H

/***
This file, iface.h, documents class interface conventions.

It is included in class header files that are intended
to confirm to these conventions (a) to give a visible
signal that this is so, and (b) so that binaries appear
to have a dependency on iface.h; otherwise it might appear
to be orphaned and hence deleted.

Functions that retrieve and modify properties of an object
are called GetFoo() and SetFoo() respectively. (There is a
temptation to call a retriever function simply Foo(), but
I don't like this, see Code Complete for explanation).

Input arguments are placed before output arguments.

Input objects are passed by const reference.

Output objecte are passed by non-const reference.

An output object is normally re-initialized before being
set, e.g. an STL contained is clear()'d. If this is not done,
e.g. to allow accumulating a vector or set of results, the
function is named GetAppendFoo() rather than GetFoo(). This
is deliberately clunky to draw the reader's attention that
there is something unusual.

Functions always die on failure, with the exception of
GetFooPtr() (see below).

GetFoo() returns a const reference to a Foo.

GetModifiableFoo() returns a non-const reference to a Foo. This
allows the caller to modify an internal value in the class, which
should of course be done with discretion.

GetFooPtr() returns a pointer to a Foo, or NULL if the requested
Foo is not found. This is an exception to the rule that functions
die on failure.

Parameters named FooIndex are integers from 0, 1 ... (N-1) where N
is FooCount.

Parameters named FooId are integers with values that are not necessarily
consecutive (contrast with FooIndex).
***/

#endif // IFACE_H
