#ifndef lohi_h
#define lohi_h

#include <utility> 

using namespace std;

struct LoHi
	{
	LoHi()
		{
		Lo = GPOS_T_MAX;
		Hi = GPOS_T_MAX;
		Index = UINT_MAX;
		}

	LoHi(gpos_t L, gpos_t H, gpos_t I = UINT_MAX) : Lo(L), Hi(H), Index(I)
		{
		}

	pair<gpos_t, gpos_t> operator()() const
		{
		return std::pair<gpos_t, gpos_t>(Lo, Hi);
		}

	std::pair<gpos_t, gpos_t> GetIv() const
		{
		return std::pair<gpos_t, gpos_t>(Lo, Hi);
		}

	gpos_t GetLength() const
		{
		return Hi - Lo + 1;
		}

	bool operator<(const LoHi &rhs) const
		{
		if (Lo != rhs.Lo)
			return Lo < rhs.Lo;
		if (Hi != rhs.Hi)
			return Hi < rhs.Hi;
		return Index < rhs.Index;
		}

	void LogMe(bool NL = false) const
		{
		Log("Lo=%"PF" Hi=%"PF" Ix=%u", Lo, Hi, Index);
		if (NL)
			Log("\n");
		}

	gpos_t Lo;
	gpos_t Hi;
	unsigned Index;
	};

struct LoHiBendpoint
	{
	gpos_t Pos;
	bool IsLo;
	unsigned Index;

/**
Sort los before his.
This gives the correct result when traversing left-
to-right to compute coverage count.

lo A hi
-------
      --------
      lo B  hi

In this case, hiA=loB, and there is one position where
the coverage count is 2. We therefore need to see loB
first so the count goes to 2 at that position.
**/
	bool operator<(const LoHiBendpoint &rhs) const
		{
		if (Pos != rhs.Pos)
			return Pos < rhs.Pos;
		return IsLo && !rhs.IsLo;
		}
	};

void LogLoHi(const LoHi &LH);

#endif // lohi_h
