#include "utils.h"
#include "getopts.h"
#include "fastastr.h"
#include "wordhist.h"
#include "progress.h"

#ifdef _MSC_VER
#include <process.h>
#else
#include <unistd.h>
#endif

void DoMakeHist(const char *SeqFileName, const char *HistFileName,
  ALPHA SeqAlpha, ALPHA HistAlpha);
void DoMakeTab(const char *HistFileName1, const char *HistFileName2,
  const char *TabFileName);
void DoMask(const char *InFileName, const char *OutFileName,
  const char *TabFileName, ALPHA SeqAlpha);
void DoExtractORFs(const char *InFileName, const char *OutFileName, ALPHA OutAlpha);
void CheckThresholdArgs();

void MaskSimple()
	{
	const char *InFileName = RequiredValueOpt("mask");
	const char *TargetFileName = RequiredValueOpt("target");
	const char *OutFileName = RequiredValueOpt("out");
	const char *ORFFileName = ValueOpt("prot");

	bool KeepTmp = FlagOpt("keeptmp");

	CheckThresholdArgs();

	char HistFileName1[64];
	char HistFileName2[64];
	char TabFileName[64];

	int pid = getpid();

	sprintf(HistFileName1, "query_hist.%d.tmp", pid);
	sprintf(HistFileName2, "target_hist.%d.tmp", pid);
	sprintf(TabFileName, "tab.%d.tmp", pid);

	if (ORFFileName != 0)
		{
		DoExtractORFs(InFileName, ORFFileName, ALPHA_Amino);
		DoMakeHist(ORFFileName, HistFileName1, ALPHA_Amino, ALPHA_Amino);
		DoMakeHist(TargetFileName, HistFileName2, ALPHA_Nucleo, ALPHA_Amino);
		DoMakeTab(HistFileName1, HistFileName2, TabFileName);
		DoMask(ORFFileName, OutFileName, TabFileName, ALPHA_Amino);
		}
	else
		{
		DoMakeHist(InFileName, HistFileName1, ALPHA_Nucleo, ALPHA_Nucleo);
		DoMakeHist(TargetFileName, HistFileName2, ALPHA_Nucleo, ALPHA_Nucleo);
		DoMakeTab(HistFileName1, HistFileName2, TabFileName);
		DoMask(InFileName, OutFileName, TabFileName, ALPHA_Nucleo);
		}

	if (!KeepTmp)
		{
		remove(HistFileName1);
		remove(HistFileName2);
		remove(TabFileName);
		}
	}
