#include "utils.h"
#include "fastastr.h"
#include "getopts.h"
#include "progress.h"
#include "xlat.h"
#include <ctype.h>

#define TRACE	0

static const unsigned CTDim = CHAR_TYPE_Max+1;
static uint64 g_CharTypeCounts[CTDim];
static uint64 g_MaskedBlastSeedCount;
static uint64 g_TotalBlastSeedCount;

static unsigned g_MerSize;

static const char *h(uint64 i)
	{
	return atoh(uint64tostr(i));
	}

static void StatSeq(std::vector<unsigned char> &Seq, ALPHA Alpha)
	{
	unsigned SeqLength = (unsigned) Seq.size();

	unsigned SeedCount = 0;
	unsigned UpperCaseRunLength = 0;
#if	TRACE
	Log("Char        Type  RunLength  Seeds\n");
	Log("====  ==========  =========  =====\n");
#endif
	for (unsigned Pos = 0; Pos < SeqLength; ++Pos)
		{
		unsigned char c = Seq[Pos];
		CHAR_TYPE Type = CharType(c, Alpha);
		if (Type < 0 || Type > CHAR_TYPE_Max)
			Die("Invalid char type");

		++(g_CharTypeCounts[Type]);

		if (Type == CHAR_TYPE_Upper)
			{
			++UpperCaseRunLength;
			if (UpperCaseRunLength >= g_MerSize)
				++SeedCount;
			}
		else
			UpperCaseRunLength = 0;

#if	TRACE
		Log("%4c  %10s  %9u  %5u\n",
		  c, CharTypeToStr(Type), UpperCaseRunLength, SeedCount);
#endif
		}
	
	if (SeqLength >= g_MerSize)
		{
		unsigned SeqMerCount = SeqLength - g_MerSize + 1;
		if (SeedCount > SeqMerCount)
			Die("BlastSeedCount > SeqMerCount");
		if (g_MaskedBlastSeedCount > g_TotalBlastSeedCount)
			Die("g_MaskedBlastSeedCount > g_TotalBlastSeedCount (before)");
		g_MaskedBlastSeedCount += SeqMerCount - SeedCount;
		g_TotalBlastSeedCount += SeqMerCount;
		if (g_MaskedBlastSeedCount > g_TotalBlastSeedCount)
			Die("g_MaskedBlastSeedCount > g_TotalBlastSeedCount (after)");
		}
	}

void MaskStats()
	{
	const char *InFileName = RequiredValueOpt("maskstats");
	g_MerSize = RequiredIntOpt("wordsize");
	bool Amino = FlagOpt("aminoseq");

	ALPHA Alpha = (Amino ? ALPHA_Amino : ALPHA_Nucleo);

	FastaStream FS;
	FS.Open(InFileName);

	std::vector<char> Label;
	std::vector<unsigned char> Seq;

	ProgressStart("Compiling statistics");
	unsigned Steps = (unsigned) (FS.GetFileSize()/1000000 + 1);
	unsigned PrevStep = UINT_MAX;
	for (;;)
		{
		unsigned Step = (unsigned) (FS.GetFilePos()/1000000);
		if (Step != PrevStep)
			{
			ProgressStep(Step, Steps);
			PrevStep = Step;
			}

		bool Ok = FS.GetNextSeq(Label, Seq);
		if (!Ok)
			break;

		StatSeq(Seq, Alpha);
		}
	ProgressDone();

	uint64 GenomeSize = 0;
	for (int t = 0; t <= CHAR_TYPE_Max; ++t)
		GenomeSize += g_CharTypeCounts[t];

	if (GenomeSize == 0)
		{
		Progress("Genome size zero");
		return;
		}

	ProgressNoRes("");
	ProgressNoRes("            Type             Bases      Pct");
	ProgressNoRes("----------------  ----------------  -------");
	for (int t = 0; t <= CHAR_TYPE_Max; ++t)
		{
		double Pct = (double) g_CharTypeCounts[t]*100.0/(double) GenomeSize;
		ProgressNoRes("%16.16s  %16.16s  %5.1f %%", CharTypeToStr((CHAR_TYPE) t),
		  h(g_CharTypeCounts[t]), Pct);
		}

	ProgressNoRes("================  ================  =======");
	ProgressNoRes("%16.16s  %16.16s  %5.1f %%", "Genome size", h(GenomeSize), 100.0);
	ProgressNoRes("");

	uint64 UnmaskedSeeds = g_TotalBlastSeedCount - g_MaskedBlastSeedCount;

	if (g_TotalBlastSeedCount > 0)
		{
		ProgressNoRes("");
		ProgressNoRes("            Type             Count      Pct");
		ProgressNoRes("----------------  ----------------  -------");

		ProgressNoRes("%16.16s  %16.16s  %5.1f %%", "Masked seeds",
		  h(g_MaskedBlastSeedCount), g_MaskedBlastSeedCount*100.0/g_TotalBlastSeedCount);

		ProgressNoRes("%16.16s  %16.16s  %5.1f %%", "Unmasked seeds",
		  h(UnmaskedSeeds), UnmaskedSeeds*100.0/g_TotalBlastSeedCount);

		ProgressNoRes("================  ================  =======");
		ProgressNoRes("%16.16s  %16.16s  %5.1f %%", "Total seeds",
		  h(g_TotalBlastSeedCount), 100.0);
		ProgressNoRes("");
		}
	}
