#include "masktab.h"
#include "utils.h"

MaskTable::MaskTable()
	{
	m_WordCount = 0;
	m_Alpha = ALPHA_Invalid;
	m_BitVector = 0;
	}

void MaskTable::Free()
	{
	if (m_BitVector != 0)
		myfree(m_BitVector);
	m_WordCount = 0;
	m_Alpha = ALPHA_Invalid;
	m_BitVector = 0;
	}

uint64 MaskTable::BitVectorBytes() const
	{
	return (m_WordCount + 7)/8;
	}

void MaskTable::Init(ALPHA Alpha, unsigned WordSize)
	{
	m_Alpha = Alpha;
	m_WordSize = WordSize;

	uint64 WordCount = CalcWordCount(Alpha, WordSize);
	m_WordCount = WordCount;

	uint64 Bytes = BitVectorBytes();

	if ((uint64) (off_t) Bytes != Bytes)
		Die("Mask table file size overflows off_t");
	if ((uint64) (size_t) Bytes != Bytes)
		Die("Mask table file size overflows size_t");

	m_BitVector = myalloc(unsigned char, (size_t) Bytes);
	memset(m_BitVector, 0, (size_t) Bytes);
	}

void MaskTable::SetBit(uint64 i)
	{
	if (i >= m_WordCount)
		Die("MaskTable::SetBit, out of range");

	uint64 Byte = i/8;
	unsigned Bit = (unsigned) i%8;
	unsigned char b = (1 << Bit);
	m_BitVector[Byte] |= b;
	}

bool MaskTable::GetBit(uint64 i) const
	{
	uint64 Byte = i/8;
	unsigned Bit = (unsigned) i%8;
	unsigned char b = (1 << Bit);
	return (m_BitVector[Byte] & b) != 0;
	}

void MaskTable::ToFile(const char *FileName) const
	{
	if (m_Alpha == ALPHA_Invalid)
		Die("WordHist::ToFile, not initialized");

	FILE *f = CreateStdioFile(FileName);

// Write header intially with incorrect magic,
// thus file will be invalid unless all data
// correctly written.
	MaskTableHdr Hdr;
	Hdr.Magic = 0;
	Hdr.Alpha = m_Alpha;
	Hdr.WordSize = m_WordSize;

	WriteStdioFile(f, 0, &Hdr, sizeof(Hdr));
	uint64 Bytes = BitVectorBytes();
	if ((uint64) (off_t) Bytes != Bytes)
		Die("Bit vector overflows off_t");
	WriteStdioFile(f, (off_t) sizeof(Hdr), m_BitVector, (off_t) Bytes);

// Re-write header with correct magic
	Hdr.Magic = MaskTableMagic;
	WriteStdioFile(f, 0, &Hdr, sizeof(Hdr));
	CloseStdioFile(f);
	}

void MaskTable::FromFile(const char *FileName)
	{
	FILE *f = OpenStdioFile(FileName);
	MaskTableHdr Hdr;
	ReadStdioFile(f, 0, &Hdr, sizeof(Hdr));
	if (Hdr.Magic != MaskTableMagic)
		Die("Invalid mask table file, bad magic");

	m_Alpha = Hdr.Alpha;
	m_WordSize = Hdr.WordSize;
	m_WordCount = CalcWordCount(m_Alpha, m_WordSize);

// Bytes in bit vector
	uint64 Bytes = BitVectorBytes();

	if ((uint64) (off_t) Bytes != Bytes)
		Die("Mask table file size overflows off_t");
	if ((uint64) (size_t) Bytes != Bytes)
		Die("Mask table file size overflows size_t");

	uint64 ExpectedFileSize = sizeof(MaskTableHdr) + Bytes;
	off_t FileSize = GetStdioFileSize(f);

	if ((uint64) FileSize != ExpectedFileSize)
		{
		char s1[64];
		char s2[64];
		Die("Alpha %s, word count %s, size of tab file should be %s, is %s",
		  AlphaToStr(m_Alpha),
		  uint64tostr(m_WordCount),
		  uint64tostr(ExpectedFileSize, s1),
		  uint64tostr(FileSize, s2));
		}

	m_BitVector = myalloc(unsigned char, (size_t) Bytes);
	ReadStdioFile(f, (off_t) sizeof(Hdr), m_BitVector, (off_t) Bytes);
	CloseStdioFile(f);
	}
