#include "revfile.h"
#include "utils.h"

extern unsigned g_Margin;
extern unsigned g_MaxDiagDiff;
extern double g_MaxDiagFract;
extern unsigned g_DiagDiffAlwaysMerge;

extern unsigned g_GenomeIndexA;
extern unsigned g_GenomeIndexB;

bool LinkBundle(const AlnInfo &AIA, const AlnInfo &AIB);
bool OverlapWithMargin(const AlnInfo &AIA, const AlnInfo &AIB, unsigned GenomeIndex,
  unsigned Margin);

coord_t AILength(const AlnInfo &AI)
	{
	const GCSeg &SegA = AI.GetSeg(g_GenomeIndexA);
	const GCSeg &SegB = AI.GetSeg(g_GenomeIndexA);

	coord_t LenA = SegA.GetLength();
	coord_t LenB = SegB.GetLength();
	return (LenA + LenB)/2;
	}

int AIDiag(const AlnInfo &AI)
	{
	const GCSeg &SegA = AI.GetSeg(g_GenomeIndexA);
	const GCSeg &SegB = AI.GetSeg(g_GenomeIndexB);

	bool Plus = (SegA.Plus == SegB.Plus);

	coord_t d_lo;
	coord_t d_hi;
	if (Plus)
		{
		d_lo = SegA.lo - SegB.lo;
		d_hi = SegA.hi - SegB.hi;
		}
	else
		{
		d_lo = SegA.lo + SegB.lo;
		d_hi = SegA.hi + SegB.hi;
		}
	int Diag_lo = (int) d_lo;
	int Diag_hi = (int) d_hi;
	if ((coord_t) Diag_lo != d_lo || (coord_t) Diag_hi != d_hi)
		Die("Chromosome too long (int / coord_t truncated)");
	return (Diag_lo + Diag_hi)/2;
	}

bool Mergeable(const AlnInfo &AI1, const AlnInfo &AI2)
	{
	if (!LinkBundle(AI1, AI2))
		return false;

	bool Ov = OverlapWithMargin(AI1, AI2, g_GenomeIndexA, g_Margin) ||
	  OverlapWithMargin(AI1, AI2, g_GenomeIndexB, g_Margin);
	if (!Ov)
		return false;

	coord_t Length1 = AILength(AI1);
	coord_t Length2 = AILength(AI2);
	coord_t MinLength = std::min(Length1, Length2);

	int Diag1 = AIDiag(AI1);
	int Diag2 = AIDiag(AI2);
	unsigned DiagDiff = std::abs(Diag1 - Diag2);

	if (DiagDiff <= g_DiagDiffAlwaysMerge)
		return true;

	if (DiagDiff > g_MaxDiagDiff)
		return false;

	if ((double) DiagDiff / (double) MinLength > g_MaxDiagFract)
		return false;

	return true;
	}
