#include "revfile.h"
#include "utils.h"
#include "progress.h"

extern unsigned g_WindowLength;
extern unsigned g_MaxDiagDiff;
extern unsigned g_DiagDiffAlwaysMerge;
extern unsigned g_Margin;
extern double g_MaxDiagFract;

extern unsigned g_GenomeIndexA;
extern unsigned g_GenomeIndexB;

typedef bool (*ptrLinkFunction)(const AlnInfo &AI1, const AlnInfo &AI2);

void MakeEdges(const AIVec &AIs, ptrLinkFunction Link, unsigned GenomeIndex,
  unsigned MaxDist, IntVecVec &Edges, bool ShowProgress);
bool Mergeable(const AlnInfo &AI1, const AlnInfo &AI2);
void GetConnectedComponents(const IntVecVec &AdjMx_, IntVecVec &CCs,
  bool ShowProgress);
void GetBoundRect(const AIVec &AIs, unsigned GenomeIndexA, unsigned GenomeIndexB,
  AlnInfo &Rect);

static unsigned GetChrId(const AlnInfo &AI, unsigned GenomeIndex)
	{
	const GCSeg &Seg = AI.GetSeg(GenomeIndex);
	return Seg.ChrId;
	}

static void AssertChrPair(const AIVec &AIs, unsigned ChrIdA,
  unsigned ChrIdB)
	{
	const unsigned AICount = (unsigned) AIs.size();
	for (unsigned AIIndex = 0; AIIndex < AICount; ++AIIndex)
		{
		const AlnInfo &AI = AIs[AIIndex];

		unsigned AIChrIdA = GetChrId(AI, g_GenomeIndexA);
		unsigned AIChrIdB = GetChrId(AI, g_GenomeIndexB);

		if (AIChrIdA != ChrIdA || AIChrIdB != ChrIdB)
			Die("ChrId failed %u,%u %u,%u",
			  AIChrIdA, ChrIdA, AIChrIdB, ChrIdB);
		}
	}

static void MergeAIs(const AIVec &AIs, AlnInfo &MergedAI)
	{
	GetBoundRect(AIs, g_GenomeIndexA, g_GenomeIndexB, MergedAI);

	const unsigned AICount = (unsigned) AIs.size();
	ai_score_t Score = 0;
	for (unsigned AIIndex = 0; AIIndex < AICount; ++AIIndex)
		{
		const AlnInfo &AI = AIs[AIIndex];
		Score += AI.Score;
		}

	MergedAI.Score = Score;

	if (AICount == 1)
		{
		unsigned AlnLength = AIs[0].GetAlnLength();
		MergedAI.SetAlnLength(AlnLength);
		}
	}

void MergeHitsChrPair(const RevFile &In, unsigned ChrIdA, unsigned ChrIdB,
  const AIVec &AIs, RevFile &Out)
  	{
	const unsigned AICount = (unsigned) AIs.size();
	if (AICount == 0)
		return;

	AssertChrPair(AIs, ChrIdA, ChrIdB);

	const std::string &ChrNameA = In.GetChrName(g_GenomeIndexA, ChrIdA);
	const std::string &ChrNameB = In.GetChrName(g_GenomeIndexB, ChrIdB);

	Progress("MergeHitsChrPair %.16s[%u] %.16s[%u] %u AIs",
	  ChrNameA.c_str(), ChrIdA, ChrNameB.c_str(), ChrIdB, AICount);

	IntVecVec Edges;
	MakeEdges(AIs, Mergeable, g_GenomeIndexA, g_WindowLength, Edges, true);

	IntVecVec MergedAIIndexes;
	GetConnectedComponents(Edges, MergedAIIndexes, true);

	const unsigned MergedAICount = (unsigned) MergedAIIndexes.size();
	Progress("%u AIs, %u merged AIs", AICount, MergedAICount);
	for (unsigned MergedAIIndex = 0; MergedAIIndex < MergedAICount; ++MergedAIIndex)
		{
		AIVec MergedAIs;
		const IntVec &AIIndexes = MergedAIIndexes[MergedAIIndex];
		for_CIntVec(AIIndexes, p)
			{
			unsigned AIIndex = *p;
			assert(AIIndex < AICount);
			const AlnInfo &AI = AIs[AIIndex];
			MergedAIs.push_back(AI);
			}

		AlnInfo MergedAI;
		MergeAIs(MergedAIs, MergedAI);
		Out.AddAlnInfo(MergedAI);
		}
	}
