#include "utils.h"
#include "getopts.h"
#include "revfile.h"
#include "progress.h"

static const char *g_Prefix;
static const char *g_Suffix;

static char *MakeFileName(unsigned FileIndex)
	{
	size_t len = strlen(g_Prefix) + strlen(g_Suffix) + 64;
	char *buff = myalloc(char, len);
	sprintf(buff, "%s%u%s", g_Prefix, FileIndex + 1, g_Suffix);
	return buff;
	}

// Append one input file to output file
static void AppendFile(const RevFile &InFile, RevFile &OutFile)
	{
	const unsigned AlnInfoCount = InFile.GetAlnInfoCount();
	ProgressStart("Copying %u AIs", AlnInfoCount);
	for (unsigned AlnInfoIndex = 0; AlnInfoIndex < AlnInfoCount; ++AlnInfoIndex)
		{
		if (AlnInfoIndex%1000 == 0)
			ProgressStep(AlnInfoIndex, AlnInfoCount);
		AlnInfo AI;
		unsigned AlnIndex = InFile.GetAlnInfo(AlnInfoIndex, AI);
		if (AlnIndex == UINT_MAX)
			OutFile.AddAlnInfo(AI);
		else
			{
			std::vector<std::string> Rows;
			unsigned AlnIndex2 = InFile.GetAlnRows(AlnIndex, Rows);
			if (AlnIndex2 != AlnIndex)
				Die("AlnIndex2 != AlnIndex");
			OutFile.AddAln(AI, Rows);
			}
		}
	ProgressDone();
	}

void MergeRev()
	{
	const char *strInputFileCount = RequiredValueOpt("mergerev");
	const char *strOutputFileName = RequiredValueOpt("out");
	g_Prefix = ValueOpt("prefix");
	g_Suffix = ValueOpt("suffix");
	if (g_Prefix == 0)
		g_Prefix = "";
	if (g_Suffix == 0)
		g_Suffix = "";

	const unsigned InFileCount = (unsigned) atoi(strInputFileCount);
	if (InFileCount == 0)
		Die("Must specify > 0 input files");

// Open input files
	RevFile *InFiles = new RevFile[InFileCount];
	for (unsigned InFileIndex = 0; InFileIndex < InFileCount; ++InFileIndex)
		{
		RevFile &InFile = InFiles[InFileIndex];
		char *FileName = MakeFileName(InFileIndex);
		InFile.Open(FileName);
		myfree(FileName);
		}

// Verify that genome information is same in all input files
	const std::vector<GenInfo> &Genomes = InFiles[0].GetGenomes();
	for (unsigned InFileIndex = 1; InFileIndex < InFileCount; ++InFileIndex)
		{
		if (Genomes != InFiles[InFileIndex].GetGenomes())
			Die("Genome information differs between files");
		}

// Create output file
	RevFile OutFile;
	OutFile.Create(strOutputFileName, Genomes);

// Copy data
	for (unsigned InFileIndex = 0; InFileIndex < InFileCount; ++InFileIndex)
		{
		Progress("File %u of %u", InFileIndex + 1, InFileCount);
		RevFile &InFile = InFiles[InFileIndex];
		AppendFile(InFile, OutFile);
		}

// Clean up
	Progress("Closing files and copying data");
	for (unsigned InFileIndex = 0; InFileIndex < InFileCount; ++InFileIndex)
		InFiles[InFileIndex].Close();
	delete[] InFiles;

	OutFile.Close();
	}
