#ifndef MALIGN_H
#define MALIGN_H

#include "utils.h"
#include "progress.h"
#include "getopts.h"
#include "revfile.h"
#include "six.h"

void LogGenomeNames(const RevFile &Rev);
unsigned TrueCount(const std::vector<bool> &v);
coord_t GetOverlapLength(const GCSeg &SegA,const GCSeg &SegB);
coord_t GetOverlapLength(coord_t loA, coord_t hiA, coord_t loB, coord_t hiB);
void GetAlignmentVectors(const StdStrVec &Rows, unsigned RowIndexA,
  unsigned RowIndexB, IntVec &AToB, IntVec &BToA);
unsigned GetOverlappingSegIndex(const AlnInfo &AI, unsigned GenomeIndex,
  unsigned ChrId, coord_t lo, coord_t hi);
unsigned TruncateEndBlocks(const GCSeg &GuideSeg, AlnInfoX &AIX,
  StdStrVec &Rows, unsigned FileGenIndexRightOrLeft, unsigned FileIndex);
void MakeGenIndexCvtVector(const RevFile &RevA, const RevFile &RevB, IntVec &AToB);
void SetGenomeFoundVector(const RevFile &RevA, const RevFile &RevB, BoolVec &FoundInA);
void MultiAlign(const AlnInfo &GuideAI, const StdStrVec &GuideRows,
  const AIXVec &LeftAIXs, const StdStrVecVec &LeftRowsVec,
  const AIXVec &RightAIXs, const StdStrVecVec &RightRowsVec);
void Multix(const char *GuideFileName, const char *LeftFileName,
  const char *RightFileName, const char *ParentFileName);
void MultixOneGuideAln(unsigned GuideAIIndex);
void GetGenIndexes(const AlnInfo &AI, IntSet &GenIndexes);
void LogAIXs(const AIXVec &AIXs);
void LogAIRows(const AlnInfo &AI, const StdStrVec &Rows, const RevFile *Rev = 0);
void LogSix(const Six &Index);
void LogGCSegXs(const GCSegXVec &Segs);
void MakeSubAln(const AlnInfoX &AIX, unsigned GenomeIndex, unsigned ChrId, coord_t lo,
  coord_t hi, const StdStrVec &Rows, AlnInfoX &SubAIX, StdStrVec &SubRows);
const std::vector<GenInfo> &CombineGenomes(
  const std::vector<const RevFile *> &RevFiles);
unsigned CombinedGenIndexToFile(unsigned FileIndex, unsigned CombinedGenIndex);
unsigned FileGenIndexToCombined(unsigned FileIndex, unsigned FileGenIndex);
void GetCommonGenomes(unsigned FileIndexA, unsigned FileIndexB,
  IntVec &CombinedGenIndexes);
void LogCombinedGenomes();
bool SeqIsAllGaps(const std::string &Seq);
unsigned GetUngappedSeqLength(const std::string &Seq);
void ConvertGenIndexesToCombined(AlnInfo &AI, unsigned FileIndex);
bool CatchPos(const AlnInfo &AI);
bool CatchPos(const GCSeg &Seg);
bool MakeSubSeg(const GCSeg &Seg, const StdStr &Row, GCSeg &SubSeg,
  StdStr &SubRow, unsigned StartColIndex, unsigned EndColIndex);
void GetSubAlns(const RevFile &Rev, const Six &Index,
  unsigned GenIndex, unsigned ChrId, coord_t lo,
  coord_t hi, AIXVec &SubAIXs, StdStrVecVec &SubRowsVec,
  GCSegXVec &ResultSegs);
void DeleteFromSix(Six &Index, const GCSegXVec &Segs);
void WriteAllSubAlns(unsigned FileIndex, unsigned CombinedGenIndex);
void ValidateSegLengths(const AlnInfo &AI, const StdStrVec &Rows);

extern unsigned g_GuideLeftCombinedGenIndex;
extern unsigned g_GuideRightCombinedGenIndex;
extern bool g_TraceAttrs;
extern bool g_LogDetails;

extern unsigned g_RemainsCount;
extern unsigned g_LeftAlnCount;
extern unsigned g_RightAlnCount;
extern unsigned g_LeftEndBlockCount;
extern unsigned g_RightEndBlockCount;
extern unsigned g_LeftUnalignedBlockCount;
extern unsigned g_RightUnalignedBlockCount;

extern RevFile *g_ptrGuide;
extern RevFile *g_ptrLeft;
extern RevFile *g_ptrRight;
extern RevFile *g_ptrParent;

extern SixVec g_GuideSixes;

const unsigned GUIDE = 0;
const unsigned LEFT = 1;
const unsigned RIGHT = 2;

#endif // MALIGN_H
