#include <math.h>
#include "utils.h"
#include "newton.h"

double Newton(FuncObj &Func, double y, double x0, double x1, double e,
  double Trace)
	{
	if (Trace)
		{
		Log(" n     x_nminus1           x_n      x_nplus1   f(xnminus1)         f(xn)\n");
		Log("==  ============  ============  ============  ============  ============\n");
		}

	double x_nminus1 = x0;
	double x_n = x1;
	for (int n = 0; n < 10; ++n)
		{
		if (fabs(x_n - x_nminus1) <= e)
			break;
		double f_xn = Func.f(x_n);
		double f_xnminus1 = Func.f(x_nminus1);
		if (fabs(f_xn - f_xnminus1) < 1e-6)
			break;
		double x_nplus1 = x_n - (f_xn - y)*(x_n - x_nminus1)/(f_xn - f_xnminus1);
		if (Trace)
			Log("%2d  %12.5g  %12.5g  %12.5g  %12.5g  %12.5g\n",
			  n, x_nminus1, x_n, x_nplus1, f_xnminus1, f_xn);
		x_nminus1 = x_n;
		x_n = x_nplus1;
		}
	if (Trace)
		Log("root=%.10g\n", x_n);
	return x_n;
	}

struct SinObj : public FuncObj
	{
	double f(double x)
		{
		return sin(x);
		}
	} SO;

void TestNewton()
	{
	Newton(SO, 0, 3.1, 3.2, 1e-6, true);
	}
