#include "utils.h"
#include "evomodel.h"

double Pct(double, double);

double g_SubstProbs[4][4] =
	{
//	     A     C     G     T
	{    0,  0.2,  0.6,  0.2  }, // A
	{  0.2,    0,  0.2,  0.6  }, // C
	{  0.6,  0.2,    0,  0.2  }, // G
	{  0.2,  0.6,  0.2,    0  }, // T
	};
double g_CGFailProb = 0;
double g_ATFailProb = 0;

double GetSubstituteRate(double ATFract)
	{
	asserta(ATFract >= 0.0 && ATFract <= 1.0);

	double AC = P_AC_Rate;
	double AG = P_AG_Rate;
	double AT = P_AT_Rate;

	double CA = P_CA_Rate;
	double CG = P_CG_Rate;
	double CT = P_CT_Rate;

	double SumAx = AC + AG + AT;
	double SumCx = CA + CG + CT;

	double Rate = ATFract*SumAx + (1.0 - ATFract)*SumCx;
	return Rate;
	}

void LogNucMx()
	{
	static char Letters[] = "ACGT";

	Log("\n");
	Log("GCFailProb=%.3g\n", g_CGFailProb);
	Log("ATFailProb=%.3g\n", g_ATFailProb);
	Log("NucleotideSubstProbs=\n");
	Log("          A        C        G        T\n");
	for (unsigned i = 0; i < 4; ++i)
		{
		Log("%c ", Letters[i]);
		for (unsigned j = 0; j < 4; ++j)
			{
			if (i == j)
				Log("         ");
			else
				Log("  %7.5f", g_SubstProbs[i][j]);
			}
		Log("\n");
		}
	Log("\n");
	}

void CalcNucMx()
	{
	double AC = P_AC_Rate;
	double AG = P_AG_Rate;
	double AT = P_AT_Rate;

	double CA = P_CA_Rate;
	double CG = P_CG_Rate;
	double CT = P_CT_Rate;

	double SumAx = AC + AG + AT;
	double SumCx = CA + CG + CT;

	if (SumAx == 0.0 || SumCx == 0.0)
		Die("Invalid subst rates, sum A->x and/or sum C->x is zero");

	const int A = 0;
	const int C = 1;
	const int G = 2;
	const int T = 3;

	g_SubstProbs[A][A] = 0;
	g_SubstProbs[A][C] = AC/SumAx;
	g_SubstProbs[A][G] = AG/SumAx;
	g_SubstProbs[A][T] = AT/SumAx;

	g_SubstProbs[C][A] = CA/SumCx;
	g_SubstProbs[C][C] = 0;
	g_SubstProbs[C][G] = CG/SumCx;
	g_SubstProbs[C][T] = CT/SumCx;

	g_SubstProbs[G][A] = g_SubstProbs[C][T];
	g_SubstProbs[G][C] = g_SubstProbs[C][G];
	g_SubstProbs[G][G] = 0;
	g_SubstProbs[G][T] = g_SubstProbs[C][A];

	g_SubstProbs[T][A] = g_SubstProbs[A][T];
	g_SubstProbs[T][C] = g_SubstProbs[A][G];
	g_SubstProbs[T][G] = g_SubstProbs[A][C];
	g_SubstProbs[T][T] = 0;

	g_CGFailProb = 0.0;
	g_ATFailProb = 0.0;

/***
Let P = AT fail prob, assuming SumAx < SumCx:
	(total AT subs per A)/(total CG subs per C)
		= SumAx/SumCx
		= (Nr attempted Ax subs per A)/(Nr attempted Cx subs per C)
		= (1 - P)/1
So,
	P	= 1 - SumAx/SumCx
***/
	if (SumAx < SumCx)
		g_ATFailProb = 1.0 - SumAx/SumCx;
	else
		g_CGFailProb = 1.0 - SumCx/SumAx;

	LogNucMx();
	}
