#include "utils.h"
#include "evomodel.h"
#include <map>

#define T(p, t) const char *g_ParamText_##p = t;

T(	EventFailTries,			"Number of times to retry if an event fails.")

T(	MaxRateApproxErrPct,	"Maximum allowed pct. error due to approximations in computing rates.")

T(	MinAlnLength,			"Minimum length of an alignment between ancestor and evolved sequences.\n"
							"This is to avoid short alignments, if desired.")

T(	MinCopyAln,				"Minimum length of a copied fragment to be considered homologous.\n"
							"This is to avoid short alignments, if desired.")

T(	CpG_C_to_T_Ratio,		"Ratio (C->T and G->A rate outside CpG islands) / (rate inside islands).\n"
							"To indicate genome is unmethyated, set MinCpGSweeps=0.")

T(	CpGFraction,			"Maximum fraction of CpGs to mutate in a single CpG sweep.\n"
							"Should be << 1 because substitution does not commute with copy.")

T(	MinCpGSweeps,			"Minimum number of CpG sweeps. Set to zero if genome not methylated.")

T(	MaxCpGInsert,			"Maximum size of insert that becomes CpG island if inserted into island.")

T(	NCCEMeanMean,			"Mean and std. deviation of mean probabilities generated for non-coding CEs.")
T(	NCCEMeanStdDev,			"")

T(	CDSMeanMean,			"Mean and std. deviation of mean probabilities generated for CDSs.")
T(	CDSMeanStdDev,			"")

T(	MeanInterGeneFract,		"Mean fraction of space between two genes that is assigned to inter-geneic space.\n"
							"Remaining space is assiged to the two flanking genes.")

T(	MaxGeneMargin,			"Maximum number of bases from first/last transcribed base to beginning/end of gene.\n"
							"Used only by -assncces command.")

T(	StdDevInterGeneFract,	"Standard deviation of inter-gene fraction (see MeanInterGeneFract).")

T(	AccelIncrementMean,		"Mean and standard deviation of q. A random acceleration factor is a=1+q\n"
							"(accel.) or a=1/(1+q) (decel.), with equal probability of acc/decel.")
T(	AccelIncrementStdDev,	"")

T(	GeneCDSToNCEAccel,		"Accel factor to convert CDSMeanMean to gene NCE mean.")

T(	GenomeNCCEPct,			"Fraction of genome to cover with non-coding conserved elements.")

T(	CEClumpMeanLength,		"Mean and std. deviation of CE clump length (~= correlation distance).")
T(	CEClumpLengthStdDev,	"")

T(	ClumpShufflePct,		"Percentage of probabilties to be shuffled when clumping.") 

T(	AminoAcidProbMultiplier,"Amino acid substitution probabilities are multiplied by this factor.")

T(	AC_Rate,				"A->C substitution rate")
T(	AG_Rate,				"A->G substitution rate")
T(	AT_Rate,				"A->T substitution rate")
T(	CA_Rate,				"C->A substitution rate")
T(	CG_Rate,				"C->G substitution rate")
T(	CT_Rate,				"C->T substitution rate")

T(	TandemExpandRate,		"Rate of TandemExpand event (per tandem base).")
T(	TandemContractRate,		"Rate of TandemContract event (per tandem base).")
T(	ChrFuseRate,			"Rate of ChrFuse event (per genome).")
T(	ChrSplitRate,			"Rate of ChrSplit event (per genome).")
T(	NonRecipTranslocRate,	"Rate of NonRecipTransloc event (per genome).")
T(	RecipTranslocRate,		"Rate of RecipTransloc event (per genome).")

T(	CreateNontermUTRRate,	"Rate of CreateNonTermUTR event (per UTR).")
T(	CreateTermUTRRate,		"Rate of CreateTermUTR event (per UTR).")
T(	DeleteUTRRate,			"Rate of DeleteUTRRate (per UTR).")

T(	CreateCDSRate,			"Rate of CreateCDS event (per CDS).")
T(	DeleteCDSRate,			"Rate of DeleteCDS event (per CDS).")

T(	MoveUTRTermRate,		"Rate of MoveUTRTerm event (per UTR).") 

T(	MoveStartCodonIntoUTRRate,		"Rate of MoveStartCodonIntoUTR event (per gene).")
T(	MoveStartCodonIntoCDSRate,		"Rate of MoveStartCodonIntoCDS event (per gene).")
T(	MoveStopCodonIntoUTRRate,		"Rate of MoveStopCodonIntoUTR event (per gene).")
T(	MoveStopCodonIntoCDSRate,		"Rate of MoveStopCodonIntoCDS event (per gene).")

T(	MoveCDSDonorIntoIntronRate,		"Rate of MoveCDSDonorIntoIntron event (per CDS).")
T(	MoveDonorIntoCDSRate,			"Rate of MoveDonorIntoCDSRate event (per CDS).")
T(	MoveCDSAcceptorIntoIntronRate,	"Rate of MoveCDSAcceptorIntoIntronRate event (per CDS).")
T(	MoveAcceptorIntoCDSRate,		"Rate of MoveAcceptorIntoCDSRate event (per CDS).")

T(	MoveUTRDonorIntoIntronRate,		"Rate of MoveUTRDonorIntoIntron event (per UTR).")
T(	MoveDonorIntoUTRRate,			"Rate of MoveDonorIntoUTRRate event (per UTR).")
T(	MoveUTRAcceptorIntoIntronRate,	"Rate of MoveUTRAcceptorIntoIntronRate event (per UTR).")
T(	MoveAcceptorIntoUTRRate,		"Rate of MoveAcceptorIntoUTRRate event (per UTR).")

T(	ChangeGeneSpeedRate,	"Rate of ChangeGeneSpeed event (per gene).")
T(	ChangeNGESpeedRate,		"Rate of ChangeNGESpeed event (per non-gene element).")

T(	CreateNXERate,			"Rate of CreateNXE event (per non-exon element).")
T(	DeleteNXERate,			"Rate of DeleteNXE event (per non-exon element).")
T(	MoveNXERate,			"Rate of MoveNXE event (per non-exon element).")

T(	CreateNGERate,			"Rate of CreateNGE event (per non-exon element).")
T(	DeleteNGERate,			"Rate of DeleteNGE event (per non-exon element).")
T(	MoveNGERate,			"Rate of MoveNGE event (per non-exon element).")

T(	CreateCpGIslandRate,	"Rate of CreateIsland event (per CpG island).")
T(	DeleteCpGIslandRate,	"Rate of DeleteIsland event (per CpG island).")
T(	MoveCpGIslandRate,		"Rate of MoveIsland event (per CpG island).")

T(	MinMoveInvertLength,	"Minimum length of a move that can be inverted.")
T(	MinCopyInvertLength,	"Minimum length of a non-tandem copy that can be inverted.")
T(	MinTandemInvertLength,	"Minimum length of a tandem copy that can be inverted.")

T(	MoveInvertProb,			"Probability of inverting a move.")
T(	CopyInvertProb,			"Probability of inverting a non-tandem copy.")
T(	TandemInvertProb,		"Probability of inverting a tandem copy.")

T(	MinIntronLength,		"Minimum length of an intron (applied to constraint change events only).")
T(	MinTandemAnnot,			"Minimum length of a tandem repeat (for internal tracking).")
T(	MaxTandemAnnot,			"Maximum length of a tandem repeat (for internal tracking).")
T(	MinChrLength,			"Minimum chromosome length (for ChrSplit event).")
T(	MaxChrLength,			"Maximum chromosome length (for ChrFuse event).")
T(	MinChrCount,			"Minimum number of chromosomes (for ChrFuse event).")
T(	MaxChrCount,			"Maximum number of chromosome (for ChrSplit event).")
T(	MaxChrName,				"Maximum length of chromosome name (for ChrSplit and ChrFuse events).")

T(	MaxMoveStartStopCodons,	"Maximum number of codons to move when moving START or STOP codon.")
T(	MaxMoveSpliceBases,		"Maximum number of bases to move a donor or acceptor site.")

T(	MoveNGEMeanDist,		"Mean and standard deviation of distance to move a non-gene element.")
T(	MoveNGEStdDevDist,		"")

T(	MoveCpGIslandMeanDist,	"Mean and standard deviation of distance to move a CpG island.")
T(	MoveCpGIslandStdDevDist,"")

T(	MoveNXEMeanDist,		"Mean and standard deviation of distance to move a non-exon element.")
T(	MoveNXEStdDevDist,		"")

T(	PromoterCpGSlopBases,	"Maximum distance between CpG island and first exon.")

T(	CreateExonLengthSlop,	"Maximum allowed difference between target and actual newly created exon length.")

T(	MoveUTRTermMeanDist,	"Mean and standard deviation of distance to move transcription terminal.")
T(	MoveUTRTermDistStdDev,	"")

T(	MinUTRBases,			"Minimum number of bases in a 5' or 3' UTR.")
