#include <math.h>
#include "pdf.h"
#include "utils.h"

RandDist::RandDist()
	{
	m_Min_x = FLT_MAX;
	m_Max_x = FLT_MAX;
	}

void RandDist::Init(const ProbDensFunc &PDF, double Min_x, double Max_x,
  unsigned BinCount)
	{
	asserta(BinCount > 1);
	asserta(Max_x > Min_x);

	m_Min_x = Min_x;
	m_Max_x = Max_x;

	double Sum_y = 0;
	vector<double> yVec;
	for (unsigned i = 0; i < BinCount; ++i)
		{
		double x = Min_x + ((Max_x - Min_x)*i)/(BinCount - 1);
		double y = PDF.Func(x);
		asserta(y >= 0);
		yVec.push_back(y);
		Sum_y += y;
		}
	asserta(Sum_y > 0);

	double Accum = 0;
	for (unsigned i = 0; i < BinCount; ++i)
		{
		double y = yVec[i];
		if (y == 0)
			m_ThreshVec.push_back(UINT_MAX);
		else
			{
			Accum += y;
			double d = ((Accum*UINT_MAX)/Sum_y);
			double a = d + 0.5;
			unsigned Thresh = (unsigned) a;
			m_ThreshVec.push_back((unsigned) Thresh);
			}
		}
	}

double RandDist::Get() const
	{
	unsigned r = RandInt(UINT_MAX);
	const unsigned BinCount = SIZE(m_ThreshVec);
	asserta(BinCount > 1);
	for (unsigned i = 0; i < BinCount; ++i)
		{
//		if (r <= m_ThreshVec[i] && m_ThreshVec[i] != UINT_MAX)
		if (r <= m_ThreshVec[i])
			{
			double d = (m_Max_x - m_Min_x);
			return m_Min_x + i*d/(BinCount - 1);
			}
		}
//	return m_Max_x;
	Die("RandDist::Get failed");
	UNREACHABLE_RETURN(0);
	}

void RandDist::Test(unsigned SampleCount, unsigned SampleBinCount) const
	{
	vector<unsigned> Counts(SampleBinCount, 0);
	unsigned MaxCount = 0;
	double Sum = 0;
	for (unsigned i = 0; i < SampleCount; ++i)
		{
		double x = Get();
		asserta(x >= m_Min_x && x <= m_Max_x);
		Sum += x;
		double r = (x - m_Min_x)/(m_Max_x - m_Min_x); // 0.0 .. 1.0
		asserta(r >= 0.0 && r <= 1.0);
		unsigned SampleBinIndex = (unsigned) (r*SampleBinCount);
		if (SampleBinIndex == SampleBinCount)
			--SampleBinIndex;
		unsigned NewCount = ++(Counts[SampleBinIndex]);
		MaxCount = max(NewCount, MaxCount);
		}

	Log("\n");
	Log("     From           To       Count\n");
	Log("==========  ==========  ==========\n");
	for (unsigned i = 0; i < SampleBinCount; ++i)
		{
		double f = m_Min_x + i*(m_Max_x - m_Min_x)/SampleBinCount;
		double t = m_Min_x + (i + 1)*(m_Max_x - m_Min_x)/SampleBinCount;
		unsigned Count = Counts[i];
		Log("%10.4g  %10.4g  %10u  ", f, t, Count);
		unsigned Stars = (unsigned) ((Count*64)/MaxCount + 0.5);
		for (unsigned j = 0; j < Stars; ++j)
			Log("*");
		Log("\n");
		}
	Log("Mean=%.4g\n", Sum/SampleCount);
	Log("\n");
	}

double RandDist::ComputeMean(unsigned SampleCount) const
	{
	double Sum = 0;
	for (unsigned i = 0; i < SampleCount; ++i)
		{
		double x = Get();
		asserta(x >= m_Min_x && x <= m_Max_x);
		Sum += x;
		}
	return Sum/SampleCount;
	}

void RandDist::LogBins(unsigned N, unsigned BinCount) const
	{
	vector<double> v;
	for (unsigned i = 0; i < N; ++i)
		v.push_back(Get());

	vector<unsigned> Bins(BinCount);
	for (unsigned i = 0; i < BinCount; ++i)
		Bins[i] = 0;

	unsigned MaxCount = 0;
	double Sum = 0;
	double Sum2 = 0;
	double Range = m_Max_x - m_Min_x;
	if (Range == 0)
		Die("LogBins, range = 0");

	for (unsigned i = 0; i < N; ++i)
		{
		double x = v[i];
		asserta(x >= m_Min_x && x <= m_Max_x);

		Sum += x;
		Sum2 += x*x;

		double f = (x - m_Min_x)/Range;
		asserta(f >= 0.0 && f <= 1.0);
		unsigned BinIndex = (unsigned) (f*BinCount + 0.5);
		if (BinIndex >= BinCount)
			BinIndex = BinCount - 1;

		unsigned Count = Bins[BinIndex] + 1;
		Bins[BinIndex] = Count;
		if (Count > MaxCount)
			MaxCount = Count;
		}

	double Mean = Sum/N;
	double StdDev = sqrt(Sum2/N - Mean*Mean);

	Log("\n");
	Log("Mean %.4f StdDev %.4f\n", Mean, StdDev);
	double BinLength = Range/BinCount;
	for (unsigned i = 0; i < BinCount; ++i)
		{
		unsigned Count = Bins[i];
		double lox = m_Min_x + i*BinLength;
		double hix = m_Min_x + (i + 1)*BinLength;
		if (m_Min_x == 0 && m_Max_x == 1.0)
			Log("%6.2f- %6.2f  %7u  ", lox, hix, Count);
		else
			Log("%10.3g- %10.3g  %7u  ", lox, hix, Count);

		unsigned L = (Count*32)/MaxCount;
		for (unsigned j = 0; j < L; ++j)
			Log("*");

		Log("\n");
		}
	}
