#ifndef ratemgr_h
#define ratemgr_h

#include "utils.h"
#include "eventresult.h"

using namespace std;

enum RATE_TYPE
	{
#define R(r)	RT_##r,
#include "ratetypes.h"
	};

const unsigned RateTypeCount = 0 +
#define R(r)	+ 1
#include "ratetypes.h"
	;

const char *RateTypeToStr(RATE_TYPE Type);
bool IsInterChrRateType(RATE_TYPE Type);

#ifndef __APPLE_CC__
#pragma pack(push,1)
#endif

struct RateRange
	{
	gpos_t LoLength;
	gpos_t HiLength;
	double LoRate;
	double HiRate;
	double TotalRate;
	double BinRate;
	double BinErr;
	unsigned RandMark;

	RateRange()
		{
		LoLength = GPOS_T_MAX;
		HiLength = GPOS_T_MAX;
		LoRate = DBL_MAX;
		HiRate = DBL_MAX;
		TotalRate = DBL_MAX;
		BinRate = DBL_MAX;
		RandMark = UINT_MAX;
		}

	void LogMe() const
		{
		Log(" Len %"PF" - %"PF", Rate %.3g - %.3g",
		  LoLength, HiLength, LoRate, HiRate);
		}
	};

#ifndef __APPLE_CC__
#pragma pack(pop)
#endif

struct RateStats
	{
	double TotalRate;
	double Rate1to10;
	double Rate1to99;
	double Rate100plus;
	double Bases1to10;
	double Bases1to99;
	double Bases100plus;

	RateStats()
		{
		TotalRate = DBL_MAX;
		Rate1to10 = DBL_MAX;
		Rate1to99 = DBL_MAX;
		Rate100plus = DBL_MAX;
		Bases1to10 = DBL_MAX;
		Bases1to99 = DBL_MAX;
		Bases100plus = DBL_MAX;
		}
	};

typedef pair<gpos_t, double> LRPair;
typedef vector<LRPair> RateVec;
typedef list<LRPair> RateList;
typedef vector<RateRange> RRVec;

class RateMgr
	{
public:
	RateMgr();
	void Clear();
	void LogMe() const;
	void LogParams() const;
	void Init(RATE_TYPE Type, const RateVec &Rates);
	gpos_t GetLength() const;
	void ToFile(FILE *f) const;
	void FromFile(FILE *f);

	double GetTotalRate() const;
	void GetStats(RateStats &Stats) const;
	double GetRate(gpos_t Length) const;
	RATE_TYPE GetType() const;
	const char *TypeStr() const;

	void StoreResult(EVENT_RESULT Result, gpos_t Length);
	const unsigned *GetResultCounts(EVENT_RESULT ER) const
		{
		return m_ResultCounts[ER];
		}
	unsigned GetTotalAcceptedBases() const
		{
		return m_TotalAcceptedBases;
		}
	unsigned GetTotalRejectedBases() const
		{
		return m_TotalRejectedBases;
		}

private:
	void MakeTrimmedRates();
	void InsertInterpolatedRate(RateList &Rates, gpos_t Length);
	void MakeRanges();
	void SetBinRates();

private:
	RATE_TYPE m_Type;
	RateVec m_Rates;
	RateList m_TrimmedRates;
	RRVec m_Ranges;
	unsigned m_TotalAcceptedBases;
	unsigned m_TotalRejectedBases;
	unsigned m_ResultCounts[3][16];
	};

#endif // ratemgr_h
