#include "utils.h"
#include "progress.h"
#include <string>
#include <errno.h>
#include "readfasta.h"

void ReadFasta(const char *FileName, ptrFnOnSeq OnSeq, ptrFnOnLabel OnLTLabel)
	{
	FILE *f = OpenStdioFile(FileName);
	off64_t FileSize = GetStdioFileSize(f);

	std::string SeqLabel;
	std::string GroupLabel;
	std::string Seq;
	Seq.reserve((size_t) FileSize);
	ProgressStartFile(f, "Read %s", FileName);
	off64_t Counter = 0;
	unsigned SeqCount = 0;

	enum
		{
		Start,
		Label,
		LTLabel,
		SeqData,
		SeqDataNL,
		} State = Start;

	for (;;)
		{
		if (Counter++%10000 == 0)
			ProgressStepFile();

		int Letter = fgetc(f);
		if (Letter < 0)
			{
			if (feof(f))
				break;
			else
				Die("Error reading FASTA file, errno=%d", errno);
			}

		switch (State)
			{
		case Start:
			if (Letter == '\n' || Letter == '\r')
				continue;
			if (Letter == '>')
				{
				State = Label;
				continue;
				}
			else if (Letter == '<')
				{
				State = LTLabel;
				continue;
				}
			else
				{
				if (isprint(Letter))
					Die("Expected '>' at start of FASTA file %s, got '%c'",
					  FileName, Letter);
				else
					Die("Expected '>' at start of FASTA file %s, got '0x%02x'",
					  FileName, (unsigned char) Letter);
				}

		case Label:
			if (Letter == '\n' || Letter == '\r')
				{
				if (SeqLabel.empty())
					Die("Empty label in FASTA file, replaced by '?'");

				State = SeqData;
				SetProgressMsg("Reading %s [%.10s]", FileName, SeqLabel.c_str());
				ProgressStepFile();
				continue;
				}
			else
				SeqLabel.push_back((char) Letter);
			continue;

		case LTLabel:
			if (Letter == '\n' || Letter == '\r')
				{
				if (OnLTLabel != 0)
					OnLTLabel(GroupLabel);
				State = SeqDataNL;
				continue;
				}
			else
				GroupLabel.push_back((char) Letter);
			continue;

		case SeqDataNL:
			if (Letter == '<')
				{
				State = LTLabel;
				GroupLabel.clear();
				}
			else if (Letter == '>')
				State = Label;
			if (Letter == '>' || Letter == '<')
				{
				if (!Seq.empty() || !SeqLabel.empty())
					{
					OnSeq(SeqLabel, Seq);
					++SeqCount;
					}

				SeqLabel.clear();
				Seq.clear();
				//Seq.reserve(10*1024*1024);
				continue;
				}
		// Fall through to case SeqData by design.
		case SeqData:
			if (Letter == '\r' || Letter == '\n')
				{
				State = SeqDataNL;
				continue;
				}
			else if (isspace(Letter))
				continue;
			else
				{
				Seq.push_back((char) Letter);
				State = SeqData;
				continue;
				}

		default:
			asserta(false);
			}
		}

	if (!Seq.empty())
		{
		OnSeq(SeqLabel, Seq);
		++SeqCount;
		}

	SetProgressMsg("Read %s, %u seqs", FileName, SeqCount);
	ProgressDone();

	CloseStdioFile(f);
	}
