#include "utils.h"
#include "getopts.h"
#include "revfile.h"
#include "progress.h"

static const char *g_Prefix;
static const char *g_Suffix;

static char *MakeFileName(unsigned FileIndex)
	{
	size_t len = strlen(g_Prefix) + strlen(g_Suffix) + 64;
	char *buff = myalloc(char, len);
	sprintf(buff, "%s%u%s", g_Prefix, FileIndex + 1, g_Suffix);
	return buff;
	}

void SplitRev()
	{
	const char *InputFileName = RequiredValueOpt("splitrev");

	unsigned OutFileCount = RequiredIntOpt("n");

	g_Prefix = ValueOpt("prefix");
	g_Suffix = ValueOpt("suffix");
	if (g_Prefix == 0)
		g_Prefix = "";
	if (g_Suffix == 0)
		g_Suffix = "";

	if (OutFileCount == 0)
		Die("Must specify > 0 output files");

// Open input file
	RevFile In;
	In.Open(InputFileName);

	const std::vector<GenInfo> &Genomes = In.GetGenomes();

	unsigned AICount = In.GetAlnInfoCount();
	unsigned AICountPerFile = AICount/OutFileCount;
	if (AICountPerFile == 0)
		Die("Too many output files / too few AIs, AI's per file = 0");

	for (unsigned OutFileIndex = 0; OutFileIndex < OutFileCount; ++OutFileIndex)
		{
		char *FileName = MakeFileName(OutFileIndex);
		RevFile Out;
		Out.Create(FileName, Genomes);

		unsigned StartAIIndex = AICountPerFile*OutFileIndex;
		unsigned EndAIIndex = StartAIIndex + AICountPerFile - 1;
		if (EndAIIndex >= AICount)
			EndAIIndex = AICount - 1;

		for (unsigned AIIndex = StartAIIndex; AIIndex <= EndAIIndex; ++AIIndex)
			{
			AlnInfo AI;
			unsigned AlnIndex = In.GetAlnInfo(AIIndex, AI);
			if (AlnIndex != UINT_MAX)
				{
				StdStrVec Rows;
				In.GetAlnRows(AlnIndex, Rows);
				Out.AddAln(AI, Rows);
				}
			else
				Out.AddAlnInfo(AI);
			}
		}
	}
