#include "revfile.h"
#include "getopts.h"
#include "progress.h"
#include "utils.h"

void StripParalogs()
	{
	const char *InFileName = RequiredValueOpt("stripparalogs");
	const char *OutFileName = RequiredValueOpt("out");

	Progress("Opening %s", InFileName);
	RevFile In;
	In.Open(InFileName);

	Progress("Creating %s", OutFileName);
	RevFile Out;
	const std::vector<GenInfo> &Genomes = In.GetGenomes();
	Out.Create(OutFileName, Genomes);

	unsigned AICount = In.GetAlnInfoCount();
	ProgressStart("Processing %u AIs", AICount);
	for (unsigned AIIndex = 0; AIIndex < AICount; ++AIIndex)
		{
		if (AIIndex%1000 == 0)
			ProgressStep(AIIndex, AICount);
		AlnInfo AI;
		StdStrVec Rows;
		In.GetAlnRows(In.GetAlnInfo(AIIndex, AI), Rows);
		unsigned Para = AI.GetAttr(ALN_ATTR_Paralog);
		if (Para == UINT_MAX || Para == 0)
			Out.AddAln(AI, Rows);
		}
	ProgressDone();

	Progress("Finalizing %s", OutFileName);
	In.Close();
	Out.Close();
	}
