#include "nw.h"

#undef DPM
#undef DPD
#undef DPI

#define TRACE	0

void Band(unsigned i, unsigned LA, unsigned LB, unsigned r,
   unsigned *ptrMidj, unsigned *ptrMinJ, unsigned *ptrMaxJ);

score_t TraceBackBanded(const unsigned char *A, unsigned LA, const unsigned char *B,
  unsigned LB, unsigned r, score_t **DPM, score_t **DPD, score_t **DPI, std::string &Path)
	{
#if	TRACE
	Log("\n");
	Log("TraceBackBanded LengthA=%u LengthB=%u\n", LA, LB);
#endif
	assert(LB > 0 && LA > 0);

	Path.clear();

	unsigned i = LA - 1;
	unsigned j = LB - 1;

	const score_t M = DPM[i][r];
	const score_t D = DPD[i][r];
	const score_t I = DPI[i][r];

	score_t Max = M;
	char Edge = 'M';
	if (D > Max)
		{
		Max = D;
		Edge = 'D';
		}
	if (I > Max)
		{
		Max = I;
		Edge = 'I';
		}

#if	TRACE
	Log("TraceBackBanded Max=%d Edge=%c\n\n", Max, Edge);
#endif

	for (;;)
		{
#if	TRACE
		Log("\n");
		Log("i=%u j=%u %c\n", i, j, Edge);
#endif
		Path.push_back(Edge);

	// Current cell is DP<Edge>[i][j]
	// How did we get here?
		if (i == 0 && Edge == 'M')
			{
			while (j > 0)
				{
				Path.push_back('I');
				--j;
				}
			break;
			}

		if (j == 0 && Edge == 'M')
			{
			while (i > 0)
				{
				Path.push_back('D');
				--i;
				}
			break;
			}

//		assert(i > 0 && j > 0);

		unsigned midj;
		unsigned minJ;
		unsigned maxJ;

		unsigned Prev_midj;
		unsigned Prev_minJ;
		unsigned Prev_maxJ;

		Band(i, LA, LB, r, &midj, &minJ, &maxJ);
		Band(i-1, LA, LB, r, &Prev_midj, &Prev_minJ, &Prev_maxJ);

		const unsigned RowShift = midj - Prev_midj;
		const unsigned J = j + r - midj;
		const unsigned Prev_J = J + RowShift;
#if	TRACE
		Log("Band(i=%u, LA=%u, LB=%u, r=%u, midj=%u, minJ=%u, maxJ=%u)\n",
		  i, LA, LB, r, midj, minJ, maxJ);
		Log("Band(i-1=%u, LA=%u, LB=%u, r=%u, Prev_midj=%u, Prev_minJ=%u, Prev_maxJ=%u)\n",
		  i-1, LA, LB, r, Prev_midj, Prev_minJ, Prev_maxJ);
		Log("RowShift=%u, J=%u, Prev_J=%u\n", RowShift, J, Prev_J);
#endif
		assert(midj >= Prev_midj);

		switch (Edge)
			{
		case 'M':
			{
			assert(i > 0);
			assert(Prev_J >= Prev_minJ + 1);
			assert(Prev_J <= Prev_maxJ + 1);

			const unsigned char cA = A[i];
			const unsigned char cB = B[j];

			score_t Score = DPM[i][J];
			score_t Subst = SUBST(cA, cB);

			score_t MM = MINUS_INF;
			score_t DM = MINUS_INF;
			score_t IM = MINUS_INF;
			if (i == 0)
				{
				if (j == 0)
					MM = 0;
				else
					IM = TERMGAP + (j - 1)*GAPEXTEND;
				}
			else
				{
				if (j > 0)
					{
				// This test is Prev_J - 1 >= Prev_minJ, move 1
				// to rhs because unsigned.
					if (Prev_J >= Prev_minJ + 1)
						{
					// move 1 to rhs here also.
						if (Prev_J <= Prev_maxJ + 1)
							{
							MM = DPM[i-1][Prev_J - 1];
							DM = DPD[i-1][Prev_J - 1];
							IM = DPI[i-1][Prev_J - 1];
							}
						}
					}
				else
					{
					assert(i > 0 && j == 0);
					DM = TERMGAP + (i - 1)*GAPEXTEND;
					}
				}

			MM += Subst;
			DM += Subst;
			IM += Subst;
#if	TRACE
			Log("M: Score=%s ", ScoreToStr(Score));
			Log(" MM=%s", ScoreToStr(MM));
			Log(" DM=%s", ScoreToStr(DM));
			Log(" IM=%s\n", ScoreToStr(IM));
#endif
			if (EQ(MM, Score))
				Edge = 'M';
			else if (EQ(DM, Score))
				Edge = 'D';
			else if (EQ(IM, Score))
				Edge = 'I';
			else
				Die("TraceBackBanded, Failed to match M[%u][j=%u,J=%u]=%d, MM=%d DM=%d IM=%d",
				  i, j, J, Score, MM, DM, IM);

			--i;
			--j;
			break;
			}

		case 'D':
			{
			assert(i > 0);
			if (Prev_J > Prev_maxJ)
				{
				Log("A[%u]=", LA);
				for (unsigned i = 0; i < LA; ++i)
					Log("%c", A[i]);
				Log("\n");
				Log("B[%u]=", LB);
				for (unsigned i = 0; i < LB; ++i)
					Log("%c", B[i]);
				Log("\n");
				Die("Prev_J=%u > Prev_maxJ=%u", Prev_J, Prev_maxJ);
				}
			if (Prev_J < Prev_minJ)
				{
				Log("A[%u]=", LA);
				for (unsigned i = 0; i < LA; ++i)
					Log("%c", A[i]);
				Log("\n");
				Log("B[%u]=", LB);
				for (unsigned i = 0; i < LB; ++i)
					Log("%c", B[i]);
				Log("\n");
				Die("Prev_J=%u < Prev_minJ=%u", Prev_J, Prev_minJ);
				}

			const score_t Score = DPD[i][J];
			score_t MD = DPM[i-1][Prev_J] + (j == LB - 1 ? TERMGAP : GAPOPEN);
			score_t DD = DPD[i-1][Prev_J] + GAPEXTEND;

#if	TRACE
			Log("D: Score=%s ", ScoreToStr(Score));
			Log(" MD=%s", ScoreToStr(MD));
			Log(" DD=%s", ScoreToStr(DD));
#endif
			if (EQ(MD, Score))
				Edge = 'M';
			else if (EQ(DD, Score))
				Edge = 'D';
			else
				Die("TraceBackBanded, Failed to match D");

			--i;
			break;
			}

		case 'I':
			{
			assert(J > 0);

			const score_t Score = DPI[i][J];
			score_t MI = DPM[i][J-1] + (i == LA - 1 ? TERMGAP : GAPOPEN);
			score_t II = DPI[i][J-1] + GAPEXTEND;

#if	TRACE
			Log("Score=%d MI=%d II=%d\n", Score, MI, II);
#endif
			if (EQ(MI, Score))
				Edge = 'M';
			else if (EQ(II, Score))
				Edge = 'I';
			else
				Die("TraceBackBanded, Failed to match I");

			--j;
			break;
			}

		default:
			assert(false);
			}
		}

	std::reverse(Path.begin(), Path.end());
#if	TRACE
	{
	score_t sp = ScorePath(A, LA, B, LB, Path.c_str());
	if (!EQ(sp, Max))
		{
		Log("Path=%s\n", Path.c_str());
		Die("TraceBackBanded: scoreMax != ScorePath");
		}
	}
#endif
	return Max;
	}
