#include "evolver.h"
#include "revfile.h"
#include "getopts.h"
#include "progress.h"
#include "myrand.h"

void LogERDetails();
void SetModel(const char *FileName);

extern PTR_TRY_EVENT_FUNC TryEventFuncs[];

const char *EventResultToStr(EVENT_RESULT Result)
	{
	switch (Result)
		{
	case ER_Fail: return "Fail";
	case ER_Reject: return "Reject";
	case ER_Accept: return "Accept";
	default:
		break;
		}
	return "EVENT_RESULT_???";
	}

EVENT_TYPE EventNameToType(const string &Name)
	{
	for (unsigned iEvent = 0; iEvent < EventTypeCount; ++iEvent)
		{
		EVENT_TYPE e = (EVENT_TYPE) iEvent;
		if (Name == EventTypeToStr(e))
			return e;
		}
	Die("Unknown event '%s'", Name.c_str());
	return EVENT_TYPE_Invalid;
	}

void TestEvent()
	{
	const char *EventName = RequiredValueOpt("testevent");
	const char *InSeqFileName = ValueOpt("seq");
	const char *InEvoFileName = ValueOpt("evo");
	const char *InAnnotFileName = ValueOpt("annots");
	const char *ModelFile = ValueOpt("model");
	unsigned GenomeIndex = 0;
	IntOpt("genix", &GenomeIndex);
	unsigned ChrId = 0;
	IntOpt("chrid", &ChrId);
	const char *InChrName = ValueOpt("chrname");
	unsigned Tries = 10;
	IntOpt("events", &Tries);
	unsigned ValEvents = 0;
	unsigned StartVal = 0;
	IntOpt("valevents", &ValEvents);
	IntOpt("startval", &StartVal);
	unsigned TraceTry = UINT_MAX;
	IntOpt("trace", &TraceTry);
	EVENT_TYPE Event = EventNameToType(EventName);

	if (StartVal > 0 && ValEvents == 0)
		ValEvents = 1;

	SetModel(ModelFile);
	InitRand();

	Evolver Evo;

	if (InSeqFileName != 0)
		{
		if (InEvoFileName)
			Die("Both -evo and -seq");

		RevFile In;
		In.Open(InSeqFileName);

		if (InChrName != 0)
			ChrId = In.GetChrId(GenomeIndex, InChrName);
		else
			InChrName = In.GetChrName(GenomeIndex, ChrId).c_str();

		Evo.FromRevFile(In, GenomeIndex, ChrId, true);
		In.Close();

		if (InAnnotFileName != 0)
			Evo.GetAnnots(InAnnotFileName, InChrName);
		}
	else if (InEvoFileName != 0)
		Evo.FromFile(InEvoFileName, true);
	else
		Die("Must specify -seq or -evo");

	Evo.Validate(true);

	Evo.ComputeEventRates(0.5);

// Don't waste time validating if not accepted
	bool DueToValidate = false;
	for (unsigned Try = 1; Try <= Tries; ++Try)
		{
		SetTrace(Try == TraceTry);
		if (Tries > 1 && Try - 1 == Tries/2)
			{
			Evo.Validate(true);
			Progress("Invert");
			Evo.Invert(0, Evo.GetAncestralLength());
			Evo.Validate(true);
			}

		EVENT_RESULT Result = (TryEventFuncs[Event])(Evo);
		Progress("%s [%u of %u] %s", EventName, Try, Tries, EventResultToStr(Result));
		if (ValEvents != 0 && Try%ValEvents == 0 && Try >= StartVal)
			DueToValidate = true;

		if (Result == ER_Accept && DueToValidate)
			{
			Evo.Validate(true);
			DueToValidate = false;

			const char *FileName = ValueOpt("savegood");
			if (FileName != 0)
				Evo.ToFile(FileName);
			}
		}

	Evo.Validate(true);
	}
