#include "evolver.h"

#define	LOG	if (Show) Log

void TestMicro()
	{
	Evolver Evo;

	bool Show = true;

	{
	string Seq;
	Evo.FromString("GATTACA");
	Evo.Validate();

	Evo.Delete(2, 1);
	Evo.Validate();

	Evo.ToString(Seq);
	LOG("After delete(2,2): %s\n", Seq.c_str());
	Evo.Validate();
	assert(Seq == "GATACA");
	}

	{
	string Seq;
	Evo.FromString("GATTACA");
	Evo.Validate();

	Evo.Delete(2, 2);
	Evo.Validate();

	Evo.ToString(Seq);
	LOG("After delete(2,2): %s\n", Seq.c_str());
	Evo.Validate();
	assert(Seq == "GAACA");
	}

	{
	string Seq;
	Evo.FromString("GATTACA");
	Evo.Validate();

	Evo.Delete(0, 2);
	Evo.Validate();

	Evo.ToString(Seq);
	LOG("After delete(0,2): %s\n", Seq.c_str());
	assert(Seq == "TTACA");
	}

	{
	string Seq;
	Evo.FromString("GATTACA");
	Evo.Validate();

	Evo.Delete(5, 2);
	Evo.Validate();

	Evo.ToString(Seq);
	LOG("After delete(5,2): %s\n", Seq.c_str());
	assert(Seq == "GATTA");
	}

	{
	string Seq;
	Evo.FromString("GATTACA");
	Evo.Validate();

	Evo.Insert(3, "ACT");
	Evo.Validate();

	Evo.ToString(Seq);
	LOG("After insert(3,ACT): %s\n", Seq.c_str());
	assert(Seq == "GATTACTACA");
	}

	{
	string Seq;
	Evo.FromString("GATTACA");
	Evo.Validate();

	Evo.Insert(1, "A");
	Evo.Validate();

	Evo.ToString(Seq);
	LOG("After insert(*,1,0): %s\n", Seq.c_str());
	assert(Seq == "GAATTACA");
	}

	{
	string Seq;
	Evo.FromString("GATTACA");
	Evo.Validate();

	Evo.Insert(6, "ACT");
	Evo.Validate();

	Evo.ToString(Seq);
	LOG("After insert(6, ACT): %s\n", Seq.c_str());
	assert(Seq == "GATTACAACT");
	}

	{
	string Seq;
	Evo.FromString("GATTACA");
	Evo.Validate();

	Evo.Invert(3, 1);
	Evo.Validate();

	Evo.ToString(Seq);
	LOG("After invert(3,1): %s\n", Seq.c_str());
	assert(Seq == "GATAACA");
	}

	{
	string Seq;
	Evo.FromString("GATTACA");
	Evo.Validate();

	Evo.Invert(3, 3);
	Evo.Validate();

	Evo.ToString(Seq);
	LOG("After invert(3,3): %s\n", Seq.c_str());
	assert(Seq == "GATGTAA");
	}

	{
	string Seq;
	Evo.FromString("GATTACA");
	Evo.Validate();

	Evo.Invert(2, 4);
	Evo.Validate();

	Evo.ToString(Seq);
	LOG("After invert(2,4,*): %s\n", Seq.c_str());
	assert(Seq == "GAGTAAA");
	}

	{
	string Seq;
	Evo.FromString("GATTACA");
	Evo.Validate();

	Evo.Invert(0, 7);
	Evo.Validate();

	Evo.ToString(Seq);
	LOG("After invert(0,7,*): %s\n", Seq.c_str());
	assert(Seq == "TGTAATC");
	}

	{
	string Seq;
	Evo.FromString("GATTACA");
	Evo.Validate();

	Evo.Move(2, 5, 2);
	Evo.Validate();

	Evo.ToString(Seq);
	LOG("After move(2,5,2): %s\n", Seq.c_str());
	assert(Seq == "GAACTTA");
	}

	{
	string Seq;
	Evo.FromString("GATTACA");
	Evo.Validate();

	Evo.Move(0, 5, 2);
	Evo.Validate();

	Evo.ToString(Seq);
	LOG("After move(0,2,5): %s\n", Seq.c_str());
	assert(Seq == "TTACGAA");
	}

	{
	string Seq;
	Evo.FromString("GATTACA");
	Evo.Validate();

	Evo.Move(0, 6, 2);
	Evo.Validate();

	Evo.ToString(Seq);
	LOG("After move(0,2,6): %s\n", Seq.c_str());
	Evo.Validate();
	assert(Seq == "TTACAGA");
	}

	{
	string Seq;
	Evo.FromString("GATTACA");
	Evo.Validate();

	vector<gpos_t> Genes;
	vector<gpos_t> CopiedGenes;
	Evo.Copy(3, 5, 2, Genes, CopiedGenes);
	Evo.Validate();

	Evo.ToString(Seq);
	LOG("After copy(3,2,5): %s\n", Seq.c_str());
	assert(Seq == "GATTACTAA");
	}

	{
	string Seq;
	Evo.FromString("GATTACA");
	Evo.Validate();

	vector<gpos_t> Genes;
	vector<gpos_t> CopiedGenes;
	Evo.Copy(3, 0, 2, Genes, CopiedGenes);
	Evo.Validate();

	Evo.ToString(Seq);
	LOG("After copy(3,2,0): %s\n", Seq.c_str());
	assert(Seq == "GTAATTACA");
	}

	{
	string Seq;
	Evo.FromString("GATTACAGATTACAGATTACA");
	Evo.Validate();

	Evo.Invert(3, 3);
	Evo.Validate();

	Evo.ToString(Seq);
	LOG("After invert(3,3): %s\n", Seq.c_str());

	Evo.Delete(10, 3);
	Evo.Validate();

	Evo.ToString(Seq);
	LOG("After delete(10,3): %s\n", Seq.c_str());

	Evo.Insert(16, "CCC");
	Evo.Validate();

	Evo.ToString(Seq);
	LOG("After insert(16, CCC): %s\n", Seq.c_str());
	}
	}
