#define TRACE	0

#include "evolver.h"
#include "ratemgr.h"
#include "myrand.h"
#include "evomodel.h"

bool IsStopCodon(const string &Codon);

EVENT_RESULT Evolver::Try_DeleteCDS()
	{
	vector<GeneInfo> Genes;
	vector<double> MACs;
	InitTryGenes(Genes, MACs);

	GeneInfo Gene;
	unsigned ExonIndexToDelete = UINT_MAX;
	for (;;)
		{
		bool Any = NextTryGene(Genes, MACs, Gene);
		if (!Any)
			EV_FAILN(DeleteCDS, NoCDSs)

		ExonIndexToDelete = GetCandDeleteCDSExonIndex(Gene);
		if (ExonIndexToDelete != UINT_MAX)
			break;
		}
	if (ExonIndexToDelete == UINT_MAX)
		EV_FAILN(DeleteCDS, NoCDSs)

#if	TRACE
	Log("\n");
	Log("DeleteCDS, Gene:\n");
	LogGene(Gene);
	Log("ExonIndexToDelete=%u\n", ExonIndexToDelete);
#endif

	asserta(ExonIndexToDelete < SIZE(Gene.Exons));
	const ExonInfo &Exon = Gene.Exons[ExonIndexToDelete];
	asserta(Exon.AllCoding);
	gpos_t Pos = Exon.Pos;
	m_CurrentEventPos = Pos;

// Add 2 to include following splice site
	for (gpos_t i = 0; i < Exon.Length + 2; ++i)
		{
		BaseInfo &Base = GetBase(Pos);
		Base.SetIntron();
		Base.Deconstrain();

		Pos = GetNextPos(Pos);
		}

// Preceding splice site
	Pos = GetPrevPos(Exon.Pos);
	BaseInfo &Base1 = GetBase(Pos);
	Base1.SetIntron();
	Base1.Deconstrain();

	Pos = GetPrevPos(Pos);
	BaseInfo &Base2 = GetBase(Pos);
	Base2.SetIntron();
	Base2.Deconstrain();

	GeneInfo NewGene;
	GetGene(Pos, NewGene);
#if	TRACE
	Log("New gene:\n");
	LogGene(NewGene);
#endif
	unsigned ErrCount = ValidateGene(NewGene);
	if (ErrCount > 0)
		Die("DeleteCDS: Validate gene failed");

//	EV_ACCEPT(DeleteCDS, CDS.Pos, CDS.Length)
	EV_ACCEPTN(DeleteCDS, Exon.Pos)
	}

// Candidate criteria:
//		- doesn't contain any bases from start or stop codon
//		- multiple of 3 bases
//		- does not create STOP codon as side effect of deletion
unsigned Evolver::GetCandDeleteCDSExonIndex(const GeneInfo &Gene) const
	{
#if	TRACE
	Log("\n");
	Log("GetCandDeleteCDSExonIndex, Gene=\n");
	Gene.LogMe();
#endif
	const unsigned CDSCount = SIZE(Gene.CDSs);
	if (CDSCount < 3)
		return UINT_MAX;

	const unsigned ExonCount = SIZE(Gene.Exons);
	vector<unsigned> ExonIndexes;
	GetRandomizedExonIndexes(Gene, ExonIndexes);
	asserta(SIZE(ExonIndexes) == ExonCount);

	bool Strand = Gene.Plus;
	for (unsigned k = 0; k < ExonCount; ++k)
		{
		unsigned ExonIndex = ExonIndexes[k];
#if	TRACE
		Log("Try exon %u\n", ExonIndex);
#endif
		if (ExonIndex == 0 || ExonIndex == ExonCount - 1)
			continue;

		const ExonInfo &Exon = Gene.Exons[ExonIndex];
		if (!Exon.AllCoding)
			continue;
#if TRACE
		Log("Exon: ");
		Exon.LogMe();
		Log("\n");
#endif

		unsigned CDSIndex = Exon.CDSIndex;
		if (CDSIndex == 0 || CDSIndex + 1 == CDSCount)
			continue;

		const CDSInfo &CDS = Gene.CDSs[CDSIndex];
		if (CDS.Length%3 != 0)
			continue;

		unsigned PrevExonIndex = (Strand ? ExonIndex - 1 : ExonIndex + 1);
		unsigned NextExonIndex = (Strand ? ExonIndex + 1 : ExonIndex - 1);

		asserta(PrevExonIndex < ExonCount);
		asserta(NextExonIndex < ExonCount);

		const ExonInfo &PrevExon = Gene.Exons[PrevExonIndex];
		const ExonInfo &NextExon = Gene.Exons[NextExonIndex];

		string FractCodon1;
		string FractCodon2;
		bool StartStop = false;
		GetFractCodonExonEnd(PrevExon, FractCodon1, Strand, StartStop);
		if (StartStop)
			return UINT_MAX;
		GetFractCodonExonStart(NextExon, FractCodon2, Strand, StartStop);
		if (StartStop)
			return UINT_MAX;

		unsigned SumLengths = SIZE(FractCodon1) + SIZE(FractCodon2);
		asserta(SumLengths == 0 || SumLengths == 3);
		if (SumLengths == 3 && IsStopCodon(FractCodon1 + FractCodon2))
			continue;

#if	TRACE
		Log("FOUND, ExonIndex=%u\n", ExonIndex);
#endif
		return ExonIndex;
		}
	return UINT_MAX;
	}
