#include "ice.h"
#include "ratemgr.h"
#include "evomodel.h"
#include "myrand.h"

#define TRACE	0

extern RateMgr *g_InterChrMoveRateMgr;
extern unsigned g_InvertableMoveCount;
extern unsigned g_InvertedMoveCount;
extern unsigned g_TotalMoveCount;

EVENT_RESULT InterChrEvolver::Try_InterChrMove()
	{
	if (GetChrCount() == 1)
		EV_FAILN(InterChrMove, ChrCount)

	gpos_t FromPos;
	unsigned FromChrIndex;
	GetRandomPos(FromChrIndex, FromPos);
	m_CurrentEventPos = FromPos;

	gpos_t MoveLength = g_InterChrMoveRateMgr->GetLength();
	if (!RangeExists(FromChrIndex, FromPos, MoveLength))
		EV_FAIL(InterChrMove, MoveLength, RangeExists)

	gpos_t ToPos;
	unsigned ToChrIndex;
	GetRandomPosDifferentChr(FromChrIndex, ToChrIndex, ToPos);

	if (RangeHasCE(ToChrIndex, ToPos, 2))
		EV_REJECT(InterChrMove, MoveLength, InsIntoCE)//(1)

	if (PosInGene(FromChrIndex, FromPos))
		EV_REJECT(InterChrMove, MoveLength, PartialGene)//(2)

	if (PosIsConstrained(FromChrIndex, FromPos))
		EV_REJECT(InterChrMove, MoveLength, PartialCE)//(3)

	gpos_t EndPos = GetRangeEndPos(FromChrIndex, FromPos, MoveLength);

	if (PosInGene(FromChrIndex, EndPos))
		EV_REJECT(InterChrMove, MoveLength, PartialGene)//(4)

	if (PosIsConstrained(FromChrIndex, EndPos))
		EV_REJECT(InterChrMove, MoveLength, PartialCE)//(5)

	bool InsertIntoGene = PosInGene(ToChrIndex, ToPos);
	bool SourceHasCE = RangeHasCE(FromChrIndex, FromPos, MoveLength);
	bool SourceHasGene = RangeHasGene(FromChrIndex, FromPos, MoveLength);

	if (SourceHasGene && InsertIntoGene)
		EV_REJECT(InterChrMove, MoveLength, GeneIntoGene)//(6)

	if (SourceHasCE && InsertIntoGene)
		EV_REJECT(InterChrMove, MoveLength, CEIntoGene)//(7)

	bool DoInvert = false;
	if (MoveLength >= P_MinMoveInvertLength)
		{
		++g_InvertableMoveCount;
		DoInvert = RandTrue(P_MoveInvertProb);
		if (DoInvert)
			++g_InvertedMoveCount;
		}
	++g_TotalMoveCount;

	bool Strand;
	unsigned GeneIndex = GetGeneIndex(ToChrIndex, ToPos, Strand);

#if TRACE
	{
	Log("ICMOVE From %u,%"PF" To %u,%"PF" Length %"PF" Invert %c\n",
	  FromChrIndex, FromPos, ToChrIndex, ToPos, MoveLength, tof(DoInvert));
	ContigSet Contigs;
	GetOverlappingContigs(FromChrIndex, FromPos, MoveLength, Contigs);
	Log("Contigs to move (overlapping):\n");
	LogContigs(Contigs);
	}
#endif

	Move(FromChrIndex, FromPos, ToChrIndex, ToPos, MoveLength, DoInvert,
	  GeneIndex, Strand);

#if TRACE
	{
	ContigSet Contigs;
	GetOverlappingContigs(ToChrIndex, ToPos-1, MoveLength+2, Contigs);
	Log("Inserted move (overlapping):\n");
	LogContigs(Contigs);
	}
#endif

	EV_ACCEPT(InterChrMove, ToPos, MoveLength)
	}
