#include "utils.h"
#include "evomodel.h"
#include "evolver.h"
#include "myrand.h"

bool IsOffByOneStartCodon(const string &Codon);
uint8 GetRandProbIndex(double MAC);

EVENT_RESULT Evolver::Try_MoveStartCodonIntoCDS()
	{
	vector<GeneInfo> Genes;
	vector<double> MACs;
	InitTryGenes(Genes, MACs);

	GeneInfo Gene;
	gpos_t StartPos = GPOS_T_MAX;
	for (;;)
		{
		bool Any = NextTryGene(Genes, MACs, Gene);
		if (!Any)
			EV_FAILN(MoveStartCodonIntoCDS, NoStarts)

		StartPos = GetMoveStartCodonIntoCDSCand(Gene);
		if (StartPos != GPOS_T_MAX)
			break;
		}
	if (StartPos == GPOS_T_MAX)
		EV_FAILN(MoveStartCodonIntoCDS, NoStarts)
	m_CurrentEventPos = StartPos;

	UpdateStartCodonMap(Gene.StartCodonFirstBasePos, StartPos);

	bool Strand = Gene.Plus;

	SetTriplet(StartPos, "ATG", Strand);

	unsigned BaseCount = 0;
	for (gpos_t Pos = Gene.StartCodonFirstBasePos; Pos != StartPos;
	  Pos = GetAdjExonPos(Pos, Gene.Plus))
		{
		BaseInfo &Base = GetBase(Pos);
		Base.ProbIx = GetRandProbIndex(Gene.MeanAccProb);
		Base.SetUTR();
		++BaseCount;
		}
	asserta(BaseCount > 0);
	asserta(BaseCount%3 == 0);

	SetStartCodon(StartPos, Strand);

	GeneInfo Gene2;
	GetGene(StartPos, Gene2);
	unsigned ErrCount = ValidateGene(Gene2);
	if (ErrCount > 0)
		Die("Try_MoveStartCodonIntoCDS: ValidateGene failed");

	EV_ACCEPTN(MoveStartCodonIntoCDS, StartPos)
	}

gpos_t Evolver::GetMoveStartCodonIntoCDSCand(const GeneInfo &Gene) const
	{
	gpos_t Pos = Gene.StartCodonFirstBasePos;
	string Codon;
	for (gpos_t CodonsMovedCount = 0; CodonsMovedCount < P_MaxMoveStartStopCodons;
	  ++CodonsMovedCount)
		{
		Pos = GetNextExonTriplet(Pos, Codon, Gene.Plus);
		if (Pos == GPOS_T_MAX)
			return GPOS_T_MAX;
		if (IsOffByOneStartCodon(Codon))
			return Pos;
		}
	return GPOS_T_MAX;
	}
