#define TRACE	0

#include "evolver.h"
#include "ratemgr.h"
#include "myrand.h"
#include "evomodel.h"

int8 IncCodexStrand(int8 Codex, bool Strand);
bool IsStopCodon(const string &Codon);
bool IsStartCodon(const string &Codon);
uint8 GetRandProbIndex(double MAC);

bool IsOffByOne(const string &Codon1, const string &Codon2)
	{
	asserta(Codon1.size() == 3);
	asserta(Codon2.size() == 3);

	unsigned DiffCount = 0;
	if (Codon1[0] != Codon2[0])
		++DiffCount;
	if (Codon1[1] != Codon2[1])
		++DiffCount;
	if (Codon1[2] != Codon2[2])
		++DiffCount;
	return DiffCount <= 1;
	}

bool IsOffByOneStartCodon(const string &Codon)
	{
	return IsOffByOne(Codon, "ATG");
	}

EVENT_RESULT Evolver::Try_MoveStartCodonIntoUTR()
	{
	vector<GeneInfo> Genes;
	vector<double> MACs;
	InitTryGenes(Genes, MACs);

	GeneInfo Gene;
	gpos_t StartPos = GPOS_T_MAX;
	for (;;)
		{
		bool Any = NextTryGene(Genes, MACs, Gene);
		if (!Any)
			EV_FAILN(MoveStartCodonIntoUTR, NoStarts)

		StartPos = GetMoveStartCodonIntoUTRCand(Gene);
		if (StartPos != GPOS_T_MAX)
			break;
		}
	if (StartPos == GPOS_T_MAX)
		EV_FAILN(MoveStartCodonIntoUTR, NoStarts)
	m_CurrentEventPos = StartPos;

	bool Strand = Gene.Plus;
	string Codon;
	GetNextExonTriplet(StartPos, Codon, Strand);
	if (!IsOffByOneStartCodon(Codon))
		{
		Log("\n");
		Log("Gene: ");
		Gene.LogMe();
		Log("StartPos %"PF"%c\n", StartPos, pom(Strand));
		Die("Got %s, expected off-by-one ATG", Codon.c_str());
		}

	UpdateStartCodonMap(Gene.StartCodonFirstBasePos, StartPos);
	SetTriplet(StartPos, "ATG", Strand);

	int8 Codex = (Strand ? 1 : -1);

	gpos_t Pos = StartPos;
	unsigned BaseCount = 0;
	for (;;)
		{
		BaseInfo &Base = GetBase(Pos);
#if	TRACE
		Log("Convert base, before: ");
		Base.LogMe();
#endif
		asserta(Base.IsUTR() || Base.IsStartCodon());
		Base.SetCDS();
		Base.Codex = Codex;
		Base.ProbIx = GetRandProbIndex(Gene.MeanAccProb);
#if	TRACE
		Log(" .. after: ");
		Base.LogMe(true);
#endif
		++BaseCount;
		if (Pos == Gene.StartCodonLastBasePos)
			break;

		Codex = IncCodexStrand(Codex, Strand);

		Pos = GetAdjExonPos(Pos, Strand);
		}
	asserta(BaseCount > 0);
	asserta(BaseCount%3 == 0);

	SetCDS(Gene.StartCodonFirstBasePos, 3, Strand);
	SetStartCodon(StartPos, Strand);

	GeneInfo NewGene;
	GetGene(StartPos, NewGene);
	unsigned ErrCount = ValidateGene(NewGene);
	if (ErrCount > 0)
		Die("Try_MoveStartCodonIntoUTR: Validate gene failed");

	EV_ACCEPTN(MoveStartCodonIntoUTR, StartPos)
	}

void Evolver::SetCDS(gpos_t Pos, gpos_t Length, bool Strand)
	{
	gpos_t p = Pos;
	for (gpos_t i = 0; i < Length; ++i)
		{
		BaseInfo &Base = GetBase(Pos);
		Base.SetCDS();
		p = GetAdjExonPos(p, Strand);
		}
	}

// Returns first pos in translation order, so is always pos
// of A in AGT whether or not gene is on minus strand.
gpos_t Evolver::GetMoveStartCodonIntoUTRCand(const GeneInfo &Gene) const
	{
#if	TRACE
	Log("\n");
	Log("GetMoveStartCand\n");
	Log("Gene: ");
	Gene.LogMe();
#endif

	bool Strand = Gene.Plus;

	gpos_t FirstUTRBaseCount = 0;
	gpos_t LastUTRBaseCount = 0;
	GetUTRBaseCounts(Gene, FirstUTRBaseCount, LastUTRBaseCount);
	gpos_t UTRBaseCount = Strand ? FirstUTRBaseCount : LastUTRBaseCount;

	gpos_t Pos = Gene.StartCodonLastBasePos;
	string Codon;
	for (unsigned CodonsMovedCount = 0; ; ++CodonsMovedCount)
		{
		gpos_t PrevCodonEndPos = GetPrevExonTriplet(Pos, Codon, Strand);
		if (CodonsMovedCount == 0)
			asserta(IsStartCodon(Codon));
		if (PrevCodonEndPos == GPOS_T_MAX)
			return GPOS_T_MAX;

#if	TRACE
		Log("PrevCodonEndPos=%"PF" Pos=%"PF" Codon=%s\n",
		  PrevCodonEndPos, Pos, Codon.c_str());
#endif
		if (IsOffByOneStartCodon(Codon) && CodonsMovedCount > 1)
			{
			if (UTRBaseCount < CodonsMovedCount*3 + P_MinUTRBases)
				return GPOS_T_MAX;

		// Adjust pos from end of codon to first base in codon.
			Pos = GetPrevExonPos(Pos, Strand);
			asserta(Pos != GPOS_T_MAX);
			Pos = GetPrevExonPos(Pos, Strand);
			asserta(Pos != GPOS_T_MAX);
#if	TRACE
			Log("Cand %s pos %"PF"\n", Codon.c_str(), Pos);
#endif
			return Pos;
			}

	// Cannot be stop in frame
		if (IsStopCodon(Codon))
			return GPOS_T_MAX;

		Pos = PrevCodonEndPos;
		}
	}
