#define TRACE	0

#include "utils.h"
#include "evomodel.h"
#include "evolver.h"
#include "myrand.h"

bool IsOffByOne(const string &Codon1, const string &Codon2);
uint8 GetRandProbIndex(double MAC);

// TAG, TAA, TGA
bool IsOffByOneStopCodon(const string &Codon)
	{
	return IsOffByOne(Codon, "TAG") || IsOffByOne(Codon, "TAA") ||
	  IsOffByOne(Codon, "TGA");
	}

EVENT_RESULT Evolver::Try_MoveStopCodonIntoCDS()
	{
	vector<GeneInfo> Genes;
	vector<double> MACs;
	InitTryGenes(Genes, MACs);

	GeneInfo Gene;
	gpos_t StopPos = GPOS_T_MAX;
	for (;;)
		{
		bool Any = NextTryGene(Genes, MACs, Gene);
		if (!Any)
			EV_FAILN(MoveStopCodonIntoCDS, NoStops)

		StopPos = GetMoveStopCodonIntoCDSCand(Gene);
		if (StopPos != GPOS_T_MAX)
			break;
		}
	if (StopPos == GPOS_T_MAX)
		EV_FAILN(MoveStopCodonIntoCDS, NoStops)

	m_CurrentEventPos = StopPos;
	bool Strand = Gene.Plus;

	string Codon;
	GetNextExonTriplet(StopPos, Codon, Strand);
#if TRACE
	Log("GNET StopPos=%"PF" Strand=%c Codon=%s ", StopPos, tof(Strand), Codon.c_str());
	CodonInfo CI;
	GetCodon(StopPos, CI);
	CI.LogMe();
	Log("\n");
	LogNeighborRegions(StopPos, 100);
	LogNeighborsAsTable(StopPos);
#endif
	asserta(IsOffByOneStopCodon(Codon));

	const BaseInfo &FirstStopBase = GetBase(StopPos);
#if	TRACE
	Log("FirstStopBase= ");
	FirstStopBase.LogMe();
	Log("\n");
#endif
	asserta(FirstStopBase.Codex == (Strand ? 1 : -1));

// Mutate existing codon to STOP by zero or one substitutions
	if (IsOffByOne(Codon, "TAG"))
		SetTriplet(StopPos, "TAG", Strand);
	else if (IsOffByOne(Codon, "TAA"))
		SetTriplet(StopPos, "TAA", Strand);
	else if (IsOffByOne(Codon, "TGA"))
		SetTriplet(StopPos, "TGA", Strand);
	else
		asserta(false);

// Skip new stop codon (stop codons are CDS)
	gpos_t Pos = GetNextExonTriplet(StopPos, Codon, Strand);

	unsigned BaseCount = 0;
	for (;;)
		{
		asserta(Pos != GPOS_T_MAX);
		BaseInfo &Base = GetBase(Pos);
#if	TRACE
		Log("Convert base ");
		Base.LogMe();
#endif
	// Set base to be non-coding exon
		Base.SetUTR();
		Base.ProbIx = GetRandProbIndex(Gene.MeanAccProb);
#if	TRACE
		Log(" => ");
		Base.LogMe(true);
#endif
		++BaseCount;
		if (Pos == Gene.StopCodonLastBasePos)
			break;
		Pos = GetAdjExonPos(Pos, Strand);
		}

	asserta(BaseCount > 0);
	asserta(BaseCount%3 == 0);

	SetStopCodon(StopPos, Strand);

	GeneInfo Gene2;
	GetGene(StopPos, Gene2);
	unsigned ErrCount = ValidateGene(Gene2);
	if (ErrCount > 0)
		Die("Try_MoveStopCodonIntoCDS: ValidateGene failed");

	EV_ACCEPTN(MoveStopCodonIntoCDS, StopPos)
	}

// Return first pos in off-by-one STOP codon or GPOS_T_MAX if none.
gpos_t Evolver::GetMoveStopCodonIntoCDSCand(const GeneInfo &Gene) const
	{
#if	TRACE
	Log("\n");
	Log("GetMoveStopCodonIntoCDSCand\n");
	LogGene(Gene);
#endif
	string Codon;
	gpos_t Pos = Gene.StopCodonLastBasePos;
	Pos = GetPrevExonTriplet(Pos, Codon, Gene.Plus);
	asserta(Pos != GPOS_T_MAX);
	for (unsigned CodonsMovedCount = 0; ; ++CodonsMovedCount)
		{
		Pos = GetPrevExonTriplet(Pos, Codon, Gene.Plus);
#if	TRACE
		Log("Pos %"PF" Codon %s\n", Pos, Codon.c_str());
		Log(" CI=");
		CodonInfo CI;
		GetCodon(Pos, CI);
		CI.LogMe();
		Log("\n");
#endif
		if (Pos == GPOS_T_MAX)
			return GPOS_T_MAX;
		if (IsOffByOneStopCodon(Codon))
			{
			gpos_t StopPos = GetAdjExonPos(Pos, Gene.Plus);
#if	TRACE
			Log("FOUND %"PF"\n", StopPos);
#endif
			return StopPos;
			}
		}
	}
