#include "evolver.h"
#include "ratemgr.h"
#include "myrand.h"
#include "evomodel.h"

extern unsigned g_TotalTandemCount;
extern unsigned g_InvertableTandemCount;
extern unsigned g_InvertedTandemCount;

EVENT_RESULT Evolver::Try_Tandem()
	{
	gpos_t FromPos = GetRandomPos();
	if (FromPos == GPOS_T_MAX)
		EV_FAIL(Tandem, 0, GetRandomPos)

	EV_GETLENGTH(Tandem, CopyLength)
	if (CopyLength == GPOS_T_MAX)
		EV_FAIL(Tandem, CopyLength, GetTandemLength)

	if (!RangeExists(FromPos, CopyLength))
		EV_FAIL(Tandem, CopyLength, RangeExists)

	gpos_t ToPos = GetRangeEndPos(FromPos, CopyLength);

	bool DoInvert = false;
	if (CopyLength > P_MinTandemInvertLength)
		{
		DoInvert = RandTrue(P_TandemInvertProb);
		gpos_t NextPos = GetNextPos(ToPos);
		if (NextPos == GPOS_T_MAX)
			EV_FAIL(Tandem, CopyLength, RangeExists)

		if (DoInvert && PosInCDS(ToPos) || PosInCDS(NextPos))
			EV_REJECT(Tandem, CopyLength, InvertCDS)
		}

	string Seq;
	GetSubSeq(FromPos, CopyLength, Seq);

	//bool Accept = AcceptInsert(ToPos, Seq);
	//if (!Accept)
	//	EV_REJECT(Tandem, CopyLength, Insert)
	ACCEPT_INSERT(Tandem, ToPos, Seq)

	if (CopyLength > P_MinTandemInvertLength)
		++g_InvertableTandemCount;

	vector<gpos_t> Genes;
	vector<gpos_t> CopiedGenes;

	m_CurrentEventPos = ToPos;
	Copy(FromPos, ToPos, CopyLength, Genes, CopiedGenes);
	DupeGenes(Genes, CopiedGenes, false);

	if (DoInvert)
		{
		gpos_t FirstInsertedPos = GetNextPos(ToPos);
		Invert(FirstInsertedPos, CopyLength);
		++g_InvertedTandemCount;
		}
	++g_TotalTandemCount;

	if (CopyLength >= P_MinTandemAnnot &&
	  CopyLength <= P_MaxTandemAnnot)
		{
		gpos_t InsertStartPos = GetNextPos(ToPos);
		SetTandemMotifLength(FromPos, CopyLength, CopyLength);
		SetTandemMotifLength(InsertStartPos, CopyLength, CopyLength);
		RecomputeTandemRates();
		}
	EV_ACCEPT(Tandem, ToPos, CopyLength)
	}
