#include <time.h>
#include <stdio.h>
#include <stdarg.h>
#include "utils.h"
#include <sys/stat.h>
#include <errno.h>
#include <string.h>
#include <fcntl.h>
#include <map>
#include "wrapstdio.h"

#define MYBUFFER	1

#ifdef NOMYBUFFER
#define MYBUFFER  0
#endif

#if	MYBUFFER
const size_t MYBUFSIZ = 32000;
#endif

#ifdef _MSC_VER
#include <process.h>
#endif

#ifndef MAX_MSG_SIZE
#define MAX_MSG_SIZE 64000
#endif

#ifdef _MSC_VER
#pragma warning(disable : 4800)	// int->bool conversion
#include <windows.h>
#include <io.h>
#else
#include <unistd.h>
#endif

bool g_Trace = false;
bool g_LogOnly = false;

#if	MYBUFFER
static char *g_IOBuffers[256];

static void AllocBuffer(FILE *f, size_t Bytes)
	{
	int fd = fileno(f);
	if (fd < 0 || fd >= 256)
		return;
	if (g_IOBuffers[fd] != 0)
		myfree(g_IOBuffers[fd]);
	g_IOBuffers[fd] = myalloc(char, Bytes);
	setvbuf(f, g_IOBuffers[fd], _IOFBF, Bytes);
	}

static void FreeBuffer(FILE *f)
	{
	int fd = fileno(f);
	if (fd < 0 || fd >= 256)
		return;
	if (g_IOBuffers[fd] == 0)
		return;
	myfree(g_IOBuffers[fd]);
	g_IOBuffers[fd] = 0;
	}
#endif

void SetTrace(bool On)
	{
	g_Trace = On;
	}

char *RotatingStaticBuffer()
	{
	static int Index = 0;
	static char Buffer[32][128];
	char *p = Buffer[Index];
	++Index;
	Index = Index%32;
	return p;
	}

static time_t g_StartTime = time(0);

long GetElapsedSecs()
	{
	return (long) (time(0) - g_StartTime);
	}

void Dump(const char szFormat[], ...)
	{
	char szStr[64000];
	va_list ArgList;
	va_start(ArgList, szFormat);
	vsnprintf(szStr, sizeof(szStr), szFormat, ArgList);
	if (!g_LogOnly)
		fprintf(stdout, "%s", szStr);
	Log("%s", szStr);
	}

void *allocmem(size_t bytes)
	{
	void *p = malloc(bytes);
	if (0 == p)
		{
		fprintf(stderr, "Out of memory allocmem(%u)", (unsigned) bytes);
		Log("Out of memory, allocmem(%u)\n", (unsigned) bytes);
		}
	return p;
	}

void freemem(void *p)
	{
	free(p);
	}

char *strsave(const char *s)
	{
	if (0 == s)
		return 0;
	char *ptrCopy = strdup(s);
	if (0 == ptrCopy)
		fprintf(stderr, "Out of memory");
	return ptrCopy;
	}

char *mystrsave(const char *s)
	{
	if (0 == s)
		return 0;
	size_t len = strlen(s) + 1;
	char *ptrCopy = myalloc(char, len);
	memcpy(ptrCopy, s, len);
	return ptrCopy;
	}

void safestrcpy(char *s, const char *t, size_t n)
	{
	if (n == 0)
		return;

	size_t m = strlen(t);
	if (m >= n - 1)
		m = n - 1;
	memcpy(s, t, m);
	s[m] = 0;
	}

// NOT IMPLEMENTED CORRECTLY.
// Not sure if we will ever need values
// > 2GB, but at least it's localized here.
uint64 atouint64(const char *s)
	{
	return (uint64) atoi(s);
	}

coord_t StrToCoord(const char *s)
	{
	return (coord_t) atoi(s);
	}

gpos_t StrToCoord64(const char *s)
	{
	Die("StrToCoord64: Not implemented");
	return (gpos_t) atoi(s);
	}

const char *CoordToStr(coord_t x)
	{
	char *Str = RotatingStaticBuffer();
	sprintf(Str, "%u", x);
	return Str;
	}

const char *Coord64ToStr(gpos_t x)
	{
	char *Str = RotatingStaticBuffer();
	sprintf(Str, "%"PF, x);
	return Str;
	}

const char *UInt64ToStr(gpos_t x)
	{
	char *Str = RotatingStaticBuffer();
	sprintf(Str, "%"PF, x);
	return Str;
	}

const char *FileOffsetToStr(off64_t x)
	{
	char *Str = RotatingStaticBuffer();
	sprintf(Str, "%"OFF64_T_PRINTF, x);
	return Str;
	}

void *alloc(size_t bytes)
	{
	void *p = malloc(bytes);
	if (0 == p)
		Die("Out of memory, alloc(%u)\n", (unsigned) bytes);
	return p;
	}

//char *strrev(char *s)
//	{
//	size_t L = strlen(s);
//	char *save = s;
//	char *t = s + L - 1;
//	while (s < t)
//		{
//		char c = *s;
//		*s++ = *t--;
//		*t = c;
//		}
//	return save;
//	}

bool streq(const char *s1, const char *s2)
	{
	return strcmp(s1, s2) == 0;
	}

// 4^n
uint64 pow4(unsigned n)
	{
	assert(n < 32);
	const uint64 one = 1;
	return (one << (2*n));
	}

// 20^n
uint64 pow20(unsigned n)
	{
	uint64 p = 1;
	for (unsigned i = 0; i < n; ++i)
		p *= 20;
	return p;
	}

const char *uint64tostr(uint64 i)
	{
	char *s = RotatingStaticBuffer();
	return uint64tostr(i, s);
	}

const char *uint64tostr(uint64 i, char *s)
	{
	sprintf(s, "%"UINT64_PRINTF, i);
	return s;
	}

const char *uint32tostr(uint32 i)
	{
	return uint64tostr(i);
	}

const char *uint32tostr(uint32 i, char *s)
	{
	return uint64tostr(i, s);
	}

bool FileExists(const char *FileName)
	{
	struct stat SD;
	int i = stat(FileName, &SD);
	return i == 0;
	}

void myassertfail(const char *Exp, const char *File, unsigned Line)
	{
	Die("%s(%u) Assertion failed: %s", File, Line, Exp);
	}

const char *atoh(const char *str)
	{
	char *s = RotatingStaticBuffer();

	int L = (int) strlen(str);
	if (L >= (int) sizeof(s) - 2)
		return str;

	int j = sizeof(s) - 2;
	for (int i = L-1; i >= 0; --i)
		{
		int k = L - i - 1;
		if (k%3 == 0 && k != 0)
			s[j--] = ',';
		s[j--] = str[i];
		}
	return s + j + 1;
	}

const char *utoh(unsigned i)
	{
	return atoh(UIntToStr(i));
	}

const char *UIntToStr(unsigned i)
	{
	if (i == UINT_MAX)
		return "*";

	char *s = RotatingStaticBuffer();
	sprintf(s, "%u", i);
	return s;
	}

bool IsTerminal(int fd)
	{
	return (bool) isatty(fd);
	}

const char *SPScoreToStr(sp_score_t Score)
	{
	char *Str = RotatingStaticBuffer();
	sprintf(Str, "%d", (int) Score);
	return Str;
	}

const char *AIScoreToStr(ai_score_t Score)
	{
	char *Str = RotatingStaticBuffer();
	sprintf(Str, "%g", Score);
	return Str;
	}

const char *ScoreToStr(score_t Score)
	{
	char *Str = RotatingStaticBuffer();
#if INTEGER_DP
	sprintf(Str, "%d", (int) Score);
#else
	sprintf(Str, "%g", (double) Score);
#endif
	return Str;
	}

unsigned DoubleToUInt(double Value)
	{
	asserta(sizeof(unsigned) >= sizeof(double));
	union
		{
		unsigned uValue;
		double fValue;
		} u;
	u.fValue = Value;
	return u.uValue;
	}

unsigned Float32ToUInt(float32 Value)
	{
	asserta(sizeof(unsigned) >= sizeof(float32));
	union
		{
		unsigned uValue;
		float32 fValue;
		} u;
	u.fValue = Value;
	return u.uValue;
	}

double UIntToDouble(unsigned Value)
	{
	asserta(sizeof(unsigned) <= sizeof(double));
	union
		{
		unsigned uValue;
		double fValue;
		} u;
	u.uValue = Value;
	return u.fValue;
	}

float32 UIntToFloat32(unsigned Value)
	{
	asserta(sizeof(unsigned) <= sizeof(float32));
	union
		{
		unsigned uValue;
		float32 fValue;
		} u;
	u.uValue = Value;
	return u.fValue;
	}

#ifdef _MSC_VER
#include <io.h>
int fseeko(FILE *stream, off64_t offset, int whence)
	{
//	off64_t FilePos = _lseeki64(_fileno(stream), offset, whence);
	off64_t FilePos = _fseeki64(stream, offset, whence);
	return (FilePos == -1L) ? -1 : 0;
	}
//#define ftello(fm) _telli64(_fileno(fm))
#define ftello(fm) _ftelli64(fm)
#endif

//static std::map<FILE *, std::string> FilePtrToName;
//
//static void AddFileToMap(FILE *f, const std::string &Name)
//	{
//	FilePtrToName[f] = Name;
//	}
//
//static void DeleteFileFromMap(FILE *f)
//	{
//	FilePtrToName.erase(f);
//	}
//
//void GetStdioFileName(FILE *f, std::string &FileName)
//	{
//	std::map<FILE *, std::string>::const_iterator p =
//	  FilePtrToName.find(f);
//	if (p == FilePtrToName.end())
//		FileName = "";
//	FileName = p->second;
//	}

void LogStdioFileState(FILE *f)
	{
//	std::string Name;
//	GetStdioFileName(f, Name);
	long tellpos = ftell(f);
	long fseek_pos = fseek(f, 0, SEEK_CUR);
	int fd = fileno(f);
	Log("FILE *     %p\n", f);
//	Log("Name       %s\n", Name.c_str());
	Log("fileno     %d\n", fd);
	Log("feof       %d\n", feof(f));
	Log("ferror     %d\n", ferror(f));
	Log("ftell      %ld\n", tellpos);
	Log("fseek      %ld\n", fseek_pos);
#if	!defined(_GNU_SOURCE) && !defined(__APPLE_CC__)
	fpos_t fpos;
	int fgetpos_retval = fgetpos(f, &fpos);
	Log("fpos       %ld (retval %d)\n", (long) fpos, fgetpos_retval);
	Log("eof        %d\n", _eof(fd));
#endif
#ifdef _MSC_VER
	__int64 pos64 = _ftelli64(f);
	Log("_ftelli64  %lld\n", pos64);
#endif
	}

FILE *OpenStdioFile(const char *FileName, bool ForWriting)
	{
	if (FileName == 0)
		Die("OpenStdioFile, FileName=NULL");
	const char *Mode = (ForWriting ? "wb+" : "rb");
	FILE *f = fopen(FileName, Mode);
	if (0 == f)
		{
		char *ErrMsg = strerror(errno);
		Die("Cannot open %s, errno=%d %s", FileName, errno, ErrMsg);
		}
//	AddFileToMap(f, FileName);
#if MYBUFFER
	AllocBuffer(f, MYBUFSIZ);
#endif
	return f;
	}

FILE *CreateStdioFile(const char *FileName)
	{
	if (FileName == 0)
		Die("CreateStdioFile, FileName=NULL");
	FILE *f = fopen(FileName, "wb+");
	if (0 == f)
		Die("Cannot create %s, errno=%d %s", FileName, errno, strerror(errno));
//	AddFileToMap(f, FileName);
#if MYBUFFER
	AllocBuffer(f, MYBUFSIZ);
#endif
	return f;
	}

// Synonym for SeekStdioFile for symmetry with GetStdioFilePos.
void SetStdioFilePos(FILE *f, off64_t Pos)
	{
	SeekStdioFile(f, Pos);
	}

FILE *CreateStdioTmpFile(const char *prefix)
{
	if (prefix == NULL)
		Die("CreateStdioTmpFile called with NULL pointer");

	std::string FileNameStr = prefix;
	FileNameStr += "XXXXXX";
	char *FileName = strdup(FileNameStr.c_str());
	if (FileName == NULL)
		Die("Out of memory");

#ifdef _MSC_VER
	_mktemp_s(FileName, strlen(FileName));
	int fd = open(FileName, _O_CREAT | _O_RDWR); 
#else
	int fd = mkstemp(FileName);
#endif
	if (fd == -1)
		Die("Cannot make temp file %s, errno=%d %s", FileName, errno, strerror(errno));

	FILE *fp = fdopen(fd, "w+b");
	if (fp == NULL)
		Die("Cannot fdopen, errno=%d %s", errno, strerror(errno));

	if (unlink(FileName) == -1)
		Die("Cannot unlink temp file %s, errno=%d %s", FileName, errno, strerror(errno));

	free(FileName);

	return fp;
}

void SeekStdioFile(FILE *f, off64_t Pos)
	{
	if (0 == f)
		Die("SeekStdioFile failed, f=NULL");
	int Ok = fseeko(f, Pos, SEEK_SET);
	off64_t NewPos = ftello(f);
	if (Ok != 0 || Pos != NewPos)
		{
		LogStdioFileState(f);
		Die("SeekStdioFile(%"PF") failed, Ok=%d NewPos=%"PF,
		  Pos, Ok, NewPos);
		}
	}

void ReadStdioFile(FILE *f, off64_t Pos, void *Buffer, size_t Bytes)
	{
	if (0 == f)
		Die("ReadStdioFile failed, f=NULL");
	SeekStdioFile(f, Pos);
	size_t BytesRead = fread(Buffer, 1, Bytes, f);
	if (BytesRead != Bytes)
		{
		LogStdioFileState(f);
		Die("ReadStdioFile failed, attempted %"SIZE_T_PRINTF" bytes, read %"SIZE_T_PRINTF" bytes, errno=%d",
		  Bytes, BytesRead, errno);
		}
	}

void ReadStdioFile(FILE *f, void *Buffer, size_t Bytes)
	{
	if (0 == f)
		Die("ReadStdioFile failed, f=NULL");
	size_t BytesRead = fread(Buffer, 1, Bytes, f);
	if (BytesRead != Bytes)
		{
		LogStdioFileState(f);
		Die("ReadStdioFile failed, attempted %"SIZE_T_PRINTF" bytes, read %"SIZE_T_PRINTF" bytes, errno=%d",
		  Bytes, BytesRead, errno);
		}
	}

void WriteStdioFile(FILE *f, off64_t Pos, const void *Buffer, size_t Bytes)
	{
	if (0 == f)
		Die("WriteStdioFile failed, f=NULL");
	SeekStdioFile(f, Pos);
	size_t BytesWritten = fwrite(Buffer, 1, Bytes, f);
	if (BytesWritten != Bytes)
		{
		LogStdioFileState(f);
		Die("WriteStdioFile failed, attempted %"SIZE_T_PRINTF" bytes, wrote %"SIZE_T_PRINTF" bytes, errno=%d",
		  Bytes, BytesWritten, errno);
		}
	}

void WriteStdioFile(FILE *f, const void *Buffer, size_t Bytes)
	{
	if (0 == f)
		Die("WriteStdioFile failed, f=NULL");
	size_t BytesWritten = fwrite(Buffer, 1, Bytes, f);
	if (BytesWritten != Bytes)
		{
		LogStdioFileState(f);
		Die("WriteStdioFile failed, attempted %"SIZE_T_PRINTF" bytes, wrote %"SIZE_T_PRINTF" bytes, errno=%d",
		  Bytes, BytesWritten, errno);
		}
	}

void WriteStdioFileStr(FILE *f, off64_t Pos, const char *Buffer)
	{
	WriteStdioFile(f, Pos, Buffer, strlen(Buffer));
	}

void WriteStdioFileStr(FILE *f, const char *Buffer)
	{
	WriteStdioFile(f, Buffer, strlen(Buffer));
	}

// Return true on EOF (dumb choice, but stuck with it now).
bool ReadLineStdioFile(FILE *f, char *Line, size_t Bytes)
	{
	if (feof(f))
		return true;
	if ((int) Bytes < 0)
		Die("ReadLineStdioFile: Bytes < 0");
	char *RetVal = fgets(Line, (int) Bytes, f);
	if (NULL == RetVal)
		{
		if (feof(f))
			return true;
		if (ferror(f))
			Die("ReadLineStdioFile: errno=%d", errno);
		Die("ReadLineStdioFile: fgets=0, feof=0, ferror=0");
		}

	if (RetVal != Line)
		Die("ReadLineStdioFile: fgets != Buffer");
	size_t n = strlen(Line);
	if (n < 1 || Line[n-1] != '\n')
		Die("ReadLineStdioFile: line too long or missing end-of-line");
	if (n > 0 && (Line[n-1] == '\r' || Line[n-1] == '\n'))
		Line[n-1] = 0;
	if (n > 1 && (Line[n-2] == '\r' || Line[n-2] == '\n'))
		Line[n-2] = 0;
	return false;
	}

// Return true on EOF (dumb choice, but stuck with it now).
bool ReadLineStdioFile(FILE *f, std::string &Line)
	{
	Line.clear();
	for (;;)
		{
		int c = fgetc(f);
		if (c == -1)
			{
			if (feof(f))
				return true;
			Die("ReadLineStdioFile, errno=%d", errno);
			}
		if (c == '\r')
			continue;
		if (c == '\n')
			return false;
		Line.push_back((char) c);
		}
	}

// Line ReadLineStdioFile except die on end-of-file
void ReadLineStdioFile2(FILE *f, char *Line, size_t Bytes)
	{
	if (feof(f))
		Die("ReadLineStdioFile2: end-of-file");
	if ((int) Bytes < 0)
		Die("ReadLineStdioFile: Bytes < 0");
	char *RetVal = fgets(Line, (int) Bytes, f);
	if (NULL == RetVal)
		{
		if (feof(f))
			Die("ReadLineStdioFile2: end-of-file");
		if (ferror(f))
			Die("ReadLineStdioFile: errno=%d", errno);
		Die("ReadLineStdioFile: fgets=0, feof=0, ferror=0");
		}

	if (RetVal != Line)
		Die("ReadLineStdioFile: fgets != Buffer");
	size_t n = strlen(Line);
	if (n < 1 || Line[n-1] != '\n')
		Die("ReadLineStdioFile: line too long or missing end-of-line");
	if (n > 0 && (Line[n-1] == '\r' || Line[n-1] == '\n'))
		Line[n-1] = 0;
	if (n > 1 && (Line[n-2] == '\r' || Line[n-2] == '\n'))
		Line[n-2] = 0;
	}

// Copies all of fFrom regardless of current
// file position, appends to fTo.
void CopyStdioFile(FILE *fFrom, FILE *fTo)
	{
	off64_t SavedFromPos = GetStdioFilePos(fFrom);
	off64_t FileSize = GetStdioFileSize(fFrom);
	unsigned BUFF_SIZE = 1024*1024;
	char *Buffer = myalloc(char, BUFF_SIZE);
	SeekStdioFile(fFrom, 0);
	off64_t BytesRemaining = FileSize;
	while (BytesRemaining > 0)
		{
		off64_t BytesToRead = BytesRemaining;
		if (BytesToRead > BUFF_SIZE)
			BytesToRead = BUFF_SIZE;
		ReadStdioFile(fFrom, Buffer, (size_t) BytesToRead);
		WriteStdioFile(fTo, Buffer, (size_t) BytesToRead);
		BytesRemaining -= BytesToRead;
		}
	SeekStdioFile(fFrom, SavedFromPos);
	}

void RenameStdioFile(const char *FileNameFrom, const char *FileNameTo)
	{
	int Ok = rename(FileNameFrom, FileNameTo);
	if (Ok != 0)
		Die("RenameStdioFile failed, errno=%d", errno);
	}

void FlushStdioFile(FILE *f)
	{
	int Ok = fflush(f);
	if (Ok != 0)
		Die("fflush(%p)=%d,", f, Ok);
	}

void CloseStdioFile(FILE *f)
	{
	int Ok = fclose(f);
	if (Ok != 0)
		Die("fclose(%p)=%d", f, Ok);
#if	MYBUFFER
	FreeBuffer(f);
#endif
//	DeleteFileFromMap(f);
	}

off64_t GetStdioFilePos(FILE *f)
	{
	off64_t FilePos = ftello(f);
	if (FilePos < 0)
		Die("ftello(%p)=%ld errno=%d %s",
		  (void *) f, (long) FilePos, (int) errno, strerror(errno));
	return FilePos;
	}

off64_t GetStdioFileSize(FILE *f)
	{
	off64_t CurrentPos = GetStdioFilePos(f);
	int Ok = fseeko(f, 0, SEEK_END);
	if (Ok < 0)
		Die("fseek in GetFileSize");
	off64_t Length = ftello(f);
	if (Length < 0)
		Die("ftell in GetFileSize");
	SeekStdioFile(f, CurrentPos);
	return Length;
	}

void DeleteStdioFile(const char *FileName)
	{
	int Ok = remove(FileName);
	if (Ok != 0)
		Die("remove(%s) failed, errno=%d %s", FileName, errno, strerror(errno));
	}

void VStrPrintf(std::string &Str, const char *szFormat, va_list ArgList)
	{
	char *szStr = (char *) malloc(MAX_MSG_SIZE);
	if (szStr == 0)
		{
		fprintf(stderr, "\nOut of memory (VStrPrintf)\n");
		exit(1);
		}

	vsnprintf(szStr, MAX_MSG_SIZE, szFormat, ArgList);
	szStr[MAX_MSG_SIZE - 1] = '\0';

	Str.assign(szStr);

	free(szStr);
	}

void StrPrintf(std::string &Str, const char *szFormat, ...)
	{
	va_list ArgList;
	va_start(ArgList, szFormat);
	VStrPrintf(Str, szFormat, ArgList);
	va_end(ArgList);
	}

FILE *g_fLog = NULL;

void SetLogFile(const char *LogFileName)
{
	if (g_fLog != NULL)
	{
		CloseStdioFile(g_fLog);
	}
	g_fLog = CreateStdioFile(LogFileName);
//	setbuf(g_fLog, 0);
}

void UnsetLogFile()
{
	if (g_fLog != NULL)
	{
		CloseStdioFile(g_fLog);
	}
}

/* Does not use wrapstdio functions so as to not introduce endless
 * loops with Die -> Log -> Die -> Log .... */
void Log(const char *szFormat, ...)
	{
	if (NULL == g_fLog)
		return;

	va_list ArgList;
	va_start(ArgList, szFormat);
	vfprintf(g_fLog, szFormat, ArgList);
	va_end(ArgList);
	fflush(g_fLog);
	}

int g_DieArgc = 0;
char **g_DieArgv = NULL;
time_t g_DieStartTime = time(0);

void SetDieArgcArgv(int argc, char **argv)
	{
	g_DieArgc = argc;
	g_DieArgv = argv;
	}

void PrintAndDie(const char *szFormat, ...)
	{
	std::string Msg;

	if (g_fLog != 0)
		setbuf(g_fLog, 0);
	va_list ArgList;
	va_start(ArgList, szFormat);
	VStrPrintf(Msg, szFormat, ArgList);
	va_end(ArgList);

	/* Usually the error message contains a final '\n', but
	 * here we add another one intentionally */
	const char *szStr = Msg.c_str();
	fprintf(stderr, "\n---Fatal error---\n%s\n", szStr);
	Log("\n---Fatal error---\n%s\n", szStr);

	if (g_DieArgv != NULL)
		{
		Log("\n");
		time_t t = time(0);
		Log("%s", asctime(localtime(&t)));
		for (int i = 0; i != g_DieArgc; i++)
			{
			fprintf(stderr, (i == 0) ? "%s" : " %s", g_DieArgv[i]);
			Log((i == 0) ? "%s" : " %s", g_DieArgv[i]);
			}
		fprintf(stderr, "\n");
		Log("\n");
		}

	time_t CurrentTime = time(0);
	if ((CurrentTime > 0) && (g_DieStartTime > 0))
		{
		uint64 ElapsedSeconds = ((uint64)CurrentTime) - (uint64)g_DieStartTime;
		fprintf(stderr, "Elapsed time: %"UINT64_PRINTF" seconds\n",
		  ElapsedSeconds);
		Log("Elapsed time: %"PF" seconds\n", ElapsedSeconds);
		}

#ifdef _MSC_VER
	if (IsDebuggerPresent())
		__debugbreak();
	_CrtSetDbgFlag(0);
#endif

	exit(1);
	}

void Warning(const char *szFormat, ...)
	{
	std::string Msg;

	va_list ArgList;
	va_start(ArgList, szFormat);
	VStrPrintf(Msg, szFormat, ArgList);
	va_end(ArgList);

	const char *szStr = Msg.c_str();

	fprintf(stderr, "\nWARNING: %s\n", szStr);
	Log("\nWARNING: %s\n", szStr);
	fflush(g_fLog);
	}

const std::string &MakeTmpFileName(const std::string &Type, std::string &Name)
	{
	static unsigned Counter = 0;

	++Counter;
	char cstrName[64];
	sprintf(cstrName, ".%u.%u.tmp", getpid(), Counter);
	Name = Type + cstrName;
	return Name;
	}

uint32 coord64_t_to_uint32(gpos_t x)
	{
	if (x > UINT_MAX)
		Die("coord64_t_to_uint32(%"PF"=0x%"x64"): overflow",
		  x, x);
	uint32 ux = (uint32) x;
	return ux;
	}

void Logn(char c, unsigned n)
	{
	for (unsigned i = 0; i < n; ++i)
		Log("%c", c);
	}

unsigned off64_t_to_unsigned(off64_t x)
	{
	if (x > UINT_MAX)
		Die("off64_t_to_uint32(%"UINT64_PRINTF"=0x%"x64"): overflow", x, x);
	return (unsigned) x;
	}

unsigned uint64_to_unsigned(uint64 x)
	{
	if (x > UINT_MAX)
		Die("uint64_t_to_uint32(%"UINT64_PRINTF"=0x%"x64"): overflow", x, x);
	return (unsigned) x;
	}

coord_t uint64_to_coord_t(uint64 x)
	{
	if (x > COORD_T_MAX)
		Die("coord64_t_to_coord_t(%"UINT64_PRINTF"=0x%"x64"): overflow", x, x);
	return (unsigned) x;
	}

unsigned gpos_t_to_unsigned(gpos_t x)
	{
	if (x > UINT_MAX)
		Die("gpos_t_to_coord_t(%"PF"): overflow", x);
	return (unsigned) x;
	}

coord_t gpos_t_to_coord_t(gpos_t x)
	{
	if (x > COORD_T_MAX)
		Die("gpos_t_to_coord_t(%"PF"): overflow", x);
	return (coord_t) x;
	}

double Pct(double x, double T)
	{
	if (T == 0)
		return 0;
	return (x*100.0)/T;
	}
