#include "utils.h"
#include "getopts.h"
#include "fastastr.h"
#include "wordhist.h"
#include "progress.h"

/***
Sketch of the method.

Two genomes, a threshold and a word size k (k should normally be odd).

Create a kmer histogram for each genome. That is, for each genome it
calculates how many times each unqiue word (kmer) appears. Note that
a word and its reverse complement are counted together.

Now for each kmer you have two numbers (N1 = the number of times it
appears in the first genome, and N2 = the number of times it appears in the
second genome). Those two numbers are then multiplied; the result R=N1*N2
is the number of seeds that a blast-like program would initiate because of
that kmer. If f(R) > threshold for some function f(), then all instances of
that kmer are masked by lowermasking its center letter. So if for example,
AATGACA should be masked, whenever AATGACA (or TGTCATT) appears in the genomes,
the letter in the middle is masked: AATgACA and its reverse complement TGTcATT;
or, for hardmasking, AATNACA and TGTNATT. To prevent seeding, it suffices to
mask the middle letter, this allows the other letters to participate in
potentially useful seeds. This is why k should be odd, there is no middle
letter if k is even.

Usually, because wublast doesn't support masked databases, and the process
is symmetrical, it suffices to mask the query genome.
***/

int main(int argc, char **argv)
	{
	ProcessArgVect(argc, argv);

	void Credits();
	Credits();

	if (FlagOpt("advanced_options"))
		{
		void AdvancedOptions();
		AdvancedOptions();
		return 0;
		}
	else if (ValueOpt("mask"))
		{
		void MaskSimple();
		MaskSimple();
		}
	else if (ValueOpt("makehist"))
		{
		void MakeHist();
		MakeHist();
		}
	else if (FlagOpt("maketab"))
		{
		void MakeTab();
		MakeTab();
		}
	else if (ValueOpt("masktab"))
		{
		void MaskTab();
		MaskTab();
		}
	else if (FlagOpt("cmpmask"))
		{
		void CmpMask();
		CmpMask();
		}
	else if (ValueOpt("xorfs"))
		{
		void ExtractORFs();
		ExtractORFs();
		}
	else if (ValueOpt("maskstats"))
		{
		void MaskStats();
		MaskStats();
		}
	else if (ValueOpt("dumptab"))
		{
		void DumpTab();
		DumpTab();
		}
	else if (ValueOpt("dumphist"))
		{
		void DumpHist();
		DumpHist();
		}
	else
		{
		Usage();
		Die("No commmand option specified (must be one of -mask, -maketab etc.)");
		}

	ProgressExit();
	return 0;
	}
