#include "utils.h"
#include "getopts.h"
#include "gff2.h"
#include "ivtree.h"
#include "lohi.h"
#include "progress.h"

void XGFF()
	{
	const char *InFileName = RequiredValueOpt("xgff");
	const char *XFileName = RequiredValueOpt("gff_ns");
	const char *OutFileName = RequiredValueOpt("out");

	FILE *fX = OpenStdioFile(XFileName);
	IvTree<LoHi> Ix;
	ProgressStartFile(fX, "Reading Ns records");
	coord_t TotalExcizedLength = 0;
	unsigned Count = 0;
	GFFRecord2 Rec;
	gpos_t From = 0;
	Log("     From           To     Excised\n");
	Log("==========  ==========  ==========\n");
	while (GetNextGFFRecord(fX, Rec))
		{
		if (Count++%100 == 0)
			ProgressStepFile();

		asserta(Rec.Start > 0);
		gpos_t To = Rec.Start - 1;

		LoHi LH;
		LH.Lo = From;
		LH.Hi = To;
		LH.Index = (unsigned) TotalExcizedLength;
		Ix.Add(LH);

		From = Rec.End + 1;

		Log("%10"PF"  %10"PF"  %10u\n", LH.Lo, LH.Hi, LH.Index);

		TotalExcizedLength += Rec.GetLength();
		}
	ProgressDone();
	CloseStdioFile(fX);

	coord_t EndNs = From;

	FILE *fIn = OpenStdioFile(InFileName);
	FILE *fOut = CreateStdioFile(OutFileName);
	ProgressStartFile(fIn, "Processing");
	Count = 0;
	while (GetNextGFFRecord(fIn, Rec))
		{
		if (Count++%100 == 0)
			ProgressStepFile();

		if (Rec.Start > EndNs)
			{
			asserta(Rec.Start >= TotalExcizedLength);
			asserta(Rec.End >= TotalExcizedLength);

			Rec.Start -= TotalExcizedLength;
			Rec.End -= TotalExcizedLength;
			}
		else
			{
			pair<coord_t, coord_t> Iv(Rec.Start, Rec.End);
			vector<LoHi> Results;
			Ix.GetOverlaps(Iv, Results);
			if (Results.size() != 1)
				{
				Log("\n");
				Log("%u results:\n", SIZE(Results));
				for (vector<LoHi>::const_iterator p = Results.begin();
				  p != Results.end(); ++p)
					{
					const LoHi &LH = *p;
					LH.LogMe(true);
					}
				Rec.LogMe(true);
				Warning("%u overlaps for input record", SIZE(Results));
				continue;
				}
			const LoHi &LH = Results.front();
			coord_t ExcisedLength = LH.Index;
			asserta(Rec.Start >= ExcisedLength);
			asserta(Rec.End >= ExcisedLength);
			GFFAppendIntAttr(Rec.Attrs, "excized", (unsigned) ExcisedLength);
			Rec.Start -= ExcisedLength;
			Rec.End -= ExcisedLength;
			}

		WriteGFFRec(fOut, Rec);
		}
	ProgressDone();
	Progress("%u records processed", Count);
	}
