#include "revfile.h"
#include "getopts.h"
#include "utils.h"
#include "progress.h"
#include "gff.h"
#include "six.h"

void MakeSubAln(const AlnInfoX &AIX, unsigned GenomeIndex, unsigned ChrId, coord_t lo,
  coord_t hi, const StdStrVec &Rows, AlnInfoX &SubAIX, StdStrVec &SubRows);

void XGFFAlns()
	{
	const char *InFileName = RequiredValueOpt("xgffalns");
	const char *OutFileName = RequiredValueOpt("out");
	const char *GFFFileName = RequiredValueOpt("gff");
	const char *GenomeName = RequiredValueOpt("genome");

	FILE *fGFF = OpenStdioFile(GFFFileName);

	RevFile In;
	RevFile Out;

	In.Open(InFileName);
	const std::vector<GenInfo> &Genomes = In.GetGenomes();
	unsigned GenomeIndex = In.GetGenomeIndex(GenomeName);

	Out.Create(OutFileName, Genomes);

	SixVecVec Indexes;
	MakeSixes(In, Indexes);

	GFFRecord Rec;
	std::string CurrentChrName("?");
	unsigned ChrId = UINT_MAX;
	unsigned ChrIndex = UINT_MAX;
	off64_t FileSize = GetStdioFileSize(fGFF);
	unsigned RecordCount = 0;
	ProgressStart("Processing");
	while (GetNextGFFRecord(fGFF, Rec))
		{
		if (RecordCount%100 == 0)
			{
			off64_t FilePos = GetStdioFilePos(fGFF);
			double Fract = (double) FilePos / (double) FileSize;
			unsigned Mills = (unsigned) (Fract*10000);
			ProgressStep(Mills, 10000);
			}
		++RecordCount;
		asserta(Rec.Start > 0);
		asserta(Rec.End >= Rec.Start);

	// Optimization, only re-set chrid & index when name changes
		std::string ChrName(Rec.SeqName);
		if (ChrName != CurrentChrName)
			{
			ChrId = In.GetChrId(GenomeIndex, ChrName);
			ChrIndex = In.GetChrIndex(GenomeIndex, ChrId);
			}

		const Six &Index = Indexes[GenomeIndex][ChrIndex];
		coord_t lo = Rec.Start - 1;
		coord_t hi = Rec.End - 1;

		GCSegXVec Hits;
		SixGetOverlaps(Index, Interval(lo, hi), Hits);

		const unsigned HitCount = SIZE(Hits);
		for (unsigned i = 0; i < HitCount; ++i)
			{
			const GCSegX &Hit = Hits[i];
			unsigned AIIndex = Hit.AIIndex;
			
			AlnInfoX AIX;
			In.GetAlnInfo(AIIndex, AIX.AI);
			AIX.AlnIndex = In.GetAlnIndex(AIIndex);
			AIX.AlnInfoIndex = AIIndex;

			StdStrVec Rows;
			In.GetAlnRows(AIX.AlnIndex, Rows);

			AlnInfoX SubAIX;
			StdStrVec SubRows;
			MakeSubAln(AIX, GenomeIndex, ChrId, lo, hi, Rows, SubAIX, SubRows);

			Out.AddAln(SubAIX.AI, SubRows);
			}
		}
	ProgressDone();

	CloseStdioFile(fGFF);
	In.Close();
	Out.Close();
	}
