#include "revfile.h"
#include "getopts.h"
#include "utils.h"
#include "progress.h"
#include "gff.h"
#include <set>

void WriteFasta(FILE *f, const char *SeqData, unsigned Length);
void RevComp(char *s, unsigned L);

void XSubSeq()
	{
	const char *InFileName = RequiredValueOpt("xsubseq");
	const char *OutFileName = RequiredValueOpt("out");
	const char *GenomeName = RequiredValueOpt("genome");
	const char *ChrName = RequiredValueOpt("chr");
	coord_t lo, hi;
	bool Minus = FlagOpt("revcomp");
	const char *Label = ValueOpt("label");

	RevFile In;
	In.Open(InFileName);
	//const std::vector<GenInfo> &Genomes = In.GetGenomes();
	unsigned GenomeIndex = In.GetGenomeIndex(GenomeName);
	unsigned ChrId = In.GetChrId(GenomeIndex, ChrName);
	if (!(IntOpt("lo", &lo)))
		lo = 0;
	if (!(IntOpt("hi", &hi)))
	{
		coord_t len;
		if (IntOpt("length", &len))
			hi = lo + len - 1;
		else
			hi = In.GetChrLength(GenomeIndex, ChrId) - 1;
	}

	if (lo > hi)
		Die("lo > hi");
	coord_t Length = hi - lo + 1;
	char *Seq = new char[Length];
	In.GetSubSeq(GenomeIndex, ChrId, Seq, lo, Length);

	if (Minus)
		RevComp(Seq, Length);

	FILE *fOut = CreateStdioFile(OutFileName);
	if (Label == 0)
		fprintf(fOut, ">%s[%"COORD_T_PRINTF"-%"COORD_T_PRINTF"]%c\n",
		  ChrName, lo, hi, pom(!Minus));
	else
		fprintf(fOut, ">%s\n", Label);
	WriteFasta(fOut, Seq, Length); 

	delete[] Seq;
	CloseStdioFile(fOut);
	In.Close();
	}
